/***************************************************************************
 *                                                                         *
 *  Copyright(C) 2010-2011 LSI Corporation.  		                   *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  lsisas3108_i2c.h
 *          Title:  I2C specific implementations for LSISAS3108
 *     Programmer:  Naresh Madhusudana
 *  Creation Date:  Aug 16, 2011
 *
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  16/08/11  NM   Initial version.
 *
 * Description
 * ------------
 *
 *    File containing the I2C definitions for LSISAS3108 board
 *
 *--------------------------------------------------------------------------
 */


#ifndef _I2C_H_
#define _I2C_H_

#define PBAM_I2C_BASE_ADDR 	0xC2100000

#define SPD_I2C_REG_BASE_ADDR	PBAM_I2C_BASE_ADDR

/* Register Offsets */
#define IICGLBCTRL_OFFSET		0x00
#define IICINTSTS_OFFSET		0x04
#define IICINTEN_OFFSET			0x08
#define IICWTCTRL_OFFSET		0x0c
#define IICIBMLTOUT_OFFSET		0x10
#define IICIBMLLMEXT_OFFSET		0x14
#define IICIBMLLSEXT_OFFSET		0x18
#define IICTCLKDIV_OFFSET		0x1c
#define IICMONITOR_OFFSET		0x20
#define IICSOFTRST_OFFSET		0x24
/* Master Register Offsets */
#define IICMCMD_OFFSET			0x28
#define IICMRXLEN_OFFSET		0x2c
#define IICMTXLEN_OFFSET		0x30
#define IICMADDR1_OFFSET		0x34
#define IICMADDR2_OFFSET		0x38
#define IICMDATA_OFFSET			0x3c
#define IICMTXFIFOS_OFFSET		0x40
#define IICMRXFIFOS_OFFSET		0x44
#define IICMINTEN_OFFSET		0x48
#define IICMINTS_OFFSET			0x4c
#define IICMTXTRANSFER_OFFSET		0x50
#define IICMRXTRANSFER_OFFSET		0x54

/* Slave Register Offsets */
#define IICSDECCTRL_OFFSET		0x58
#define IICSADDR1_OFFSET		0x5c
#define IICSADDR2_OFFSET		0x60
#define IICSRXCTRL_OFFSET		0x64
#define IICSDATA_OFFSET			0x68
#define IICSRECFIFOS_OFFSET		0x6c
#define IICSINTEN_OFFSET		0x70
#define IICSINTS_OFFSET			0x74
#define IICSRDDUMMY_OFFSET		0x78

#define IICSCLHIGH_OFFSET		0x80
#define IICSCLLOW_OFFSET		0x84
#define IICSPFILTLEN_OFFSET		0x88
#define IICSDASETUPT_OFFSET		0x8c
#define IICSDAHOLDT_OFFSET		0x90


/* Interrupt Status Register Flags */
#define	IIC_INTSTS_SI_CLR		0x02
#define	IIC_INTSTS_MI_CLR		0x01

/* Master Command Register Flags*/
#define IIC_MCMD_FLUSH_TX_FIFO	0x07
#define IIC_MCMD_AUTO_MODE		0x01
#define IIC_MCMD_MANUAL_MODE	0x00
#define IIC_MCMD_SEQ_MODE		0x02
#define IIC_MCMD_MSTR_STOP		0x03
#define IIC_MCMD_SEND_CMD		0x08

/* Soft Reset Register Flags */
#define IIC_SOFTRST_SRST		0x01

/* Global Control Register Flags*/
#define IIC_GLBCTRL_IE 			0x04		//IBML timer enable
#define IIC_GLBCTRL_SE 			0x02		//slave enable
#define IIC_GLBCTRL_ME 			0x01		//master enable

/* Interrupt Enable Register Flags*/
#define IIC_INTEN_SIE			0x02
#define IIC_INTEN_MIE			0x01

/* Master Interrupt Enable Flags */
#define IIC_MINTEN_ALL			0xFFF0

/* Slave Interrupt Enable Flags */
#define IIC_SINTEN_ALL			0xFDF7

#define	IIC_MADDR1_RD			0x0001
#define	IIC_MADDR1_WR			0xFFFE

/* Master Interrupt Status Register Flags */
#define	IIC_MINTS_TS			0x0008      /* Transfer Stop */
#define	IIC_MINTS_TR_FAILED		0x00F0
#define	IIC_MINTS_SS			0x0400
#define	IIC_MINTS_TOUT			0x0080
#define	IIC_MINTS_AL			0x0040
#define	IIC_MINTS_NAK_DATA		0x0020
#define	IIC_MINTS_NAK_ADDR		0x0010
#define	IIC_MINTS_ICT   		0x0000
#define	IIC_MRXFIFOS_FD			0x000F		/* FIFO Depth */


/* *Timing and other values */

#define IIC_SCLHIGH_TIME		0x03E7		/* 5 usec */
#define IIC_SCLLOW_TIME			0x03E7		/* 5 usec */
#define IIC_SDASETUPT_TIME		0x0104		/* SDA setup = 1.3 us with margin */
#define IIC_SDAHOLDT_TIME		0x008C		/* SDA hold = 700 ns with margin */
#define IIC_SPFILT_LENGTH		0x0007		/* spikes filter width 35ns */

#define	IIC_TCLKDIV_DS			0x0008
#define	ICC_TIMER_TE			0x8000		/* Enable wait timer and IBML timer */
#define	IIC_WTCTRL_TLV			0x3D09		/*Desired time out = 20 ms */
#define	IIC_IBMLTOUT_ITLV		0x4C4B		/*Desired time out = 25 ms */
#define	IIC_IBMLLMEXT_IMTLV		0x1E84		/*Desired time out = 10 ms */
#define	IIC_IBMLLSEXT_ISTLV		0x4C4B		/*Desired time out = 25 ms */


/* Register addresses */

#define IIC_GLBCTRL		(SPD_I2C_REG_BASE_ADDR + IICGLBCTRL_OFFSET)
#define IIC_INTSTS		(SPD_I2C_REG_BASE_ADDR + IICINTSTS_OFFSET)
#define IIC_INTEN		(SPD_I2C_REG_BASE_ADDR + IICINTEN_OFFSET)
#define IIC_WTCTRL		(SPD_I2C_REG_BASE_ADDR + IICWTCTRL_OFFSET)
#define IIC_IBMLTOUT		(SPD_I2C_REG_BASE_ADDR + IICIBMLTOUT_OFFSET)
#define IIC_IBMLLMEXT		(SPD_I2C_REG_BASE_ADDR + IICIBMLLMEXT_OFFSET)
#define IIC_IBMLLSEXT		(SPD_I2C_REG_BASE_ADDR + IICIBMLLSEXT_OFFSET)
#define IIC_TCLKDIV		(SPD_I2C_REG_BASE_ADDR + IICTCLKDIV_OFFSET)
#define IIC_MONITOR		(SPD_I2C_REG_BASE_ADDR + IICMONITOR_OFFSET)
#define IIC_SOFTRST		(SPD_I2C_REG_BASE_ADDR + IICSOFTRST_OFFSET)
/* Master Register */
#define IIC_MCMD		(SPD_I2C_REG_BASE_ADDR + IICMCMD_OFFSET)
#define IIC_MRXLEN		(SPD_I2C_REG_BASE_ADDR + IICMRXLEN_OFFSET)
#define IIC_MTXLEN		(SPD_I2C_REG_BASE_ADDR + IICMTXLEN_OFFSET)
#define IIC_MADDR1		(SPD_I2C_REG_BASE_ADDR + IICMADDR1_OFFSET)
#define IIC_MADDR2		(SPD_I2C_REG_BASE_ADDR + IICMADDR2_OFFSET)
#define IIC_MDATA		(SPD_I2C_REG_BASE_ADDR + IICMDATA_OFFSET)
#define IIC_MTXFIFOS		(SPD_I2C_REG_BASE_ADDR + IICMTXFIFOS_OFFSET)
#define IIC_MRXFIFOS		(SPD_I2C_REG_BASE_ADDR + IICMRXFIFOS_OFFSET)
#define IIC_MINTEN		(SPD_I2C_REG_BASE_ADDR + IICMINTEN_OFFSET)
#define IIC_MINTS		(SPD_I2C_REG_BASE_ADDR + IICMINTS_OFFSET)
#define IIC_MTXTRANSFER 	(SPD_I2C_REG_BASE_ADDR + IICMTXTRANSFER_OFFSET)
#define IIC_MRXTRANSFER 	(SPD_I2C_REG_BASE_ADDR + IICMRXTRANSFER_OFFSET)
/* Slave Register */
#define IIC_SDECCTRL		(SPD_I2C_REG_BASE_ADDR + IICSDECCTRL_OFFSET)
#define IIC_SADDR1		(SPD_I2C_REG_BASE_ADDR + IICSADDR1_OFFSET)
#define IIC_SADDR2		(SPD_I2C_REG_BASE_ADDR + IICSADDR2_OFFSET)
#define IIC_SRXCTRL		(SPD_I2C_REG_BASE_ADDR + IICSRXCTRL_OFFSET)
#define IIC_SDATA		(SPD_I2C_REG_BASE_ADDR + IICSDATA_OFFSET)
#define IIC_SRECFIFOS		(SPD_I2C_REG_BASE_ADDR + IICSRECFIFOS_OFFSET)
#define IIC_SINTEN		(SPD_I2C_REG_BASE_ADDR + IICSINTEN_OFFSET)
#define IIC_SINTS		(SPD_I2C_REG_BASE_ADDR + IICSINTS_OFFSET)
#define IIC_SRDDUMMY		(SPD_I2C_REG_BASE_ADDR + IICSRDDUMMY_OFFSET)

#define IIC_SCLHIGH		(SPD_I2C_REG_BASE_ADDR + IICSCLHIGH_OFFSET)
#define IIC_SCLLOW		(SPD_I2C_REG_BASE_ADDR + IICSCLLOW_OFFSET)
#define IIC_SPFILTLEN		(SPD_I2C_REG_BASE_ADDR + IICSPFILTLEN_OFFSET)
#define IIC_SDASETUPT		(SPD_I2C_REG_BASE_ADDR + IICSDASETUPT_OFFSET)
#define IIC_SDAHOLDT		(SPD_I2C_REG_BASE_ADDR + IICSDAHOLDT_OFFSET)


#define IIC_OK		0
#define IIC_NOK		1
#define IIC_NOK_LA	2		/* Lost arbitration */
#define IIC_NOK_ICT	3		/* Incomplete transfer */
#define IIC_NOK_XFRA	4		/* Transfer aborted */
#define IIC_NOK_DATA	5		/* No data in buffer */
#define IIC_NOK_TOUT	6		/* Transfer timeout */

#define IIC_TIMEOUT	1		/* 1 second */

#define IIC_MAX_BYTE_XFER	4

#endif
