/*******************************************************************************

NAME            init.S
VERSION         %version: 4 %
UPDATE DATE     %date_modified: Thu Oct 17 11:30:12 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright(C) 2012 LSI Corporation.                                     *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  init.S
 *          Title:  LSISAS3108 board initialization configuration settings
 *     Programmer:  Naresh Madhusudana
 *  Creation Date:  Aug 16, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  16/08/11  NM   Initial version.
 *
 * Description
 * ------------
 *
 *    This file consisits of LSISAS3108 board memory configuration settings
 *
 *-------------------------------------------------------------------------
 */

#include <ppc_asm.tmpl>
#include <config.h>

/* General */
#define TLB_VALID   0x00000800
#define _256M       0x10000000

/* Supported page sizes */

#define SZ_4K       0x00000000
#define SZ_16K      0x00000010
#define SZ_64K      0x00000030
#define SZ_1M       0x00000070
#define SZ_16M      0x000000F0
#define SZ_256M     0x000001F0
#define SZ_1G       0x000003F0

/* Translation address space */
#define TRAN_SPACE  0x00000400 



/* Storage attributes */
#define SA_W        0x00000800      /* Write-through */
#define SA_I        0x00000400      /* Caching inhibited */
#define SA_M        0x00000200      /* Memory coherence */
#define SA_G        0x00000100      /* Guarded */
#define SA_E        0x00000080      /* Endian */
#define SA_I_L1I    0x00020000      /* L1 I cache inhibit */	
#define SA_I_L1D    0x00010000 	    /* L1 D cache inhibit */
/* User-Definable Storage Attribute */
#define SA_U3       0x00001000      /* System-dependent */

/* Access control */
#define AC_X        0x00000024      /* Execute */
#define AC_W        0x00000012      /* Write */
#define AC_R        0x00000009      /* Read */

/* Some handy macros */

#define EPN(e)       ((e) & 0xfffff000)
#define TLB0(epn,sz)    ( (EPN((epn)) | (sz) | TLB_VALID ) )
#define TLB1(rpn,erpn)  ( ((rpn)&0xfffff000) | ((erpn)&0x000003FF) )
#define TLB2(a)         ( (a)&0x00031fbf )

#define tlbtab_start\
    mflr    r1  ;\
    bl 0f       ;

#define tlbtab_end\
    .long 0, 0, 0   ;   \
0:  mflr    r0  ;   \
    mtlr    r1  ;   \
    blr     ;

#define tlbentry(epn,sz,rpn,erpn,attr)\
    .long TLB0(epn,sz),TLB1(rpn,erpn),TLB2(attr)

/**************************************************************************
 * TLB TABLE
 *
 * This table is used by the cpu boot code to setup the initial tlb
 * entries. Rather than make broad assumptions in the cpu source tree,
 * this table lets each board set things up however they like.
 *
 *  Pointer to the table is returned in r1
 *
 *************************************************************************/


    .section .bootpg,"ax"
    .globl tlbtab

tlbtab:
    tlbtab_start
    /* Cached 32MB NOR flash (for program execution). */
    tlbentry( CONFIG_SYS_FLASH_BASE3, SZ_16M, CONFIG_SYS_FLASH_BASE3, 0x60,AC_R|AC_X|AC_W)
    tlbentry( CONFIG_SYS_FLASH_BASE2, SZ_16M, CONFIG_SYS_FLASH_BASE2, 0x60,AC_R|AC_X|AC_W)

    /* Uncached 32MB NOR flash (for programming, CFI query) */
    tlbentry( CONFIG_SYS_FLASH_BASE1, SZ_16M, CONFIG_SYS_FLASH_BASE1, 0x60,AC_R|AC_X|AC_W|SA_I|SA_I_L1I|SA_I_L1D)
    tlbentry( CONFIG_SYS_FLASH_BASE,  SZ_16M, CONFIG_SYS_FLASH_BASE,  0x60,AC_R|AC_X|AC_W|SA_I|SA_I_L1I|SA_I_L1D)

    /* OCM */    
    
    tlbentry( CONFIG_SYS_ISRAM_BASE0, SZ_1M, CONFIG_SYS_ISRAM_BASE0,0,AC_R|AC_W|AC_X|SA_E)
    tlbentry( CONFIG_SYS_ISRAM_BASE1, SZ_1M, CONFIG_SYS_ISRAM_BASE1,0,AC_R|AC_W|AC_X)
    tlbentry( CONFIG_SYS_ISRAM_BASE2, SZ_1M, CONFIG_SYS_ISRAM_BASE2,0,AC_R|AC_W|AC_X)
    tlbentry( CONFIG_SYS_ISRAM_BASE3, SZ_1M, CONFIG_SYS_ISRAM_BASE3,0,AC_R|AC_W|AC_X)
   
    /* PBAM */
    tlbentry( CONFIG_XMFG_MEM_BASE_ADDR, SZ_1M, CONFIG_XMFG_MEM_BASE_ADDR,0x60,AC_R|AC_W|SA_G|SA_I|SA_I_L1I|SA_I_L1D)
    tlbentry( CONFIG_SYS_PERIPHERAL_BASE, SZ_1M,CONFIG_SYS_PERIPHERAL_BASE,0x60,AC_R|AC_W|SA_G|SA_I|SA_I_L1I|SA_I_L1D)

    /* FPGA and NVSRAM */
    tlbentry( CONFIG_SYS_FPGA_BASE,   SZ_1M, CONFIG_SYS_FPGA_BASE,  0x60,AC_R|AC_W|SA_G|SA_I|SA_I_L1I|SA_I_L1D)
    tlbentry( CONFIG_SYS_NVSRAM_BASE, SZ_1M, CONFIG_SYS_NVSRAM_BASE,0x60,AC_R|AC_W|SA_G|SA_I|SA_I_L1I|SA_I_L1D)

    /* PCE POM0 */
    tlbentry( 0xF0000000, SZ_16M, 0xF0000000,0x70, AC_R|AC_X|AC_W|SA_I|SA_I_L1I|SA_I_L1D)
    tlbentry( 0xF1000000, SZ_16M, 0xF1000000,0x70, AC_R|AC_X|AC_W|SA_I|SA_I_L1I|SA_I_L1D)
    tlbentry( 0xF2000000, SZ_16M, 0xF2000000,0x70, AC_R|AC_X|AC_W|SA_I|SA_I_L1I|SA_I_L1D)
    tlbentry( 0xF3000000, SZ_16M, 0xF3000000,0x70, AC_R|AC_X|AC_W|SA_I|SA_I_L1I|SA_I_L1D)

    /* DDR3 memory */

    /* Due to an isse with PPC476, the DDR3 is added to TLB only after completing the initial TLB entries */

   tlbtab_end

/* ***************************************************************************
 *
 *	I O P - F U N C T I O N S - S T A R T
 *
 * **************************************************************************/

#define PPC476_TCR_DIE  (0x04000000)   // Enable decrementer interrupt
#define PPC476_MSR_CE   (0x00020000)   // Critical interrupt enable
#define PPC476_MSR_EE   (0x00008000)   // External interrupt enable
#define PPC476_MSR_ME   (0x00001000)   // Machine check enable
#define IOC_PPC476_INTERRUPTS (PPC476_MSR_ME |PPC476_MSR_EE | PPC476_MSR_CE)

#define PPC476_CACHE_LINE_SIZE        (0x80)         /* 128 bytes */
#define PPC476_CACHE_LINE_SIZE_MASK (0xFFFFFF80) /* Mask is equal to -(PPC476_CACHE_LINE_SIZE) */

/* Stack pointer (sp) is r1 register in case of PPC: 
 *     refer to http://www.ibm.com/developerworks/linux/library/l-powarch/ 
 *     search for stack
 *     Or refer to: http://www.csd.uwo.ca/~mburrel/stuff/ppc-asm.html
 */
#define	sp	r1

#define  lwi2(reg, value)				\
        lis     reg, (value)@h;


#define lwi(reg, value)				\
	lis     reg, (value)@h;			\
       ori     reg, reg, (value)@l;

/*
 * iopFlushCache(AddressBase, Bytes)
 * Description:
 *     This function will set the specified memory regions to zero
 *     This should be used only for large chunk of memory
 *     Bytes should be multiples of cache line
 *     Address should be aligned to cache line size
 * Arguments
 *     r3 - AddressBase
 *     r4 - Bytes
 * Other used registers
 *     r5 - address
 */
        .globl iopMemSetZero
iopMemSetZero:
	mr	r5,r3
	srwi	r4,r4,7 /* previous val = 7*/
	mtctr	r4
memSetZero_nxt:
	dcbz	0,r5
	dcbf	0,r5
	msync
	dcbf	0,r5
	addi	r5,r5,PPC476_CACHE_LINE_SIZE
	bdnz	memSetZero_nxt
	blr

/*
 * iopFlushCache(AddressBase, Bytes)
 * Description:
 *     This function will flush all of the cache lines associated with
 *     given address range.  Start address need not be aligned to a CL boundary.
 *     Bytes need not be a CL multiple.
 * Arguments
 *     r3 - AddressBase
 *     r4 - Bytes
 * Other used registers
 *     r5 - CLS address mask
 *     r6 - Aligned end address
 */
        .globl iopFlushCache
iopFlushCache:
        subi    sp,sp,8                        #
        stw     r5,0x0(sp)                      # Save off r5
        stw     r6,0x4(sp)                      # Save off r6
        lwi      (r5,PPC476_CACHE_LINE_SIZE_MASK)    # Load Cache Line Address Mask
        add     r4,r4,r3                        # Calculate unaligned end address
        and     r3,r3,r5                        # Align AddressBase
        and     r6,r4,r5                        # Calculate Aligned end address
        cmp     0,r6,r4                           #
        bnl     iopEndAddress               # If aligned address is less than the end address:
        addi    r6,r6,PPC476_CACHE_LINE_SIZE    # Then flush an additional cache line
iopEndAddress:
        msync                                   # Ensure that previous processor data activity completes
iopFlushNextLine2:
        dcbst   0,r3                            # Flush the next dcache line
	msync
        dcbst   0,r3                            # Flush the cache line from L3 also
        addi    r3,r3,PPC476_CACHE_LINE_SIZE    # Go to next cache line
        cmp     0,r3,r6                           # If this cache line is less than aligned, end address:
        blt     iopFlushNextLine2               # Dump another cache line
        lwz     r5,0x0(sp)                      # Otherwise, restore r5,r6 and sp
        lwz     r6,0x4(sp)
        addi    sp,sp,8
        msync                                   # ensure that previous processor data activity completes
        blr

/*
 * iopCacheInvalidate(AddressBase, Bytes)
 * Description:
 *     This function will invalidate all of the cache lines associated with
 *     given address range.  Start address need not be aligned to a CL boundary.
 *     Bytes need not be a CL multiple. This can handle the DDr3 memory which is
 *     placed behind L3 Cache
 * Arguments
 *     r3 - AddressBase
 *     r4 - Bytes
 *     r5 - CLS address mask
 *     r6 - Aligned end address
 */
        .globl iopCacheInvalidate
iopCacheInvalidate:
        subi    sp,sp,8                        #
        stw     r5,0x0(sp)                      # Save off r5
        stw     r6,0x4(sp)                      # Save off r6
        lwi     (r5,PPC476_CACHE_LINE_SIZE_MASK)    # Load Cache Line Address Mask
        add     r4,r4,r3                        # Calculate unaligned end address
        and     r3,r3,r5                        # Align AddressBase
        and     r6,r4,r5                        # Calculate Aligned end address
        cmp     0,r6,r4                           #
        bnl     iopInvEndAddress                # If aligned address is less than the end address:
        addi    r6,r6,PPC476_CACHE_LINE_SIZE    # Then flush an additional cache line
iopInvEndAddress:
        msync                                   # Ensure that previous processor data activity completes
iopInvNextLine2:
        dcbi    0,r3                            # Invalidate the next dcache line
	msync
	dcbi	0,r3				# This is to invalidate from L3 if it is there
        addi    r3,r3,PPC476_CACHE_LINE_SIZE    # Go to next cache line
        cmp    0,r3,r6                           # If this cache line is less than aligned, end address:
        blt     iopInvNextLine2               # Dump another cache line
        lwz     r5,0x0(sp)                      # Otherwise, restore r5,r6 and sp
        lwz     r6,0x4(sp)
        addi    sp,sp,8
        msync                                   # ensure that previous processor data activity completes
        blr

/*
 * U32 iopDisableIRQ(void)
 * This subroutine may be called from C or assembly to disable the
 * PPC Decrmenter, Critical and External interrupts
 *
 * NOTE: R3-R6 are all volatile so it is OK for this routine to destroy them
 *
 * INPUT: void
 * OUTPUT:
 * Irq's that were cleared by this routine
 */
        .global iopDisableIRQ
iopDisableIRQ:
        /* Disable all PPC regular interrupts */
        mfmsr   r3
        mr      r5,r3
        mr      r6,r3
        lwi     (r4, IOC_PPC476_INTERRUPTS)
        and     r6,r6,r4                        /* keep current state to return */
        andc    r5, r5, r4
        mtmsr   r5                              /* Turn off 440 irqs */
        /* Also disable PPC decrementer interrupt */
        mftcr   r3
        lwi2    (r4,(PPC476_TCR_DIE))
        and     r5,r3,r4                        /* Retrieve DIE state */
        or      r5,r6,r5                        /* Or results */
        andc    r3,r3,r4
        mttcr   r3                              /* turn off DIE irq */
        mr      r3,r5                           /* Set up return register r3 */
        blr

/*
 * void iopEnableIRQ(U32 IrqMask)
 * This subroutine may be called from C or assembly to re-enable the
 * PPC Decrmenter, Critical and External interrupts
 *
 * NOTE: R3-R5 are all volatile so it is OK for this routine to destroy them
 *
 * INPUT: IrqMask (r3) - This set of DIE and 476 IRQ bits to enable
 * OUTPUT: void
 * Returns: void
 */
        .global iopEnableIRQ
iopEnableIRQ:
        /* Also disable PPC decrementer interrupt */
        lwi2    (r4,(PPC476_TCR_DIE))
        and.    r5,r3,r4
        beq     EnableOtherIrqs /* DIE Bit not set, branch */
        mftcr   r5
        or      r5,r5,r4
        mttcr   r5              /* Set DIE bit */
EnableOtherIrqs:
        /* Re-enable all the interrupts indicated by the IrqMask (r3) */
        mfmsr   r5
        lwi      (r4, IOC_PPC476_INTERRUPTS)
        and.    r3,r3,r4           /* Mask off bits not supported from IrqMask*/
        beq     DoneWithEnableIrqs /* Branch if no irq's to enable */
        or      r5, r5, r3
        mtmsr   r5
DoneWithEnableIrqs:
        blr
