/*******************************************************************************

NAME            lsisas3108_config.c
VERSION         %version: 16 %
UPDATE DATE     %date_modified: Tue Feb 18 09:54:55 2014 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013-2014 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright(C) 2010-2011 LSI Corporation.  		                   *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  lsisas3108_config.c
 *          Title:  LSISAS3108 board specific file
 *     Programmer:  Naresh Madhusudana
 *  Creation Date:  Aug 15. 2011
 *
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  15/08/11  NM   Initial Version
 *
 * Description
 * ------------
 *
 *    This file consisits of LSISAS3108 board specific initialization
 *    functions.
 *
 *-------------------------------------------------------------------------
 */

#include "include/lsisas3108_common.h"
#include "common.h"
#include <asm/io.h>
#include "boardDisplayCode.h"
#include "include/vxBootString.h"

#include <usb.h>

#undef DEBUG_PRINT_DISP_CODES
#undef VERBOSE_USB_BOOT
/* Enable this so we can poke around just in case future issues arise */
#define DEBUG_USB_BOOT_FAILURES

DECLARE_GLOBAL_DATA_PTR;

#define DCR_CS0_ENABLE_REG 0xB027
#define DCR_CS1_ENABLE_REG 0xB028


#define PBAM_REGS_MAP_BASE    0xC20C2000
#define FLASH_CTR_REG         0x200
#define CS1_CTR_REG           0x220
#define CS2_CTR_REG           0x240

/* Buffer locations for files */
#define OS_ADDR               0x10000000
#define RAMDISK_ADDR          0x10300000
#define FDT_ADDR              0x11C00000

extern int parse_file_outer(void);

/*
 * Name:board_early_init_r(void)
 *
 * Description: Do some platform init. 
 *
 * Arguments : None
 *
 * Return value : None
 */
int board_early_init_r(void)
{
    int rdata = 0;
    
    printf("NetApp Uboot version %d.%d\n", UBOOT_BIN_VERSION, UBOOT_LIB_BIN_VERSION);
    printf("Platform dependent init!!!\n");

    /* Enable NVSRAM chip select (CS1) */
    mtdcr(DCR_CS1_ENABLE_REG, 0x00807f01);  /* Enable NVRAM to 60f6000000-60f7ffffff */

    /* Enable CS1 (NVSRAM) writable */
    rdata = readl(PBAM_REGS_MAP_BASE + CS2_CTR_REG);
    writel(rdata | 0x10000, PBAM_REGS_MAP_BASE + CS2_CTR_REG);

    /* pull-high NVSRAM HSB */
    rdata = mfdcr(0xE00D);
    mtdcr(0xE00D, rdata|0x20000);

    return (0);
}

/*
 * Name: board_early_init_f(void)
 *
 * Description: This fucntion is used enable the interrupts in MPIC as required by the 
 * 		application in linux
 *
 * Arguments : None
 *
 * Return value : None
 */
int board_early_init_f (void)
{
    int rdata = 0; 

    /* Enable FPGA chip-select and make writeable so we can post 7-seg display codes. */
    mtdcr(DCR_CS0_ENABLE_REG, 0x00007f01);  /* Enable FPGA to 60f4000000-60f5ffffff */
    rdata = readl(PBAM_REGS_MAP_BASE + CS1_CTR_REG);
    writel(rdata | 0x10000, PBAM_REGS_MAP_BASE + CS1_CTR_REG);

    /*--------------------------------------------------------------------
     * Setup the interrupt controller polarities, triggers, etc.
     *-------------------------------------------------------------------*/
    return (0);
}

/*
 * Name: clear_all_exceptions(void)
 *
 * Description: This fucntion is used to clear all the exceptions
 *
 * Arguments : None
 *
 * Return value : None
 */
int clear_all_exceptions(void)
{
    /* DDR3 API clears errors that are observed.. so now clear the other errors*/

    /* Clear any OCM errors */
    mtdcr(0xC01C,0xFFFFFFFF);


    /* Clear all the errors observed in L2 till now*/
    mtdcr(0x2000,0x120);
    mtdcr(0x2004,0xFFFFFFFF);

    mtdcr(0x2000,0x150);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x140);
    mtdcr(0x2004,0xFFFFFFFF);
	
    mtdcr(0x2000,0x300);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x304);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x340);
    mtdcr(0x2004,0xFFFFFFFF);

    mtdcr(0x2000,0x500);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x504);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x508);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x540);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x544);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x548);
    mtdcr(0x2004,0xFFFFFFFF);

    mtdcr(0x2000,0x700);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x740);
    mtdcr(0x2004,0xFFFFFFFF);

    mtdcr(0x2000,0x800);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0x840);
    mtdcr(0x2004,0xFFFFFFFF);

    mtdcr(0x2000,0xd00);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0xd04);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0xd08);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0xd40);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0xd44);
    mtdcr(0x2004,0xFFFFFFFF);
    
    mtdcr(0x2000,0xd48);
    mtdcr(0x2004,0xFFFFFFFF);

    mtspr(SPRN_ESR,0xFFFFFFFF);
    mtspr(SPRN_MCSR_CLR,0xFFFFFFFF);
    
    return (0);
}

/*
 * Name: invader_usb_reset(void)
 *
 * Description: Reset the USB cores in Invader
 *
 * Arguments : None
 *
 * Return value : None
 */
void invader_usb_reset(void)
{
    mtdcr(0xE00D,0xC0000000);
    udelay(10);
    mtdcr(0xE00D,0x00000000);
    udelay(10);
}

/*
 * Name:enable_l3_cache(void)
 *
 * Description: Enable L3 Cache for CPU memory using the Mem0 and Mem1 
 *     cached L3 regions, and the Mem3 uncached L3 region.  The Mem2 
 *     uncached L3 region is used to provide access to all non-CPU 
 *     memory, which is configured after DIMM initialization.
 *
 * Arguments : None
 *
 * Return value : None
 */
int enable_l3_cache(void)
{
    unsigned long dimmSizeMB = *((unsigned long*)CONFIG_SYS_ISRAM_BASE0);

    /* Setup L3 for an 8G system */
    if (dimmSizeMB == 0x2000)
    {
        /* Disable the L3 Uncached Region to CPU memory */
        mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
        mtdcr(L3_DATA_REG,L3_MEM_REGION_INVALID);

        /* Enable the 512 L3 Cached Region to CPU memory at 0_0000_0000 */
        mtdcr(L3_INDEX_REG,L3_MEM0_REGION_REG);
        mtdcr(L3_DATA_REG,(L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB >> 6) + L3_MEM_REGION_SIZE_512MB);

        /* Enable the 256 L3 Cached Region to CPU memory at 0_2000_0000 */
        mtdcr(L3_INDEX_REG,L3_MEM1_REGION_REG);
        mtdcr(L3_DATA_REG,((L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB + (512*1024*1024)) >> 6) + L3_MEM_REGION_SIZE_256MB);

        /* Enable the 256 L3 Unached Region to CPU memory at 0_3000_0000 */
        mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
        mtdcr(L3_DATA_REG,((L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB + (768*1024*1024)) >> 6) + L3_MEM_REGION_SIZE_256MB);

        asm volatile ("msync");
        return (0);
    }

    /* Setup L3 for an 4G system */
    if (dimmSizeMB == 0x1000)
    {
        /* Disable the L3 Uncached Region to CPU memory */
        mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
        mtdcr(L3_DATA_REG,L3_MEM_REGION_INVALID);

        /* Enable the 512 L3 Cached Region to CPU memory at 0_0000_0000 */
        mtdcr(L3_INDEX_REG,L3_MEM0_REGION_REG);
        mtdcr(L3_DATA_REG,(L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB >> 6) + L3_MEM_REGION_SIZE_512MB);

        /* Enable the 512 L3 Unached Region to CPU memory at 0_2000_0000 */
        mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
        mtdcr(L3_DATA_REG,((L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB + (512*1024*1024)) >> 6) + L3_MEM_REGION_SIZE_512MB);

        asm volatile ("msync");
        return (0);
    }

    /* Setup L3 for a 2G system */
    /* Disable the L3 Uncached Region to CPU memory */
    mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
    mtdcr(L3_DATA_REG,L3_MEM_REGION_INVALID);

    /* Enable the 256 L3 Cached Region to CPU memory at 0_0000_0000 */
    mtdcr(L3_INDEX_REG,L3_MEM0_REGION_REG);
    mtdcr(L3_DATA_REG,(L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB >> 6) + L3_MEM_REGION_SIZE_256MB);

    /* Enable the 128 L3 Cached Region to CPU memory at 0_1000_0000 */
    mtdcr(L3_INDEX_REG,L3_MEM1_REGION_REG);
    mtdcr(L3_DATA_REG,((L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB + (256*1024*1024)) >> 6) + L3_MEM_REGION_SIZE_128MB);

    /* Enable the 128 L3 Unached Region to CPU memory at 0_1800_0000 */
    mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
    mtdcr(L3_DATA_REG,((L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB + (384*1024*1024)) >> 6) + L3_MEM_REGION_SIZE_128MB);

    asm volatile ("msync");
    return (0);
}

/*
 * Name: checkboard(void)
 *
 * Description: This fucntion is used to print the board properties during the start of day
 * 		on to the UART console
 *
 * Arguments : None
 *
 * Return value : 0
 */
int checkboard (void)
{
    char *s = getenv ("serial#");

    printf ("Board: LSISAS3108 - PPC476fp Evaluation Board");
    if (s != NULL) {
        puts (", serial# ");
        puts (s);
    }
    putc ('\n');

    return (0);
}


/*
 * Name: boardDisplayCode(int value)
 *
 * Description: This function is used to post something to the 7-segment display.
 *
 * Arguments : value - Value to write to display regs.
 *
 * Return value : none
 */
void boardDisplayCode (int value)
{
    unsigned char msd_data, lsd_data;

    lsd_data = (unsigned char) (value & 0xFF) | DISPLAY_DP;
    msd_data = (unsigned char) ((value>>8) & 0xFF);

    writeb(lsd_data, CONFIG_SYS_FPGA_BASE + 0x02);    /* SSD1 */
    writeb(msd_data, CONFIG_SYS_FPGA_BASE + 0x03);    /* SSD2 */

#ifdef DEBUG_PRINT_DISP_CODES
    printf("Display Code: %02x %02x\n", msd_data, (lsd_data & ~DISPLAY_DP));
#endif
}

/*
 * Name: enableResets(void)
 *
 * Description: Enable resets in FPGA. A similar function exists in the
 * ubootLib binary as well to unmask resets in case of any faults.
 *
 * Arguments : none
 *
 * Return value : none
 */
void enableResets (void)
{
    U8 val;

    /* Force everything to sync up in case reset is already asserted */
    asm volatile ("msync");

    val = readb(CONFIG_SYS_FPGA_BASE + 0x06);

    val |= 0x20;

    writeb(val, CONFIG_SYS_FPGA_BASE + 0x06);

    /* For good measure */
    asm volatile ("msync");
}

/*
 * Name: createBootString(int bootType)
 *
 * Description: This function is used to populate the structure used to
 *     pass information from the bootloader to VxWorks (or another OS).
 *
 * Arguments :  int bootType - custom boot type
 *
 * Return value : none
 */
void createBootString (void)
{   
    unsigned long initDimmSizeMB = *((unsigned long*)CONFIG_SYS_ISRAM_BASE0);
    VX_BOOTSTRING* bootStringPtr = (VX_BOOTSTRING*) CONFIG_SYS_ISRAM_BASE0;
 
    /* Repopulate the DIMM size, even though it may be at the same location,
     * to isolate it from potential changes to the struct or to how the 
     * DIMM size is determined and stored prior to this point.
     */
    bootStringPtr->DimmSizeMB = initDimmSizeMB;
    bootStringPtr->bootStrLen = sizeof(VX_BOOTSTRING);
    bootStringPtr->uboot_ver = UBOOT_BIN_VERSION;
    bootStringPtr->ubootLib_ver = UBOOT_LIB_BIN_VERSION;

    /* Default value, should be updated later */
    bootStringPtr->bootType = UBOOT_UNKNOWN;
}

/*
 * Name: updateBootType(int bootType)
 *
 * Description: This function is used to update the structure used to
 *     pass information from the bootloader to VxWorks (or another OS).
 *
 * Arguments :  int bootType - custom boot type
 *
 * Return value : none
 */
void updateBootType (int bootType)
{
    VX_BOOTSTRING* bootStringPtr = (VX_BOOTSTRING*) CONFIG_SYS_ISRAM_BASE0;

    bootStringPtr->bootType = bootType;
}


/*
 * Name: custom_hang
 *
 * Description: Custom silent hang routine. The normal hang() prints some junk
 * that we may or may not want printed. This gives us a nice way to custom
 * tailor our hang behavior. Most of the time, output will be disabled, but
 * we have enabled the output for the 'fatload' portion of cusboot() and if
 * that fails then we end up with hang() output where we'd normally not have
 * any. Some debug code could be added to let us break in and poke around if
 * we wanted. Some disabled prototype code is included below.
 *
 * Return value : void (although it will normally never return)
 */
void custom_hang(void)
{
    for (;;)
    {
#ifdef DEBUG_USB_BOOT_FAILURES
        /* Look for a sequence of ^X ^Y ^X */
        if ( getc() != 24 ) /* Ctrl-X */
            continue;
        if ( getc() != 25 ) /* Ctrl-Y */
            continue;
        if ( getc() != 24 ) /* Ctrl-X */
            continue;

        /* If the key sequence is detected, enable the display and break out
         * of the loop. This will cause the default 'bootcmd' to continue and
         * return, which will result in entering the main uboot shell. */
        gd->flags &= ~GD_FLG_SILENT;
        break;
#endif
    }
}


/*
 * Name: check_pushbutton
 *
 * Description: Check to see if push button is currently depressed
 *
 * Return value : 1=depressed, 0=not pressed (default)
 */
int check_pushbutton(void)
{
    U8 val;

    val = readb(CONFIG_SYS_FPGA_BASE + 0x24);

    if (val & 0x02)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/*
 * Name: check_keypress
 *
 * Description: Another way to check for keypress once boot sequence has been
 * initiated. If Ctrl-X has been pressed since the last check and when this
 * function is called it will drop into the uboot shell prompt.
 *
 * Return value : void
 */
void check_keypress(void)
{
    int keyPress;

    /* If key was pressed */
    if (tstc())
    {
        keyPress = getc();

        /* Return if key was not Ctrl-X */
        if ( keyPress != 24 )
        {
            return;
        }

        /* Enable prints and drop to shell. Won't return. */
        gd->flags &= ~GD_FLG_SILENT;
        parse_file_outer();
    }
}

/*
 * Name: run_boot_command
 *
 * Description: Function to set the 7-segment LED and call the boot command
 * for all of the various boot types. Mostly just to keep everything straight
 * and in a single location. Will not return if attempting to boot VxWorks or
 * if Linux boot files appear valid.
 * Found during testing that DFW does not boot properly if the GD_FLG_SILENT
 * flag is set when booting, so clear that when booting DFW. Normally, it will
 * already be cleared if a user breaks in, but not when auto-booting.
 *
 * Return value : void
 */
void run_boot_command(int bootType)
{
    unsigned long saved_flags = gd->flags;
    char cmd_buf[256];

    switch (bootType)
    {
        case UBOOT_DFW_NOR_FLASH:
            /* Clear silent flag or DFW will not boot properly */
            gd->flags &= ~GD_FLG_SILENT;
            /* Using sprintf here (and below) since uboot does not have snprintf */
            sprintf(cmd_buf, "bootm 0xfe000000 0xfe300000 0xffc00000");
            break;
        case UBOOT_DFW_TFTP:
        case UBOOT_DFW_USB:
        case UBOOT_DFW_MSATA:
            /* Clear silent flag or DFW will not boot properly */
            gd->flags &= ~GD_FLG_SILENT;
            sprintf(cmd_buf, "bootm 0x%x 0x%x 0x%x", OS_ADDR, RAMDISK_ADDR, FDT_ADDR);
            break;
        case UBOOT_NOR_FLASH:
            sprintf(cmd_buf, "bootvx 0x%x", CONFIG_SYS_LOAD_ADDR);
            break;
        case UBOOT_NORMAL:
        case UBOOT_EXTERNAL_USB:
        case UBOOT_MSATA_RECOVERY:
        case UBOOT_TFTP:
            sprintf(cmd_buf, "bootvx 0x%x", OS_ADDR);
            break;
        default:
            printf("Invalid bootType\n");
            return;
    }

    updateBootType(bootType);

    if (bootType == UBOOT_NORMAL)
    {
        boardDisplayCode(UBOOT_STARTING_DEFAULT_OS);
    }
    else
    {
        boardDisplayCode(UBOOT_USING_ALT_BOOT_PATH);
    }

    /* If booting VxWorks, this will never return.
     * If booting Linux, it will return if any of the three images (kernel,
     * ramdisk or FDT) are missing/corrupt. This helps with the DFW USB and
     * TFTP commands since it gives the user another chance if they get a
     * filename wrong. */
    run_command(cmd_buf, 0);

    /* Restore flags if we're returning */
    gd->flags = saved_flags;
}

/*
 * Name: usb_get_devnum
 *
 * Description: Get USB storage devnum for a given port number. Needed in case
 * there is an external USB device but no mSATA device.
 *
 * Return value : int (devnum or -1 if no dev on given port)
 */
int usb_get_devnum(int port)
{
    /* Could be done as a loop, but we only expect up to two devs */
    if (usb_retrieve_port(0) == port)
    {
        return 0;
    }
    else if (usb_retrieve_port(1) == port)
    {
        return 1;
    }
    else
    {
        return -1;
    }
}

/*
 * Name: usb_get
 *
 * Description: Get boot file(s) from a USB device.
 *
 * Return value : int (0=ok, non-zero=error)
 */
int usb_get(int bootType, int devNum)
{
    char cmd_buf[256];
    char *bootfile;
    char *ramdisk;
    char *fdt;
    int  retVal;

    /* boot from the USB stick */
    if(bootType == UBOOT_EXTERNAL_USB)
    {
        /* Use "recover/System" by default, but let user override with
         * 'setenv bootfile' if they want. */
        if (!getenv("bootfile"))
        {
            setenv("bootfile","recover/System");
        }
    }
    else if(bootType == UBOOT_DFW_USB || bootType == UBOOT_DFW_MSATA)
    {
        /* user must set 'bootfile' */
    }
    else if (bootType == UBOOT_MSATA_RECOVERY)
    {
        setenv("bootfile","fw/recover/System");
    }
    else
    {
        setenv("bootfile","fw/active/System");
    }

    bootfile = getenv("bootfile");
    if (!bootfile)
    {
        printf("'bootfile' env var not set\n");
        return -1;
    }

    sprintf(cmd_buf, "fatload usb %d 0x%x %s", devNum, OS_ADDR, bootfile);
    retVal = run_command(cmd_buf, 0);

    /* DFW can either be a big, blob image or three individual files. If it
     * is the latter, the user must also specify 'ramdisk' and 'fdt'. */
    if (bootType == UBOOT_DFW_USB || bootType == UBOOT_DFW_MSATA)
    {
        ramdisk = getenv("ramdisk");
        if (ramdisk)
        {
            sprintf(cmd_buf, "fatload usb %d 0x%x %s", devNum, RAMDISK_ADDR, ramdisk);
            retVal |= run_command(cmd_buf, 0);
        }
        fdt = getenv("fdt");
        if (fdt)
        {
            sprintf(cmd_buf, "fatload usb %d 0x%x %s", devNum, FDT_ADDR, fdt);
            retVal |= run_command(cmd_buf, 0);
        }
    }

    return retVal;
}

/*
 * Name: custom_usb_boot
 *
 * Description: Custom USB boot routine for the default boot case.
 * Calls the routines that "usb start" would call, but checks the return value
 * of usb_stor_scan() and refuses to continue until a device is found. This
 * takes care of power-up cases when the device isn't ready for a few seconds
 * and doesn't slow down reboot cases where the device is ready right away.
 * If a device isn't found after 15 attempts (roughly 1 second per attempt),
 * this routine will bail out and the LED codes will be stuck displaying that
 * the boot device was not found. If a device is found, we'll attempt to
 * 'fatload' the System file. If that doesn't work, this routine will bail out
 * but the LED codes will display that the System file wasn't found. If both
 * the device and System file are found we will attempt to boot from the image.
 * That boot attempt will never return, regardless of success.
 *
 * Return value : void
 */
void custom_usb_boot(int bootType)
{
    int attempts = 0;
    int usb_dev;
    int retVal;
    int port;
    unsigned long saved_flags = gd->flags;

    boardDisplayCode(UBOOT_DEFAULT_BOOT_DEVICE_NOT_FOUND);

#ifdef VERBOSE_USB_BOOT
    gd->flags &= ~GD_FLG_SILENT;
#endif

    /* Loop until we find a storage device on desired port. External USB
     * devices tend to show up immediately but internal mSATA device usually
     * takes ~3 attempts. */
    do
    {
        /* See if user has tried to break in */
        check_keypress();

        /* If push button is pressed, change bootType */
        if (check_pushbutton())
        {
            bootType = UBOOT_EXTERNAL_USB;
        }

        /* External devices are on port 1, internal on port 0 */
        if(bootType == UBOOT_EXTERNAL_USB || bootType == UBOOT_DFW_USB)
        {
            port = 1;
        }
        else
        {
            port = 0;
        }

        usb_stop();

        /* If we've reached 10 attempts, odds are that no drive is present.
         * But, if one is and we haven't detected it yet, something is likely
         * wrong and we are unlikely to find it without intervention. Reset
         * the USB cores at the Invader level in a last-ditch attempt to
         * revive things. */
        if (attempts == 10)
        {
            invader_usb_reset();
        }

        if (usb_init() == 0) /* Init == OK */
        {
            usb_stor_scan(1);
        }

        /* See if a device was found on the port */
        usb_dev = usb_get_devnum(port);
        attempts++;
    } while ( (usb_dev == -1) && (attempts < 15) );

    check_keypress();

    /* USB storage device has been found on port 0 or we've timed out while
     * looking for it (return if timed out). */
    if (usb_dev == -1)
    {
        printf("Device not found on port %d\n", port);
        return;
    }

    /* At this point USB device has been found so update LED code to indicate
     * that we're now looking for boot file. */
    boardDisplayCode(UBOOT_NO_BOOT_IMAGE_FOUND);

    /* Now temporarily enable print output so we can see what happens when
     * trying to load the default System image. It's similar to grub displaying
     * what it is doing, although the output is much different. */
    gd->flags &= ~GD_FLG_SILENT;

    retVal = usb_get(bootType, usb_dev);

    /* Restore output settings - note that this will supress the bootvx output
     * below if the display was originally disabled (default behavior). The
     * 'bootvx' output is a bit messy so that isn't necessarily a bad thing. */
    gd->flags = saved_flags;

    check_keypress();

    /* We've found the device, but not the file */
    if (retVal != 0)
    {
        /* Special case for MFG: mSATA drive is present but System file is not.
         * If this happens during the normal boot process, do one final check
         * to see if MFG DFW is present. */
        if (bootType == UBOOT_NORMAL)
        {
            setenv("bootfile","vmlinuz-dfw");
            setenv("ramdisk","ramdisk-dfw.img");
            setenv("fdt","lsisas3108-dfw.dtb");
            if (usb_get(UBOOT_DFW_MSATA, usb_dev) == 0)
            {
                run_boot_command(UBOOT_DFW_MSATA);
            }
        }

        /* Custom boot path, device was found, but no file */
        return;
    }

    run_boot_command(bootType);
}

/*
 * Name: print_tftp_help
 *
 * Description: Print help message for TFTP.
 *
 * Return value : void
 */
void print_tftp_help(void)
{
    printf("TFTP Boot Help\n");
    printf("Set the necessary environment variables and retry.\n");
    printf("---Net Options---\n");
    printf("For Static IP:\n");
    printf("  setenv ipaddr xx.xx.xx.xx     # Controller IP address\n");
    printf("  setenv serverip xx.xx.xx.xx   # TFTP server IP address\n");
    printf("  setenv gatewayip xx.xx.xx.xx  # Controller IP address\n");
    printf("  setenv netmask xx.xx.xx.xx    # Controller IP address\n");
    printf("For DHCP:\n");
    printf("  setenv ipaddr                 # (Optional) Set blank to reset DHCP\n");
    printf("  setenv serverip xx.xx.xx.xx   # (Optional) TFTP server IP if different from DHCP server\n");
    printf("Force NIC:\n");
    printf("  setenv gei [0|1]              # (Optional) Specify GEI interface if both are in-use\n");
    printf("---File Options---\n");
    printf("For Vxworks or DFW single image:\n");
    printf("  setenv bootfile path/to/file\n");
    printf("For individual DFW files:\n");
    printf("  setenv bootfile path/to/vmlinuz\n");
    printf("  setenv ramdisk path/to/ramdisk.img\n");
    printf("  setenv fdt path/to/lsisas3108.dtb\n");
}

/*
 * Name: dhcp_env_save
 *
 * Description: Save 'bootfile' and 'serverip' env vars which DHCP can
 * overwrite. Restore with dhcp_env_restore.
 *
 * Return value : void
 */
void dhcp_env_save(void)
{
    char *bootfile, *serverip;

    bootfile = getenv("bootfile");
    serverip = getenv("serverip");

    if (bootfile)
    {
        setenv("bootsave", bootfile);
    }
    if (serverip)
    {
        setenv("ipsave", serverip);
    }
}

/*
 * Name: dhcp_env_restore
 *
 * Description: Restore 'bootfile' and 'serverip' env vars which were saved
 * by dhcp_env_save.
 *
 * Return value : void
 */
void dhcp_env_restore(void)
{
    char *bootsave, *ipsave;

    bootsave = getenv("bootsave");
    ipsave = getenv("ipsave");

    if (bootsave)
    {
        setenv("bootfile", bootsave);
    }
    if (ipsave)
    {
        setenv("serverip", ipsave);
    }
}

/*
 * Name: tftp_get
 *
 * Description: Stripped down version of the generic network routine. This one
 * is just used to get a specified file via TFTP and copy it to a given address.
 *
 * Return value : int (0=ok, -1=error)
 */
int tftp_get(U32 addr, char *file)
{
    int   size;

    /* Set global var to desired addr */
    load_addr = addr;

    /* Set name of file that TFTP protocol will try to load */
    copy_filename(BootFile, file, sizeof(BootFile));

    size = NetLoop(TFTP);

    /* If size is negative, something went wrong w/the network device.
     * If size is 0, then some sort of TFTP error (most likely a bad filename
     * or IP address). Either way, just report an error. */
    if (size <= 0)
    {
        return -1;
    }

    /* Things appear to have gone ok, return success */
    return 0;
}

/*
 * Name: custom_tftp_boot
 *
 * Description: Try to get files via TFTP and then boot them. Will also set up
 * IP address via DHCP if 'ipaddr' env var is not set. This function should not
 * return if all goes well.
 *
 * Return value : -1=error
 */
int custom_tftp_boot(int bootType)
{
    char *bootfile, *ramdisk, *fdt;
    char *gei;

    /* Allow user to force NIC if both are in use. The uboot naming convention
     * is e1000#x - however, our NIC labeling is opposite from the order the HW
     * is detected: e1000#0 is gei1 and e1000#1 is gei0. Rather than tell the
     * user to specify 'ethact' with the opposite value, create a new env var
     * called gei that maps to the appropriate values. */
    gei = getenv("gei");
    if (gei)
    {
        /* Disable NIC device rotation */
        setenv("ethrotate","no");

        if (gei[0] == '0')
        {
            setenv("ethact","e1000#1");
        }
        else if (gei[0] == '1')
        {
            setenv("ethact","e1000#0");
        }
        else
        {
            printf("Invalid 'gei' env variable\n");
            return -1;
        }
    }

    /* Failure to set this will cause net code to retry indefinitely in some
     * cases where we don't want it to (bad filename, etc). */
    setenv("netretry", "once");

    /* If IP addr isn't set, attempt to get one via DHCP */
    if (getenv("ipaddr") == NULL)
    {
        dhcp_env_save();
        run_command("dhcp", 0);
        dhcp_env_restore();

        if (getenv("ipaddr") == NULL)
        {
            printf("Unable to get IP address via DHCP\n"
                   "  Set static IP: setenv ipaddr xx.xx.xx.xx");
            return -1;
        }
    }

    /* If the wrong IP addr is used (for example, DHCP is used but server IP
     * is different and user forgot to set it), the TFTP command will just
     * hang. Print a message to let the user know they can abort it. */
    printf("*** Press Ctrl+C to abort if TFTP hangs due to bad IP addr ***\n");

    bootfile = getenv("bootfile");
    if (!bootfile)
    {
        printf("'bootfile' env var not set\n");
        return -1;
    }

    if (tftp_get(OS_ADDR, bootfile))
    {
        return -1;
    }

    if (bootType == UBOOT_DFW_TFTP)
    {
        ramdisk = getenv("ramdisk");
        if (ramdisk)
        {
            if(tftp_get(RAMDISK_ADDR, ramdisk))
                return -1;
        }
        fdt = getenv("fdt");
        if (fdt)
        {
            if(tftp_get(FDT_ADDR, fdt))
                return -1;
        }
    }

    /* This command should not return if we boot an OS */
    run_boot_command(bootType);

    /* TFTP failed, return error */
    return -1;
}

/*
 * Name: do_custom_boot
 *
 * Description: Custom board-specific boot routines. This is meant to be called
 * as the default CONFIG_BOOTCOMMAND or via the shell for alternate boot
 * routines.
 * Current usages:
 *   cusboot [0] - Boot file named "System" from mSATA
 *   cusboot 1   - Boot VxWorks image from flash (address CONFIG_SYS_LOAD_ADDR)
 *   cusboot 2   - Boot file named "System" from /recover on USB Stick
 *   cusboot 3   - Boot file named "System" from /fw/recover on mSATA
 *
 * For future usage, we will want to change the default behavior so that we
 * boot off USB but we need USB working in System first.
 *
 * Return value : 0 (although we won't return when bootvx is called)
 */
int do_custom_boot ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    VX_BOOTTYPE bootType = UBOOT_NORMAL; /* Default boot routine */

    /* Get argument to specify alternate boot path */
    if (argc > 1)
    {
        bootType = (int)simple_strtoul(argv[1], NULL, 10);
    }

    printf("\nCustom Boot: %d\n", bootType);

    if (bootType == UBOOT_DFW_NOR_FLASH || bootType == UBOOT_NOR_FLASH)
    {
        run_boot_command(bootType);
    }
    else if ((bootType == UBOOT_NORMAL) || (bootType == UBOOT_EXTERNAL_USB) || \
             (bootType == UBOOT_MSATA_RECOVERY) || (bootType == UBOOT_DFW_USB) || \
             (bootType == UBOOT_DFW_MSATA))
    {

        /* custom_usb_boot() will not return unless something goes wrong */
        custom_usb_boot(bootType);

        /* On normal bootup, hang the system if the previous command returns */
        if (bootType == UBOOT_NORMAL)
        {
            custom_hang();
        }
    }
    else if (bootType == UBOOT_TFTP || bootType == UBOOT_DFW_TFTP)
    {
        boardDisplayCode(UBOOT_USING_ALT_BOOT_PATH);
        custom_tftp_boot(bootType);
        print_tftp_help();
    }
    else 
    {
        printf("\n*** Invalid option ***\n");
        printf("Type 'help cusboot' for a list of valid options \n");
        return 0;
    }

    boardDisplayCode(UBOOT_OS_STARTUP_FAILED);
    return 0;
}


/* Add 'cusboot' command to u-boot */
U_BOOT_CMD(
    cusboot, 2, 1, do_custom_boot,
    "Execute custom boot routine",
    "[type]\n"
    "cusboot [0] - Default mSATA boot path\n"
    "cusboot 1   - Boot recovery System from NOR Flash\n"
    "cusboot 2   - Boot recovery System from USB\n"
    "cusboot 3   - Boot recovery System from mSATA\n"
    "cusboot 4   - Boot recovery System from TFTP\n"
    "cusboot 5   - Boot DFW from NOR Flash\n"
    "cusboot 6   - Boot DFW from TFTP\n"
    "cusboot 7   - Boot DFW from USB\n"
    "cusboot 8   - Boot DFW from mSATA\n"
);

