/*******************************************************************************

NAME            lsisas3108_pci.h
VERSION         %version: 3 %
UPDATE DATE     %date_modified: Thu Oct 17 11:31:53 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  lsisas3108_pci.h
 *          Title:  Definitions for PCI-E root complex sample application
 *     Programmer:  Sandeep Darisala
 *  Creation Date:  March 13, 2012
 *
 * Description
 * ------------
 *
 *    Definitions required by the sample application to initialize & operate
 *    LSI SAS3108 in PCI-E root complex mode.
 *-------------------------------------------------------------------------
 */

#ifndef _LSISAS3108_PCI_H
#define _LSISAS3108_PCI_H

#define MAX_BRIDGES                                 (16)
#define MAX_DEVICES                                 (32)
#define LSISAS3108_PCIE_DEBUG                       (1)

/* PIM window sizes */
#define PIM0_WINDOW_SIZE                            (16 * 1024)
#define PIM1_WINDOW_SIZE                            (2U * 1024 * 1024 * 1024)
#define PIM0_PCIE_BASE_ADDRESS_LOW                  (0x80000000)
#define PIM0_PCIE_BASE_ADDRESS_HIGH                 (0x00)
#define PIM1_PCIE_BASE_ADDRESS_LOW                  (0x00000000)
#define PIM1_PCIE_BASE_ADDRESS_HIGH                 (0x00)

#define PCIE_TYPE0_HDR_MAX_BARS                     (6)

/* PCI-E IO and Memory Sizes and Base Addresses */
#define PCIE_IO_BASE_ADDR_LOW                       (0xF0000000)
#define PCIE_IO_BASE_ADDR_HIGH                      (0x70)
#define PCIE_IO_LIMIT_LOW                           (0xF00FFFFF)
#define PCIE_IO_LIMIT_HIGH                          (0x70)
#define PCIE_NON_PREFETCHABLE_MEMORY_BASE_ADDR_LOW  (0xF0100000)
#define PCIE_NON_PREFETCHABLE_MEMORY_BASE_ADDR_HIGH (0x70)
#define PCIE_NON_PREFETCHABLE_MEMORY_LIMIT          (0xF3FFFFFF)

#define PCIE_EMBEDDED_END_POINT_CONFIG_NUMBER       (7)

/*	Configuration space register offesets*/


typedef struct _PCIE_EMBEDDED_END_POINT_CONFIG_ADDR_DATA
{
    U32    Addr;
    U32    Data;
} PCIE_EMBEDDED_END_POINT_CONFIG_ADDR_DATA, *PTR_PCIE_EMBEDDED_END_POINT_CONFIG_ADDR_DATA;

typedef struct _PCIE_BAR_ATTRIBUTES
{
    U8 Type            :1;     /* 0 = Memory, 1 = IO */
    U8 AddrDecoding    :2;     /* 0 = 32-bit, 2 = 64-bit */
    U8 Prefetchable    :1;     /* 0 = non-prefetchable, 1 = prefetchable */
    U8 Reserved        :4;
} PCIE_BAR_ATTRIBUTES;

typedef struct _PCIE_BAR
{
    PCIE_BAR_ATTRIBUTES Attributes;
    U8             Reserved1;
    U16            Reserved2;
    U32            Size;
    //U32            Address;
    U64            Address;
} PCIE_BAR, *PTR_PCIE_BAR;

typedef struct _PCIE_DEVICE
{
    U8     Bus;
    U8     Device;
    U8     Function;
    U8     HeaderType;
    PCIE_BAR    Bar[PCIE_TYPE0_HDR_MAX_BARS];
    S16     ParentId;
    U8     BridgeNum;
    U32    MsixTableSize;
    U32    MsixTableBaseAddressLow;
    U32    MsixTableBaseAddressHigh;
} PCIE_DEVICE, *PTR_PCIE_DEVICE;

typedef struct _PCIE_BRIDGE
{
    U8         PrimaryBus;
    U8         SecondaryBus;
    U8         SubordinateBus;
    U8         DeviceNum;
    U32        IoBaseAddress;
    U32        IoLimitAddress;
    U32        MemBaseAddress;
    U32        MemLimitAddress;
    U32        PrefetchableBaseLowAddr;
    U32        PrefetchableBaseHighAddr;
    U32        PrefetchableLimitLowAddr;
    U32        PrefetchableLimitHighAddr;
} PCIE_BRIDGE, *PTR_PCIE_BRIDGE;

#endif  /* _LSISAS3108_PCI_H */
