/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  dcDB.h
 *          Title:  Dual core PMU dorbell
 *     Programmer:  Jignesh Moradiya
 *  Creation Date:  May 20,2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who       Description
 *  --------  -----     ----------------------------------------------------
 *  05/20/12  JM        Initial version.
 *
 * Description
 * ------------
 * This file contains structure, macro definitions and function prototypes
 * for the dual core library of doorbell.
 * -------------------------------------------------------------------------
 */

#ifndef DC_DB_H
#define DC_DB_H

#include "basetype.h"

typedef U32 DCSTATUS;

#define DC_PRIM_CORE                    (0)
#define DC_SEC_CORE                     (1)

#define DC_DB_ENABLE_INT_DEFAULT        (0x00007FFF)
#define DC_DB_ASSERTION_DEFAULT         (0x00000001)

#define CURR_CORE                       (0)
#define ALL_CORE                        (1)

/* -------------------------------------------------------------------------
 *                        PMU MECHANISM - DOORBELL
 * -------------------------------------------------------------------------
 */
/* -------------------------------------------------------------------------
 * PMU Address Map
 * Doorbell control register
 * -------------------------------------------------------------------------
 */
#define DC_DB_STS_WIDTH                 (15)
#define DC_RESERVED_WIDTH               (1)
#define DC_TXQ0_NOT_FULL_WIDTH          (1)
#define DC_RXQ0_NOT_EMPTY_WIDTH         (1)
#define DC_TXQ1_NOT_FULL_WIDTH          (1)
#define DC_RXQ1_NOT_EMPTY_WIDTH         (1)
#define DC_TXQ2_NOT_FULL_WIDTH          (1)
#define DC_RXQ2_NOT_EMPTY_WIDTH         (1)
#define DC_TXQ3_NOT_FULL_WIDTH          (1)
#define DC_RXQ3_NOT_EMPTY_WIDTH         (1)
#define DC_QPRIORITY_WIDTH              (4)
#define DC_DBPRIORITY_WIDTH             (4)

#define DC_DB_STS_POS                   (0)       /* Bits 0-14 */
#define DC_RESERVED_POS                 (15)      /* Bit 15 */
#define DC_TXQ0_NOT_FULL_POS            (16)      /* Bit 16 */
#define DC_RXQ0_NOT_EMPTY_POS           (17)      /* Bit 17 */
#define DC_TXQ1_NOT_FULL_POS            (18)      /* Bit 18 */
#define DC_RXQ1_NOT_EMPTY_POS           (19)      /* Bit 19 */
#define DC_TXQ2_NOT_FULL_POS            (20)      /* Bit 20 */
#define DC_RXQ2_NOT_EMPTY_POS           (21)      /* Bit 21 */
#define DC_TXQ3_NOT_FULL_POS            (22)      /* Bit 22 */
#define DC_RXQ3_NOT_EMPTY_POS           (23)      /* Bit 23 */
#define DC_QPRIORITY_POS                (24)      /* Bits 24-28 */
#define DC_DBPRIORITY_POS               (28)      /* Bits 28-30 */

/* -------------------------------------------------------------------------
 * PMU PLB Address MAP
 * Doorbell enable register
 * -------------------------------------------------------------------------
 */
#define DE_EN_DB_INT_WIDTH              (15)
#define DE_RESERVED_WIDTH               (1)
#define DE_EN_TXQ0_NOT_FULL_WIDTH       (1)
#define DE_EN_RXQ0_NOT_EMPTY_WIDTH      (1)
#define DE_EN_TXQ1_NOT_FULL_WIDTH       (1)
#define DE_EN_RXQ1_NOT_EMPTY_WIDTH      (1)
#define DE_EN_TXQ2_NOT_FULL_WIDTH       (1)
#define DE_EN_RXQ2_NOT_EMPTY_WIDTH      (1)
#define DE_EN_TXQ3_NOT_FULL_WIDTH       (1)
#define DE_EN_RXQ3_NOT_EMPTY_WIDTH      (1)
#define DE_TXQ0_NOT_FULL_STS_EN_WIDTH   (1)
#define DE_RXQ0_COAlESCING_STS_WIDTH    (1)
#define DE_TXQ1_NOT_FULL_STS_EN_WIDTH   (1)
#define DE_RXQ1_COAlESCING_STS_WIDTH    (1)
#define DE_TXQ2_NOT_FULL_STS_EN_WIDTH   (1)
#define DE_RXQ2_COAlESCING_STS_WIDTH    (1)
#define DE_TXQ3_NOT_FULL_STS_EN_WIDTH   (1)
#define DE_RXQ3_COAlESCING_STS_WIDTH    (1)

#define DE_EN_DB_INT_POS                (0)       /* Bits 0-14 */
#define DE_RESERVED_POS                 (15)      /* Bit 15 */
#define DE_EN_TXQ0_NOT_FULL_POS         (16)      /* Bit 16 */
#define DE_EN_RXQ0_NOT_EMPTY_POS        (17)      /* Bit 17 */
#define DE_EN_TXQ1_NOT_FULL_POS         (18)      /* Bit 18 */
#define DE_EN_RXQ1_NOT_EMPTY_POS        (19)      /* Bit 19 */
#define DE_EN_TXQ2_NOT_FULL_POS         (20)      /* Bit 20 */
#define DE_EN_RXQ2_NOT_EMPTY_POS        (21)      /* Bit 21 */
#define DE_EN_TXQ3_NOT_FULL_POS         (22)      /* Bit 22 */
#define DE_EN_RXQ3_NOT_EMPTY_POS        (23)      /* Bit 23 */
#define DE_TXQ0_NOT_FULL_STS_EN_POS     (24)      /* Bit 24 */
#define DE_RXQ0_COAlESCING_STS_POS      (25)      /* Bit 25 */
#define DE_TXQ1_NOT_FULL_STS_EN_POS     (26)      /* Bit 26 */
#define DE_RXQ1_COAlESCING_STS_POS      (27)      /* Bit 27 */
#define DE_TXQ2_NOT_FULL_STS_EN_POS     (28)      /* Bit 28 */
#define DE_RXQ2_COAlESCING_STS_POS      (29)      /* Bit 29 */
#define DE_TXQ3_NOT_FULL_STS_EN_POS     (30)      /* Bit 30 */
#define DE_RXQ3_COAlESCING_STS_POS      (31)      /* Bit 31 */

/* -------------------------------------------------------------------------
 * PMU PLB Address MAP
 * Doorbell assertion register
 * -------------------------------------------------------------------------
 */
#define DA_DBASSERTION_WIDTH            (15)
#define DA_RESERVED_WIDTH               (17)

#define DA_DBASSERTION_POS              (0)       /* Bits 0-14 */
#define DA_RESERVED_POS                 (17)      /* Bits 15-31 */

/* -------------------------------------------------------------------------
 * PMU PLB Address MAP
 * Doorbell enable other processor register
 * -------------------------------------------------------------------------
 */
#define DEOP_DB_EN_OTHER_PROC_WIDTH     (15)
#define DEOP_RESERVED_WIDTH             (17)

#define DEOP_DB_EN_OTHER_PROC_POS       (0)       /* Bits 0-14 */
#define DEOP_RESERVED_POS               (17)      /* Bits 15-31 */

#define GET_REG(reg, fldWidth, fldPos)   (((reg)&(~(~0 << (fldWidth))<<(fldPos)))>>(fldPos))

#define SET_REG(reg,fldWidth,fldPos,val) ((reg) = (((reg) & ~(~(~0 << (fldWidth))<<(fldPos))) |(val & (~(~0 << (fldWidth))))<<(fldPos)));

#define GET_DC_DB_STS(reg)                GET_REG((reg)->dbCntReg, DC_DB_STS_WIDTH,         DC_DB_STS_POS)
#define GET_DC_TXQ0_NOT_FULL(reg)         GET_REG((reg)->dbCntReg, DC_TXQ0_NOT_FULL_WIDTH,  DC_TXQ0_NOT_FULL_POS)
#define GET_DC_RXQ0_NOT_EMPTY(reg)        GET_REG((reg)->dbCntReg, DC_RXQ0_NOT_EMPTY_WIDTH, DC_RXQ0_NOT_EMPTY_POS)
#define GET_DC_TXQ1_NOT_FULL(reg)         GET_REG((reg)->dbCntReg, DC_TXQ1_NOT_FULL_WIDTH,  DC_TXQ1_NOT_FULL_POS)
#define GET_DC_RXQ1_NOT_EMPTY(reg)        GET_REG((reg)->dbCntReg, DC_RXQ1_NOT_EMPTY_WIDTH, DC_RXQ1_NOT_EMPTY_POS)
#define GET_DC_TXQ2_NOT_FULL(reg)         GET_REG((reg)->dbCntReg, DC_TXQ2_NOT_FULL_WIDTH,  DC_TXQ2_NOT_FULL_POS)
#define GET_DC_RXQ2_NOT_EMPTY(reg)        GET_REG((reg)->dbCntReg, DC_RXQ2_NOT_EMPTY_WIDTH, DC_RXQ2_NOT_EMPTY_POS)
#define GET_DC_TXQ3_NOT_FULL(reg)         GET_REG((reg)->dbCntReg, DC_TXQ3_NOT_FULL_WIDTH,  DC_TXQ3_NOT_FULL_POS)
#define GET_DC_RXQ3_NOT_EMPTY(reg)        GET_REG((reg)->dbCntReg, DC_RXQ3_NOT_EMPTY_WIDTH, DC_RXQ3_NOT_EMPTY_POS)
#define GET_DC_QPRIORITY(reg)             GET_REG(reg->dbCntReg,   DC_QPRIORITY_WIDTH,      DC_QPRIORITY_POS)
#define GET_DC_DBPRIORITY(reg)            GET_REG((reg)->dbCntReg, DC_DBPRIORITY_WIDTH,     DC_DBPRIORITY_POS)

#define SET_DC_DB_STS(reg,val)            SET_REG((reg)->dbCntReg, DC_DB_STS_WIDTH,DC_DB_STS_POS, val)

#define GET_DE_EN_DB_INT(reg)             GET_REG((reg)->dbEnbReg, DE_EN_DB_INT_WIDTH,       DE_EN_DB_INT_POS)
#define GET_DE_EN_TXQ0_NOT_FULL(reg)      GET_REG((reg)->dbEnbReg, DE_EN_TXQ0_NOT_FULL_WIDTH, DE_EN_TXQ0_NOT_FULL_POS)
#define GET_DE_EN_RXQ0_NOT_EMPTY(reg)     GET_REG((reg)->dbEnbReg, DE_EN_RXQ0_NOT_EMPTY_WIDTH, DE_EN_RXQ0_NOT_EMPTY_POS)
#define GET_DE_EN_TXQ1_NOT_FULL(reg)      GET_REG((reg)->dbEnbReg, DE_EN_TXQ1_NOT_FULL_WIDTH, DE_EN_TXQ1_NOT_FULL_POS)
#define GET_DE_EN_RXQ1_NOT_EMPTY(reg)     GET_REG((reg)->dbEnbReg, DE_EN_RXQ1_NOT_EMPTY_WIDTH, DE_EN_RXQ1_NOT_EMPTY_POS)
#define GET_DE_EN_TXQ2_NOT_FULL(reg)      GET_REG((reg)->dbEnbReg, DE_EN_TXQ2_NOT_FULL_WIDTH, DE_EN_TXQ2_NOT_FULL_POS)
#define GET_DE_EN_RXQ2_NOT_EMPTY(reg)     GET_REG((reg)->dbEnbReg, DE_EN_RXQ2_NOT_EMPTY_WIDTH, DE_EN_RXQ2_NOT_EMPTY_POS)
#define GET_DE_EN_TXQ3_NOT_FULL(reg)      GET_REG((reg)->dbEnbReg, DE_EN_TXQ3_NOT_FULL_WIDTH, DE_EN_TXQ3_NOT_FULL_POS)
#define GET_DE_EN_RXQ3_NOT_EMPTY(reg)     GET_REG((reg)->dbEnbReg, DE_EN_RXQ3_NOT_EMPTY_WIDTH, DE_EN_RXQ3_NOT_EMPTY_POS)

#define SET_DE_EN_DB_INT(reg,val)         SET_REG((reg)->dbEnbReg, DE_EN_DB_INT_WIDTH, DE_EN_DB_INT_POS, val)
#define SET_DE_EN_TXQ0_NOT_FULL(reg,val)  SET_REG((reg)->dbEnbReg, DE_EN_TXQ0_NOT_FULL_WIDTH, DE_EN_TXQ0_NOT_FULL_POS,  val)
#define SET_DE_EN_RXQ0_NOT_EMPTY(reg,val) SET_REG((reg)->dbEnbReg, DE_EN_RXQ0_NOT_EMPTY_WIDTH, DE_EN_RXQ0_NOT_EMPTY_POS, val)
#define SET_DE_EN_TXQ1_NOT_FULL(reg,val)  SET_REG((reg)->dbEnbReg, DE_EN_TXQ1_NOT_FULL_WIDTH, DE_EN_TXQ1_NOT_FULL_POS,  val)
#define SET_DE_EN_RXQ1_NOT_EMPTY(reg,val) SET_REG((reg)->dbEnbReg, DE_EN_RXQ1_NOT_EMPTY_WIDTH, DE_EN_RXQ1_NOT_EMPTY_POS, val)
#define SET_DE_EN_TXQ2_NOT_FULL(reg,val)  SET_REG((reg)->dbEnbReg, DE_EN_TXQ2_NOT_FULL_WIDTH, DE_EN_TXQ2_NOT_FULL_POS,  val)
#define SET_DE_EN_RXQ2_NOT_EMPTY(reg,val) SET_REG((reg)->dbEnbReg, DE_EN_RXQ2_NOT_EMPTY_WIDTH, DE_EN_RXQ2_NOT_EMPTY_POS, val)
#define SET_DE_EN_TXQ3_NOT_FULL(reg,val)  SET_REG((reg)->dbEnbReg, DE_EN_TXQ3_NOT_FULL_WIDTH, DE_EN_TXQ3_NOT_FULL_POS,  val)
#define SET_DE_EN_RXQ3_NOT_EMPTY(reg,val) SET_REG((reg)->dbEnbReg, DE_EN_RXQ3_NOT_EMPTY_WIDTH, DE_EN_RXQ3_NOT_EMPTY_POS, val)

#define GET_DA_DBASSERTION(reg)           GET_REG((reg)->dbAssReg, DA_DBASSERTION_WIDTH, DA_DBASSERTION_POS)
#define SET_DA_DBASSERTION(reg, val)      SET_REG((reg)->dbAssReg, DA_DBASSERTION_WIDTH, DA_DBASSERTION_POS,val)

#define GET_DEOP_DB_EN_OTHER_PROC(reg)    GET_REG((reg)->dbEnbOthReg, DEOP_DB_EN_OTHER_PROC_WIDTH, DEOP_DB_EN_OTHER_PROC_POS)

void dcSecCoreEnable      (void);
void dcSecCoreDisable     (void);
DCSTATUS dcSecCoreStatus  (void);
DCSTATUS dcInitDB         (void);
DCSTATUS dcClearDBInt     (U32);
DCSTATUS dcSetDBInt       (U32);
DCSTATUS dcSetEnableDBInt (U32);
DCSTATUS dcGetDBStatus    (PTR_DC_PMU_DB_REG);

#endif  /* DC_DB_H */
