/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  dcQP.h
 *          Title:  Dual core PMU queue pointer
 *     Programmer:  Jignesh Moradiya
 *  Creation Date:  May 20,2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who       Description
 *  --------  -----     ----------------------------------------------------
 *  05/20/12  JM        Initial version.
 *
 *
 * Description
 * ------------
 * This file contains definitions of the PMU dual core library for
 * ICM - queue pointer
 * -------------------------------------------------------------------------
 */

#ifndef DC_QP_H
#define DC_QP_H

#include "basetype.h"

#define Q_UB_ADDR_VALUE             (0x00000000)

#define RD_BUF_MAX_SIZE             (50)
#define MAX_Q_NUM                   (3)

#define Q_WR_PTR_WIDTH              (16)
#define Q_RD_PTR_WIDTH              (16)
#define Q_SIZE_WIDTH                (16)
#define Q_RESET_REQ_WIDTH           (1)
#define Q_RESET_WIDTH               (1)
#define Q_LB_ADDR_WIDTH             (32)
#define Q_UB_ADDR_WIDTH             (32)

#define Q_WR_PTR_POS                (0)
#define Q_RD_PTR_POS                (16)
#define Q_SIZE_POS                  (0)
#define Q_RESET_REQ_POS             (30)
#define Q_RESET_POS                 (31)
#define Q_LB_ADDR_POS               (0)
#define Q_UB_ADDR_POS               (0)

#define COAL_INT_TIMER_WIDTH        (8)
#define COAL_INT_TIMER_TICK_WIDTH   (1)
#define COAL_INT_CNT_WIDTH          (8)

#define COAL_INT_TIMER_POS          (0)
#define COAL_INT_TIMER_TICK_POS     (15)
#define COAL_INT_CNT_POS            (16)

/* Macros for  Tx Queue Regsiters */
#define GET_TXQ_RWPTR_WR_PTR(reg)        GET_REG(reg->qpTxRWPtrReg, Q_WR_PTR_WIDTH, Q_WR_PTR_POS)
#define GET_TXQ_RWPTR_RD_PTR(reg)        GET_REG(reg->qpTxRWPtrReg, Q_RD_PTR_WIDTH, Q_RD_PTR_POS)

#define GET_TXQ_CNTL_SIZE(reg)           GET_REG(reg->qpTxCntReg, Q_SIZE_WIDTH,      Q_SIZE_POS)
#define GET_TXQ_CNTL_RESET_REQ(reg)      GET_REG(reg->qpTxCntReg, Q_RESET_REQ_WIDTH, Q_RESET_REQ_POS)
#define GET_TXQ_CNTL_RESET(reg)          GET_REG(reg->qpTxCntReg, Q_RESET_WIDTH ,    Q_RESET_POS)

#define GET_TXQ_LB_ADDR(reg)             reg->qpTxLBAddrReg
#define GET_TXQ_UB_ADDR(reg)             reg->qpTxUBAddrReg

#define SET_TXQ_RWPTR_WR_PTR(reg,val)    SET_REG(reg->qpTxRWPtrReg, Q_WR_PTR_WIDTH, Q_WR_PTR_POS, val)
#define SET_TXQ_CNTL_SIZE(reg,val)       SET_REG(reg->qpTxCntReg,   Q_SIZE_WIDTH,   Q_SIZE_POS,   val)
#define SET_TXQ_CNTL_RESET(reg,val)      SET_REG(reg->qpTxCntReg,   Q_RESET_WIDTH,  Q_RESET_POS,  val)
                                                
#define SET_TXQ_LB_ADDR(reg,val)         (reg->qpTxLBAddrReg = val)
#define SET_TXQ_UB_ADDR(reg,val)         (reg->qpTxUBAddrReg = val)

/* Macros for  Rx Queue Regsiters */
#define GET_RXQ_RWPTR_WR_PTR(reg)        GET_REG(reg->qpRxRWPtrReg, Q_WR_PTR_WIDTH, Q_WR_PTR_POS)
#define GET_RXQ_RWPTR_RD_PTR(reg)        GET_REG(reg->qpRxRWPtrReg, Q_RD_PTR_WIDTH, Q_RD_PTR_POS)

#define GET_RXQ_CNTL_SIZE(reg)           GET_REG(reg->qpRxCntReg, Q_SIZE_WIDTH,      Q_SIZE_POS)
#define GET_RXQ_CNTL_RESET_REQ(reg)      GET_REG(reg->qpRxCntReg, Q_RESET_REQ_WIDTH, Q_RESET_REQ_POS)
#define GET_RXQ_CNTL_RESET(reg)          GET_REG(reg->qpRxCntReg, Q_RESET_WIDTH,     Q_RESET_POS)

#define GET_RXQ_LB_ADDR(reg)             GET_REG(reg->qpRxLBAddrReg, Q_LB_ADDR_WIDTH , Q_LB_ADDR_POS)
#define GET_RXQ_UB_ADDR(reg)             GET_REG(reg->qpRxUBAddrReg, Q_UB_ADDR_WIDTH , Q_UB_ADDR_POS)

#define SET_RXQ_RWPTR_RD_PTR(reg,val)    SET_REG(reg->qpRxRWPtrReg, Q_RD_PTR_WIDTH, Q_RD_PTR_POS, val)
#define SET_RXQ_CNTL_SIZE(reg,val)       SET_REG(reg->qpRxCntReg,   Q_SIZE_WIDTH,   Q_SIZE_POS,   val)
#define SET_RXQ_CNTL_RESET(reg,val)      SET_REG(reg->qpRxCntReg,   Q_RESET_WIDTH,  Q_RESET_POS,  val)

/* Macros for  Queue Coelscing Regsiters */
#define GET_COAL_INT_TIMER(reg)          GET_REG(*reg, COAL_INT_TIMER_WIDTH,      COAL_INT_TIMER_POS)
#define GET_COAL_INT_TIMER_TICK(reg)     GET_REG(*reg, COAL_INT_TIMER_TICK_WIDTH, COAL_INT_TIMER_TICK_POS)
#define GET_COAL_INT_CNT(reg)            GET_REG(*reg, COAL_INT_CNT_WIDTH,        COAL_INT_CNT_POS)

#define SET_COAL_INT_TIMER(reg,val)      SET_REG(*reg, COAL_INT_TIMER_WIDTH,      COAL_INT_TIMER_POS,      val)
#define SET_COAL_INT_TIMER_TICK(reg,val) SET_REG(*reg, COAL_INT_TIMER_TICK_WIDTH, COAL_INT_TIMER_TICK_POS, val)
#define SET_COAL_INT_CNT(reg,val)        SET_REG(*reg, COAL_INT_CNT_WIDTH,        COAL_INT_CNT_POS,        val)

/* --------------------------------------------------------------------------
 *                COMMON STRUCTURES - Queue pointer
 * --------------------------------------------------------------------------
 */

#define QP_TX_RW_REG                (0x00)
#define QP_TX_CNT_REG               (0x01)
#define QP_TX_LB_ADDR_REG           (0x02)
#define QP_TX_UB_ADDR_REG           (0x03)
#define QP_RX_RW_REG                (0x04)
#define QP_RX_CNT_REG               (0x05)
#define QP_RX_LB_ADDR_REG           (0x06)
#define QP_RX_UB_ADDR_REG           (0x07)

typedef volatile struct _DC_PMU_QP_REG
{
    U32 qpTxRWPtrReg;
    U32 qpTxCntReg;
    U32 qpTxLBAddrReg;
    U32 qpTxUBAddrReg;
    U32 qpRxRWPtrReg;
    U32 qpRxCntReg;
    U32 qpRxLBAddrReg;
    U32 qpRxUBAddrReg;
} __attribute__((__packed__)) DC_PMU_QP_REG, *PTR_DC_PMU_QP_REG;

DCSTATUS dcInitQP          (U32, U32, U32);
DCSTATUS dcWriteQP         (U32, char *);
DCSTATUS dcReadQP          (U32, char *, int *);
DCSTATUS dcSetCoalIntCnt   (U32, U32);
DCSTATUS dcSetCoalIntTimer (U32, U32, U32);
DCSTATUS dcSetNotFullInt   (U32, U32);
DCSTATUS dcSetNotEmptyInt  (U32, U32);
DCSTATUS dcResetTxQ        (U32);
DCSTATUS dcResetRxQ        (U32);

#endif    /* DC_QP_H */
