/***************************************************************************
 *                                                                         *
 *                                                                         *
 *  Copyright 2003-2006 LSI Logic Corporation.  All rights reserved.       *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiIFault.h
 *          Title:  SAS Firmware IOP Interface IFAULT code Definitions
 *     Programmer:  Steve Johnson
 *  Creation Date:  March 21, 2003
 *
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface IFAULT codes
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPI_FAULT_H_INCLUDED
#define IOPI_FAULT_H_INCLUDED


/******************************************************************************
* Generic Interface IFAULT code definitions 0 - 0xFFFF
*******************************************************************************/

/******************************************************************************
* IOP IFAULT code definitions 0x0000 - 0x3FFF
*******************************************************************************/

#define IFAULT_IOP_BOOTL_INVALID_DOORBELL              (0x0001) /* Host wrote an invalid value to the doorbell during the handshake */
//#define IFAULT_IOP_BOOTL_DMA_ERROR_PORT1             (0x0002)
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_1             (0x0002) /* Creation of TLBs failed (too many required) */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_2             (0x0003) /* IR requested an invalid NVSRAM size */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_3             (0x0004) /* Insufficient memory detected for code and data. */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_4             (0x0005) /* Bad Context RAM found */
#define IFAULT_IOP_BOOTL_INVALID_BL_IMAGE_1            (0x0006) /* Boot loader not found */
#define IFAULT_IOP_BOOTL_INVALID_BL_IMAGE_2            (0x0007) /* Boot loader failed checksum (when copied to Context RAM) */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_5             (0x0008) /* IIC read of SPD failed for DDR */
//#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_6             (0x0009) /* Bad IIC data */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_7             (0x000A) /* DDR DIMM is an invalid size */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_8             (0x000B) /* DDR DIMM POST failure */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_9             (0x000C) /* DDR DIMM Zero failure */
#define IFAULT_IOP_BOOTL_DDR_FW_EXCEPTION              (0x000D) /* DDR training failure */
#define IFAULT_IOP_BOOTL_DDR_GATEON_FAILED             (0x000E) /* DDR Gateon training failure */
#define IFAULT_IOP_BOOTL_DDR_UNCOR_DDR_ECC             (0x000F) /* Uncorrectable DDR ECC failure */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_10            (0x0010) /* DDR enabled without chip support */
#define IFAULT_IOP_BOOTL_EDRAM_BISR_TIMEOUT            (0x0011) /* EDRAM Build In Self Repair timeout */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_11            (0x0012) /* Debugging fault for DDR DIMM POST failure */
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_12            (0x0013) /* Debugging fault for DDR DIMM Zero failure */
#define IFAULT_IOP_BOOTL_NO_NVSRAM                     (0x0014) /* No NVSRAM */
#define IFAULT_IOP_FREE_INVALID_FRAME_1                (0x0100) /* Tried to free a nonexistent message frame  */
//#define IFAULT_IOP_FREE_INVALID_FRAME_2                (0x0101) // stress test
//#define IFAULT_IOP_SHARED_MEM_PARITY                 (0x0200)
#define IFAULT_IOP_BAD_FRAME_SIZE                      (0x0300) /* Invalid message size defined in manufacturing page 9 */
#define IFAULT_IOP_EXT_MEM_INSUFFICIENT_1              (0x0301) /* Memory allocation process was not successful and all resources are at minimum. */
#define IFAULT_IOP_EXT_MEM_INSUFFICIENT_2              (0x0302) /* Failed to allocate resources for Start Of Day variables. */
//#define IFAULT_IOP_EXT_MEM_INSUFFICIENT_3              (0x0303) // stress test
//#define IFAULT_IOP_EXT_MEM_INSUFFICIENT_4              (0x0304) // stress test
#define IFAULT_IOP_EXT_MEM_INSUFFICIENT_5              (0x0305) /* Failed Start Of Day allocaction logic check */
#define IFAULT_IOP_PARAM_COPY_FAILED                   (0x0306) /* copying of boot params from bootloader to main code failed */
//#define IFAULT_IOP_EXT_MEM_TEST_FAIL                 (0x0302)
//#define IFAULT_IOP_SYS_DOORBELL_FAIL_1                 (0x0401) // #if 0
#define IFAULT_IOP_SYS_DOORBELL_FAIL_2                 (0x0402) /* Host failed to clear doorbell interrupt. */
//#define IFAULT_IOP_INVALID_FRAME_STATE_FREE_1            (0x0500)
#define IFAULT_IOP_INVALID_FRAME_STATE_FREE_2          (0x0501) /* Tried to free the same frame context twice. */
//#define IFAULT_IOP_INVALID_FRAME_STATE_REQ_FREE        (0x0502) /* Tried to free the same frame context twice. */
#define IFAULT_IOP_SCSIO_MSG_INVALID_CHAIN             (0x0600) /* SGE is not chain type, local and 32-bit */
//#define IFAULT_IOP_SCSIO_MSG_INVALID_SGE_RESERVED    (0x0602)
#define IFAULT_IOP_SCSIO_MSG_INVALID_CHAIN1            (0x0602) /* SGE is not chain type, local  */
#define IFAULT_IOP_SCSIO_MSG_INVALID_CHAIN2            (0x0603) /* SGE is not chain type, local */
#define IFAULT_IOP_SCSIO_MSG_INVALID_CHAIN3            (0x0604) /* Chain SGE must be chain type, local and 32-bit */
#define IFAULT_IOP_SCSIO_MSG_INVALID_LE                (0x0605) /* More than one Link Element found in SGL */
#define IFAULT_IOP_SCSIO_MSG_INVALID_ELEM_TYPE         (0x0606) /* SGE is not a simple element */
#define IFAULT_IOP_SCSIO_MSG_INVALID_LOCAL_ADDR        (0x0607) /* SGE has an invalid local address */
#define IFAULT_IOP_SCSIO_MSG_INVALID_NUM_CHAINS        (0x0608) /* Calculated and expected number of chains do not match */
#define IFAULT_IOP_SCSIO_MSG_INVALID_CDB_LEN           (0x0609) /* Encountered SCSI CDB length of zero */
//#define IFAULT_IOP_SCSIO_MSG_INVALID_RSVD            (0x060A)
#define IFAULT_IOP_SCSIO_MSG_INVALID_MSGFLAGS          (0x060B) /* Encountered invalid message flags in SCSI IO */
#define IFAULT_IOP_SCSIO_MSG_INVALID_CHAIN_OFF_1       (0x060C) /* SGL chain offset found in message with no data */
#define IFAULT_IOP_SCSIO_MSG_INVALID_CHAIN_OFF_2       (0x060D) /* SGL chain offset larger than message size */
#define IFAULT_IOP_SCSIO_MSG_EXCEED_CHAIN_MAX          (0x060E) /* The host exceeded the maximum chain limit */
#define IFAULT_IOP_SCSIO_MSG_MASA_OVERFLOW             (0x060F) /* The flow-through request descriptor FIFO overflowed */
#define IFAULT_IOP_SCSIO_TM_MUR_FAIL                   (0x0610) /* Task Management failed to remove outstanding IO */
#define IFAULT_IOP_SCSIO_PART_MUR_FAIL                 (0x0611) /* Simple MUR (Message Unit Reset) failed to free all non-IR frames */
#define IFAULT_IOP_SCSIO_MUR_MSGCNT_NEGATIVE           (0x0612) /* ActiveMsgCount became negative during MUR (Message Unit Reset) */
#define IFAULT_IOP_SCSIO_TA_MUR_FAIL                   (0x0613) /* Task Abort failed to remove outstanding IO */

#define IFAULT_IOP_FLASH_VALIDATION_FAILED_1          (0x0701) /* Failed to initialize configuration region in flash  */
#define IFAULT_IOP_FLASH_VALIDATION_FAILED_2          (0x0702) /* Validation of configuration region failed */
#define IFAULT_IOP_FLASH_VALIDATION_FAILED_3          (0x0703) /* Migration of configuration region failed */
#define IFAULT_IOP_FLASH_VALIDATION_FAILED_4          (0x0704) /* Update of current configuration page failed */
#define IFAULT_IOP_FLASH_VALIDATION_FAILED_5          (0x0705) /* Read of flash region failed */
#define IFAULT_IOP_FLASH_VALIDATION_FAILED_6          (0x0706) /* Write of log page failed */
#define IFAULT_IOP_FLASH_VALIDATION_FAILED_7          (0x0707) /* Write of log entry failed */
#define IFAULT_IOP_AHB_BUS_FAULT_1                    (0x0800) /* PBAM Error */
#define IFAULT_IOP_AHB_BUS_FAULT_2                    (0x0801) /* PBAM Error */
#define IFAULT_IOP_STACK_OVERFLOW_1                   (0x0900) /* Call stack overflow or underflow */
//#define IFAULT_IOP_STACK_OVERFLOW_2                 (0x0901) // Stress test
//#define IFAULT_IOP_FIQ_STACK_OVERFLOW               (0x0901)
//#define IFAULT_IOP_NO_IOP_RESOURCE_FRAMES           (0x0A00)
//#define IFAULT_IOP_NO_IOP_RESOURCE_FRAMES_MFG_5     (0x0A01)
#define IFAULT_IOP_NO_IOP_RESOURCE_FRAMES_DOORBELL  (0x0A02) /* Failed to allocate critical frame */
#define IFAULT_IOP_RESOURCE_QUEUE_OVERFLOW          (0x0A03) /* Logical assertion - too many resources placed on resource queue */
#define IFAULT_IOP_FREE_QUEUE_OVERFLOW              (0x0A04) /* Logical check - ensure number of frames placed on free queues is correct */
#define IFAULT_IOP_NO_IOP_CRITICAL_FRAMES_AVAIL     (0x0B00) /* Critical frame is already allocated */
#define IFAULT_IOP_INVALID_FRAME_SIZE               (0x0C00) /* MPI message size > hardware message frame size */
#define IFAULT_IOP_DCR_ARBITER_TIMEOUT              (0x0D02) /* DCR Arbiter timeout for master port 1, UIC 1 */
#define IFAULT_IOP_MIRQ                             (0x0D03) /* Process local bus MIRQ error */
#define IFAULT_IOP_UIC_UNEXP_INTS                   (0x0D04) /* Unexpected UIC interrupt, critical error */
#define IFAULT_IOP_ADAPTER_RESET_IRQ                (0x0D05) /* Adapter Reset was turned off or delayed */
#define IFAULT_IOP_GPIO_INDICATION                  (0x0D10) /* Fault requested via GPIO input */
#define IFAULT_IOP_UNEXPECTED_GPIO_EXT_INT          (0x0D11) /* Unexpected GPIO external interrupt */
//#define IFAULT_IOP_RING_BUFFER_ERROR                (0x0E00)
//#define IFAULT_IOP_INVALID_MAX_DEVICES              (0x0F00)
//#define IFAULT_IOP_INVALID_SAS_ADDRESS              (0x1000)
//#define IFAULT_IOP_INVALID_FLASH_IMAGE              (0x1100)
#define IFAULT_IOP_TASK_MGMT_1                        (0x1201) /* Unknown Task Management source */
#define IFAULT_IOP_TASK_MGMT_2                        (0x1202) /* Task Management not found in queue */
#define IFAULT_IOP_TASK_MGMT_3                        (0x1203) /* Task Management failed to remove BIOS IO during asynchronous broadcast */


#define IFAULT_IOP_NVDATA_MISMATCH_PERISTENT_PAGES    (0x1301) /* Failed to find configuration page */
#define IFAULT_IOP_NVDATA_MISMATCH_CURRENT_PAGES_1    (0x1302) /* Failed to find IOC configuration page 8 */
#define IFAULT_IOP_NVDATA_MISMATCH_CURRENT_PAGES_2    (0x1303) /* Failed to find persistent configuration page in NVDATA */
#define IFAULT_IOP_NO_NVDATA_1                        (0x1304) /* Failed to find NVDATA matching chip in Main code */
#define IFAULT_IOP_NO_NVDATA_2                        (0x1305) /* Encountered invalid NVDATA signature  */
#define IFAULT_IOP_NO_NVDATA_5                        (0x1308) /* Failed to find manufacturing page 11 in NVDATA */
#define IFAULT_IOP_NO_NVDATA_6                        (0x1309) /* Failed to find IoUnitPage1 in NVDATA */
#define IFAULT_IOP_NO_NVDATA_7                        (0x130A) /* Failed to find NVDATA matching chip in boot loader */
#define IFAULT_IOP_NO_NVDATA_8                        (0x130B) /* Failed to find manufacturing page 16 in NVDATA */
#define IFAULT_IOP_NO_NVDATA_9                        (0x130C) /* Invalid SysRefClk in manufacturing page 11 */
#define IFAULT_IOP_NO_NVDATA_10                       (0x130D) /* Invalid PPC clock frequency */
#define IFAULT_IOP_NO_NVDATA_11                       (0x130E) /* Failed to find manufacturing page 2 in NVDATA */
#define IFAULT_IOP_NO_NVDATA_12                       (0x130F) /* Failed to find one of manufacturing pages 6, 8 or 11 */

#define IFAULT_IOP_REV_ID_MISMATCH                    (0x1400) /* Found unsupported chip ID or revision */
#define IFAULT_IOP_REV_NO_WORKAROUND_MATCH            (0x1401) /* Failed to find a workaround matrix for chip ID & revision */
#define IFAULT_IOP_MISSING_BITS_HW_WKRD_MATRIX        (0x1402) /* Incorrect initialization of workaround bits */
#define IFAULT_IOP_GPIO_INIT_ERROR                    (0x1480) /* Tried to initialize an invalid GPIO */
#define IFAULT_IOP_SCHEDULER_ERROR_1                  (0x1500) /* Encountered Fusion Messaging Unit error  */
//#define IFAULT_IOP_SCHEDULER_ERROR_2                  (0x1501) // stress test

#define IFAULT_IOP_MID_OUT_OF_RANGE_SCSIIO_COMP_1     (0x1600) /* Encountered an invalid MID number */
//#define IFAULT_IOP_MID_OUT_OF_RANGE_SCSIIO_COMP_2     (0x1601) // stress test
#define IFAULT_IOP_MID_OUT_OF_RANGE_SCSIIO_COMP_ERR_1 (0x1602) /* Encountered an invalid MID number */
//#define IFAULT_IOP_MID_OUT_OF_RANGE_SCSIIO_COMP_ERR_2 (0x1603) // stress test
#define IFAULT_IOP_MID_OUT_OF_RANGE_SCSIIO_START      (0x1604) /* Encountered an invalid MID number */
#define IFAULT_IOP_MID_OUT_OF_RANGE_MSG_COMP_1        (0x1605) /* Encountered an invalid MID number */
//#define IFAULT_IOP_MID_OUT_OF_RANGE_MSG_COMP_2        (0x1606) // stress test

#define IFAULT_IOP_MID_OUT_OF_RANGE_FREE_FRAME_1      (0x1607) /* Encountered an invalid MID number */
#define IFAULT_IOP_MID_OUT_OF_RANGE_FREE_FRAME_2      (0x1608) /* Encountered an invalid MID number */
#define IFAULT_IOP_MID_OUT_OF_RANGE_FREE_FRAME_3      (0x1609) /* Encountered an invalid MID number */
#define IFAULT_IOP_MID_OUT_OF_RANGE_FREE_FRAME_4      (0x160A) /* Encountered an invalid MID number */
#define IFAULT_IOP_MID_OUT_OF_RANGE_FREE_FRAME_5      (0x160B) /* Encountered an invalid MID number */
//#define IFAULT_IOP_MID_OUT_OF_RANGE_FREE_FRAME_6      (0x160C) // stress test
#define IFAULT_IOP_UNEXPECTED_FPE_EXCEPTION             (0x1611) /* Received an FPE exception that IOP doesn't handle */

/* Raid Accelerator Faults */
#define IFAULT_IOP_MID_OUT_OF_RANGE_RAID_ACCELERATOR_SUCCESS    (0x1700) /* Raid Accelerator completion path successful */
#define IFAULT_IOP_MID_OUT_OF_RANGE_RAID_ACCELERATOR_ERROR      (0x1701) /* Raid Accelerator completion path error */
#define IFAULT_IOP_INVALID_RAID_ACCELERATOR_REPLY_1             (0x1702) /* Failed to determine source of RAID Accelerator IO */
#define IFAULT_IOP_INVALID_RAID_ACCELERATOR_REPLY_2             (0x1703) /* Failed to determine source of RAID Accelerator IO */
#define IFAULT_IOP_INVALID_RAID_ACCELERATOR_INVALID_COMP_Q_SIZE (0x1704) /* Failed to determine source of RAID Accelerator IO */
#define IFAULT_IOP_RAID_ACCELERATOR_INIT_FAILED_DDR             (0x1705) /* Initialization failed due to missing DDR */
#define IFAULT_IOP_RAID_ACCELERATOR_INIT_FAILED_CONTROL_SIZE    (0x1706) /* Initialization failed due to Control Size being out of range */

#define IFAULT_IOP_SELFTEST_FAILED_RSRC_BUF           (0x1800) /* Resource buffer self-test failed */
//#define IFAULT_IOP_SELFTEST_FAILED_IOPFLAGS         (0x1801)
//#define IFAULT_IOP_SELFTEST_FAILED_CACHE_INIT       (0x1802)
//#define IFAULT_IOP_SELFTEST_FAILED_TRANS_TABLE      (0x1803)
//#define IFAULT_IOP_SELFTEST_FAILED_TIMER            (0x1804)
//#define IFAULT_IOP_SELFTEST_FAILED_SYSIF            (0x1805)
#define IFAULT_IOP_SELFTEST_FAILED_DIVIDER            (0x1806) /* Division self-test failed */
#define IFAULT_IOP_TIMER_CREATE_FAILED                (0x1807) /* Failed to create and initialize a timer */
#define IFAULT_IOP_DDR_SPD_FAILED                     (0x1808) /* Failed to acquire the DDR SPD information for manufacturing page 16 */
#define IFAULT_IOP_DDR_SBR_SPEED_EXCEED_SPD           (0x1809) /* The SBR memory clock is set at a higher speed than the DIMM supports */
#define IFAULT_IOP_INVALID_REPLY_1                    (0x1900) /* Failed to determine source of SCSI IO */
#define IFAULT_IOP_INVALID_REPLY_2                    (0x1901) /* Failed to determine source of SCSI IO */
#define IFAULT_IOP_INVALID_REPLY_3                    (0x1902) /* Failed to determine source of SCSI IO */
//#define IFAULT_IOP_INVALID_REPLY_4                    (0x1903) // stress test
//#define IFAULT_IOP_INVALID_REPLY_5                    (0x1904) // stress test
//#define IFAULT_IOP_INVALID_REPLY_6                    (0x1905) // stress test
#define IFAULT_IOP_INVALID_REPLY_7                    (0x1906) // Invalid Source found when processing HBD Action Reply
#define IFAULT_IOP_INVALID_REPLY_8                    (0x1907)  /* Invalid source for Power Management Control Request Message */

//#define IFAULT_IOP_UNHANDLED_QUAD_CPLT_INTERRUPT    (0x1B00)
//#define IFAULT_IOP_UNHANDLED_QUAD_INT_INTERRUPT     (0x1C00)
#define IFAULT_IOP_UNHANDLED_EVENT_1                  (0x1D00) /* Unsupported reason code for a device status change event */
#define IFAULT_IOP_UNHANDLED_EVENT_2                  (0x1D01) /* Unsupported MPI event */
//#define IFAULT_SRW_BUS_FAULT_ERROR                  (0x1E00)
//#define IFAULT_SRW_PARITY_ERROR                     (0x1F00)
//#define IFAULT_IOP_PCI_BUS_FAULT_ERROR              (0x2000)
//#define IFAULT_IOP_MESSAGING_QUEUE_OVERFLOW         (0x2100) /* FIQ triggered fault */
//#define IFAULT_INVALID_RUNTIME_CONFIGURATION        (0x2200)
#define IFAULT_IOP_NO_THREAD_TIMER                    (0x2300) /* Failed to create and initialize a timer */
//#define IFAULT_INSUFFICIENT_HOST_BUFFER             (0x2400)
//#define IFAULT_IOP_SDRAM_UNCORRECTABLE_ECC_ERROR    (0x2600) /* FIQ triggered fault */
//#define IFAULT_IOP_PLB2AHB_ERROR                    (0x2610) /* FIQ triggered fault */
//#define IFAULT_IOP_PCI_EXPRESS_ERROR_1              (0x2621) /* FIQ triggered fault */
#define IFAULT_IOP_PCI_EXPRESS_ERROR_2                (0x2622) /* PCI Express error */
//#define IFAULT_IOP_PCI_EXPRESS_ERROR_3              (0x2623) // stress test
//#define IFAULT_IOP_MIRQ_ERROR                       (0x2630) /* FIQ triggered fault */
//#define IFAULT_IOP_I2O_DMA_ERROR                    (0x2640) /* FIQ triggered fault */
//#define IFAULT_DMA_ERROR_PORT1                      (0x2650)

//#define IFAULT_FATAL_INT                            (0x2650)  /* 0x2650 - 0x265F */
#define IFAULT_FATAL_INT_CRITICAL                   (0x2650) /* Critical interrupt pin triggered */
#define IFAULT_FATAL_INT_MACHINE_CHECK              (0x2651) /* Critical machine check exception */
#define IFAULT_FATAL_INT_DATA_STORAGE               (0x2652) /* Noncritical data storage interrupt */
#define IFAULT_FATAL_INT_INSTRUCTION_STORAGE        (0x2653) /* Noncritical instruction storage interrupt */
#define IFAULT_FATAL_INT_EXTERNAL_INPUT             (0x2654) /* Noncritical external interrupt */
#define IFAULT_FATAL_INT_ALIGNMENT                  (0x2655) /* Noncritical alignment exception */
#define IFAULT_FATAL_INT_PROGRAM                    (0x2656) /* Noncritical program interrupt */
#define IFAULT_FATAL_INT_FLOATING_POINT_UNAVAILABLE (0x2657) /* Noncritical floating point unavailable interrupt */
#define IFAULT_FATAL_INT_SYSTEM_CALL                (0x2658) /* Noncritical system call interrupt */
#define IFAULT_FATAL_INT_AUXILIARY_PROC_UNAVAILABLE (0x2659) /* Noncritical auxiliary process unavailable interrupt */
#define IFAULT_FATAL_INT_DECREMENTER                (0x265A) /* Noncritical decrementer interrupt */
#define IFAULT_FATAL_INT_FIXED_INTERVAL_TIMER       (0x265B) /* Noncritical fixed interval interrupt */
#define IFAULT_FATAL_INT_WATCHDOG_TIMER             (0x265C) /* Critical watchdog timer interrupt */
#define IFAULT_FATAL_INT_DATA_TLB_ERROR             (0x265D) /* Noncritical data TLB interrupt */
#define IFAULT_FATAL_INT_INSTRUCTION_TLB_ERROR      (0x265E) /* Noncritical instruction TLB interrupt */
#define IFAULT_FATAL_INT_DEBUG                      (0x265F) /* Critical debug interrupt */

// #define IFAULT_IOP_SYSDMA                           (0x2660)
#define IFAULT_IOP_SYSDMA_SEND_PARITY_ERR_1           (0x2661) /* Parity error on host-to-IOC DMA */
#define IFAULT_IOP_SYSDMA_SEND_PARITY_ERR_2           (0x2662) /* Parity error on host-to-IOC DMA */
#define IFAULT_IOP_SYSDMA_RECEIVE_PARITY_ERR_1        (0x2663) /* Parity error on IOC-to-host DMA */
#define IFAULT_IOP_SYSDMA_RECEIVE_PARITY_ERR_2        (0x2664) /* Parity error on IOC-to-host DMA */
#define IFAULT_IOP_SYSDMA_SEND_BUS_FAULT_1            (0x2665) /* Bus fault on host-to-IOC memory move */
#define IFAULT_IOP_SYSDMA_SEND_BUS_FAULT_2            (0x2666) /* Bus fault on host-to-IOC memory move */
#define IFAULT_IOP_SYSDMA_RECEIVE_BUS_FAULT_1         (0x2667) /* Bus fault on IOC-to-host memory move */
#define IFAULT_IOP_SYSDMA_RECEIVE_BUS_FAULT_2         (0x2668) /* Bus fault on IOC-to-host memory move */
//#define IFAULT_IOP_SYSDMA_RECEIVE_BUS_FAULT_3         (0x2669) // stress test
#define IFAULT_IOP_SYSDMA_UNMAPPED_MEMORY_ERR_1       (0x266A) /* Address translation failed */
#define IFAULT_IOP_SYSDMA_UNMAPPED_MEMORY_ERR_2       (0x266B) /* Address translation failed */
#define IFAULT_IOP_SYSDMA_TCMMAPPED_MEMORY_ERR        (0x266C) /* Address is not within TCM */
#define IFAULT_IOP_SYSDMA_IRQ_ERR                    (0x266D) /* SysDMA Error bit set during transfer */

#define IFAULT_POWER_LOSS_EXPECTED                    (0x2670) /* EPOW  */

#define IFAULT_IOP_ISTWI_BUS_RESERVE_ERROR            (0x2680) /* Reservation of ISTWI bus failed */
#define IFAULT_IOP_ISTWI_NO_SBR_IN_NVDATA             (0x2681) /* SBR type not found in NVDATA */

#define IFAULT_IOP_MFG_PAGE_NOT_FOUND                 (0x2690) /* Failed to find manufacturing page */

#define IFAULT_IOP_INVALID_CRIT_FRAME_REQUEST         (0x2700) /* Bits 5:0 = frame, bits 7:6 = subsystem */
//#define IFAULT_IOP_UNKNOWN                          (0x3333)

#define IFAULT_IOP_PWRMGMT_BAD_EXT_BUFFER_UPLOAD    (0x280B) /* Data to be uploaded is greater than available buffer space */
#define IFAULT_IOP_PWRMGMT_CORRUPTED_TAIL_POINTER   (0x280C) /* Extended diagnostic buffer tail pointer contains junk data */
#define IFAULT_IOP_HOST_INTENTIONAL_FAULT_SPECIAL_VALUE (0x3FFF) /* Host wrote 0xC0FFEE00 to System Doorbell register  */
#define IFAULT_IOP_IRQ_FAULT                            (0x4000) /* unhandled interrupt irq fault */

#define IFAULT_IOP_FRM_CHANGE_PL_OWNED                  (0x5000) /* Changing frame owner when PL already own the frame */
#define IFAULT_IOP_FRM_CHANGE_IOP_OWNED                 (0x5001) /* Changing frame owner when IOP already own the frame */
/******************************************************************************
* PL IFAULT code definitions 0x4000 - 0x7FFF
*******************************************************************************/

/* See pliFault.h */

/**************************************************s****************************
* IR IFAULT code definitions 0x8000 - 0xBFFF
*******************************************************************************/

//Interface
#define IFAULT_IR_ALLOC_TOPO_EVENT_FAILED           (0x8001) /* Failed to create topology change event */
#define IFAULT_IR_NO_NVSRAM                         (0x8002) /* Insufficient NVSRAM to hold Start of Day parameters */
#define IFAULT_IR_BAD_NVSRAM_1                      (0x8003) /* NVSRAM failed byte read check */
#define IFAULT_IR_BAD_NVSRAM_2                      (0x8004) /* NVSRAM failed word read check */
#define IFAULT_IR_BAD_NVSRAM_3                      (0x8005) /* NVSRAM failed byte read check */
#define IFAULT_IR_BAD_NVSRAM_4                      (0x8006) /* NVSRAM failed signature check */
#define IFAULT_IR_RETARGET_1                        (0x8007) /* _sys_exit retarget */
#define IFAULT_IR_RETARGET_2                        (0x8008) /* __rt_lib_shutdown retarget */
#define IFAULT_IR_RETARGET_3                        (0x8009) /* End retarget */
#define IFAULT_IR_RETARGET_4                        (0x800A) /* __cxa_begin_catch retarget */
#define IFAULT_IR_RETARGET_5                        (0x800B) /* __cxa_end_catch retarget */
#define IFAULT_IR_RETARGET_6                        (0x800C) /* __cxa_rethrow retarget */
#define IFAULT_IR_RETARGET_7                        (0x800D) /* __dso_handle retarget */
#define IFAULT_IR_RETARGET_8                        (0x800E) /* __aeabi_atexit retarget */

//IO Path
#define IFAULT_IR_QUEUE_IO_COUNT_ERROR              (0x8101) /* IO count does not match actual number of IOs */
#define IFAULT_IR_DEFERRED_ENGINE_NULL              (0x8102) /* Null deferred engine pointer */
#define IFAULT_IR_ABORT_VIRT_IO_ERROR               (0x8103) /* Null virtual SCSI IO pointer */
#define IFAULT_IR_NOT_BOUND                         (0x8104) /* Null virtual SCSI IO pointer */
#define IFAULT_IR_INVALID_PHYS_DISK_STATE           (0x8105) /* Invalid physical disk state encountered */
#define IFAULT_IR_DISK_SET_MEMBER_OF_VOLUME_ERROR   (0x8106) /* Error replacing failed drive with hot spare */
#define IFAULT_IR_ACTIVE_PATH_INDEX_INVALID         (0x8107) /* Invalid path index encountered */
#define IFAULT_IR_ACTIVE_PATH_NOT_IN_RPD0           (0x8108) /* Failing path is not pointed to by RAID Physical Disk Page 0 */
#define IFAULT_IR_NON_ZERO_IO_COUNT                 (0x8109) /* Not all IOs were aborted by task management when drive went missing */
#define IFAULT_IR_DIVIDE_BY_ZERO_ERROR              (0x810A) /* Tried to divide by zero */
#define IFAULT_IR_INVALID_CDB_LENGTH                (0x810B) /* Received CDB of unsupported length */
#define IFAULT_IR_INVALID_MID_ON_COMPLETION         (0x810C) /* Invalid MID encountered */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_1       (0x8111) /* Performed an illegal math operation */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_2       (0x8112) /* Performed an illegal math operation */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_3       (0x8113) /* Performed an illegal math operation */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_4       (0x8114) /* Performed an illegal math operation */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_5       (0x8115) /* Performed an illegal math operation */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_6       (0x8116) /* Performed an illegal math operation */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_7       (0x8117) /* Performed an illegal math operation */
#define IFAULT_IR_64BIT_MATH_SUBTRACT_ERROR_8       (0x8118) /* Performed an illegal math operation */

//MetaData
#define IFAULT_IR_LDF_ILLEGAL_CONFIG_ASSIGMENT      (0x8201) /* Assigned a configuration with pending or deferred updates */
//#define IFAULT_IR_FAILED_TO_MODIFY_DEV_MISSING_TO   (0x8202)
//#define IFAULT_IR_LDF_PD_REFRESH                    (0x8203)
#define IFAULT_IR_LDF_PDREF_TO_PDN_CONVERSION       (0x8204) /* Invalid physical disk number */
#define IFAULT_IR_LDF_PDN_TO_PDREF_CONVERSION       (0x8205) /* Invalid physical disk reference */
#define IFAULT_IR_VD_ELM_NOT_FOUND                  (0x8206) /* Null virtual disk element pointer */
#define IFAULT_IR_DUPLICATE_PD_REFERENCE_FOUND      (0x8207) /* Found a duplicate physical disk reference */
#define IFAULT_IR_VD_ELM_NOT_FOUND_CONST            (0x8208) /* Null virtual disk element pointer */
#define IFAULT_IR_PD_ELM_NOT_FOUND                  (0x8209) /* Null physical disk element pointer */
#define IFAULT_IR_PD_ELM_NOT_FOUND_CONST            (0x820A) /* Null physical disk element pointer */
//#define IFAULT_IR_SATA_NO_48BIT_LBA_SUPPORT         (0x820B)
#define IFAULT_IR_LDF_UPDATE_BUFFER_TOO_SMALL       (0x820C) /* LDF buffer is not large enough */
#define IFAULT_IR_LDF_HEADER_OPEN_READ_ONLY         (0x820D) /* LDF configuration not opened as read only */
//#define IFAULT_IR_IO_BUFFER_USAGE_ERROR             (0x820F)
#define IFAULT_IR_LDF_SECTION_CLOSE_WO_HEADER_OPEN  (0x8210) /* Tried to close LDF header that is not open */
#define IFAULT_IR_LDF_SECTION_READ_WO_HEADER_OPEN   (0x8211) /* Tried to read LDF section without opening it */
#define IFAULT_IR_LDF_SECTION_WRITE_WO_HEADER_OPEN  (0x8212) /* Tried to write LDF header without opening it */
#define IFAULT_IR_LDF_INCORECT_STRUCTURE_SIZE       (0x8213) /* LDF structures were generated incorrectly */
#define IFAULT_IR_LDF_INVALID_CONFIG_NUMBER         (0x8214) /* Created an invalid LDF metadata configuration number */
#define IFAULT_IR_LDF_PD_REF_PDE_SEARCH             (0x8215) /* Failed to find physical disk reference */
#define IFAULT_IR_LDF_SEQUENCE_ROLLOVER             (0x8216) /* Sequence number reached 0xFFFFFFFF */
#define IFAULT_IR_LDF_GUID_VDE_SEARCH               (0x8217) /* Virtual Disk elements exceed maximum number supported */
#define IFAULT_IR_VD_NOT_FOUND                      (0x8218) /* Failed to find virtual disk during activation */
#define IFAULT_IR_VD_ELM_LOCALITY_EMPTY             (0x8219) /* Failed to find RAID physical disk configuration page */
#define IFAULT_IR_LDF_CONFIGURATION_NOT_FOUND_1     (0x821A) /* Failed to find correct configuration object */
#define IFAULT_IR_LDF_CONFIGURATION_NOT_FOUND_2     (0x821B) /* Failed to find correct configuration object */
#define IFAULT_IR_LDF_INVALID_PD_STATE              (0x821C) /* A state of ONLINE or UNCONFIGURED was stored in metadata */
#define IFAULT_IR_LDF_GSN_UPDATE_FAILED_PD_STATE    (0x821D) /* Phys Disk is moved to FAILED state and we can not update GSN inline because we were not in any task */

//Volume
#define IFAULT_IR_UNKNOWN_RPDP1                     (0x8301) /* Failed to activate physical disk */
#define IFAULT_IR_MAX_VOLUMES                       (0x8302) /* Current volume number exceeds maximum number of volumes */
#define IFAULT_IR_DEVICE_INVALID                    (0x8303) /* Volume construction failed, device is invalid */
#define IFAULT_IR_FREE_VIRTDH_FAILED                (0x8304) /* Failed to free virtual device handle */
#define IFAULT_IR_ALLOC_VIRTDH_FAILED               (0x8305) /* Failed to allocate virtual device handle */
#define IFAULT_IR_MAP_NEXT_NON_PRIMARY              (0x8306) /* Tried to map a secondary partition */
#define IFAULT_IR_BAD_RAID_MAP                      (0x8307) /* Tried to map a RAID 1 volume */
#define IFAULT_IR_DISK_LOAD_BALANCE_ERROR_1         (0x8308) /* Invalid load balancing value */
#define IFAULT_IR_DISK_LOAD_BALANCE_ERROR_2         (0x8309) /* Invalid load balancing value */
#define IFAULT_IR_INVALID_VOLUME_1                  (0x830A) /* Null next volume pointer */
#define IFAULT_IR_INVALID_VOLUME_2                  (0x830B) /* Null next volume pointer */
#define IFAULT_IR_IO_EXCEEDED_VOL_SIZE_1            (0x8310) /* Tried IO beyond the last LBA */
#define IFAULT_IR_IO_EXCEEDED_VOL_SIZE_2            (0x8311) /* Tried IO beyond the last LBA */
#define IFAULT_IR_IO_EXCEEDED_VOL_SIZE_3            (0x8312) /* Tried IO beyond the last LBA */
#define IFAULT_IR_IO_EXCEEDED_VOL_SIZE_4            (0x8313) /* Tried IO beyond the last LBA */
#define IFAULT_IR_IO_EXCEEDED_VOL_SIZE_5            (0x8314) /* Tried IO beyond the last LBA */

//PhysDisk
#define IFAULT_IR_DEVICE_PROTOCOL_INVALID           (0x8401) /* Failed to determine protocol type of device */
#define IFAULT_IR_DEVICE_COMPATIBLE_ERROR_1         (0x8402) /* Incompatible device type encountered */
#define IFAULT_IR_DEVICE_COMPATIBLE_ERROR_2         (0x8403) /* Incompatible device type encountered */
#define IFAULT_IR_DEVICE_PD_CAPACITY_0              (0x8404) /* RAID Physical Disk Page 0 disk size incorrectly set to 0 for online physical disk */
#define IFAULT_IR_DEVICE_WC_NOT_SET                 (0x8405) /* Write cache not turned off on physical disk */

//Internal IO
#define IFAULT_IR_INVALID_INT_IO_1                  (0x8501) /* Unexpected SCSI IO opcode */
#define IFAULT_IR_INVALID_IO_STATE                  (0x8502) /* Unexpected IO state */
#define IFAULT_IR_INVALID_OPCODE_SMALL              (0x8503) /* Unexpected SCSI IO opcode for 10-byte CDB */
#define IFAULT_IR_INVALID_OPCODE_LARGE              (0x8504) /* Unexpected SCSI IO opcode for 16-byte CDB */
#define IFAULT_IR_NO_CRIT_FRM_AVAIL                 (0x8505) /* No critical frame available */

//Task
#define IFAULT_IR_INVALID_VOL_NUM                   (0x8601) /* Volume number is no longer valid */
#define IFAULT_IR_PURE_VIRTUAL                      (0x8602) /* Compiler found a call to a purely virtual function */
#define IFAULT_IR_RESYNC_TASK_NO_TIMER              (0x8603) /* Could not create timer */
#define IFAULT_IR_ADD_RESYNCH_ENTRY_ERROR           (0x8604) /* Null write journal pointer */
#define IFAULT_IR_RESYNC_MAP_IO_ERROR               (0x8605) /* Null volume pointer */
#define IFAULT_IR_NEXT_RESYNC_ENTRY_ERROR           (0x8606) /* Resynchronization entry is beyond the last LBA */
#define IFAULT_IR_RESYNC_IO_ENGINE_ERROR            (0x8607) /* Tried resynchronization with IOs outstanding */
#define IFAULT_IR_TASK_STACK_OVERFLOW               (0x8608) /* The task stack has overflow */
#define IFAULT_IR_TIMER_HANDLER_ERROR               (0x8609) /* No task associated with timer */
#define IFAULT_IR_RESYNC_INVALID_ENTRY              (0x860A) /* Invalid resynchronization entry */
#define IFAULT_IR_NEW_WJ_ENTRY_ERROR                (0x860B) /* Current write journal entry is already in use */
#define IFAULT_IR_INVALID_RAID_VOLUME_TYPE          (0x860C) /* Invalid RAID volume type */
#define IFAULT_IR_NO_IO_BUFFER_1                    (0x860D) /* Null IO buffer pointer */
#define IFAULT_IR_NO_IO_BUFFER_2                    (0x860E) /* Null IO buffer pointer */
//#define IFAULT_IR_NO_IO_BUFFER_3                    (0x860F) /* Null IO buffer pointer */
#define IFAULT_IR_NO_IO_BUFFER_4                    (0x8610) /* Buffer size smaller than volume block size */
#define IFAULT_IR_NO_IO_BUFFER_5                    (0x8611) /* Null IO buffer pointer */
#define IFAULT_IR_NO_IO_BUFFER_6                    (0x8612) /* Null IO buffer pointer */
#define IFAULT_IR_CONSIST_TASK_NO_TIMER             (0x8613) /* Failed to allocate timer */
#define IFAULT_IR_CONSIST_NEXT_ENTRY_ERROR          (0x8614) /* Entry out of range */
#define IFAULT_IR_CONSIST_MAP_IO_ERROR              (0x8615) /* Null Consistency Check volume pointer */
#define IFAULT_IR_CONSIST_IO_ENGINE_ERROR           (0x8616) /* Outstanding IOs during consistency check */
#define IFAULT_IR_FRJ_INVALID_RESYNC_COLUMN         (0x8617) /* No rebuilding columns during the resynchronization column cache flush  */
#define IFAULT_IR_FRJ_INVALID_VOLUME                (0x8618) /* ptrVolume in the fast resynchronization journal object is invalid */
#define IFAULT_IR_INVALID_WJ_SECTOR_SIZE            (0x8619) /* The number of sectors in the write journal entry is 0xFFFFFFFF */
#define IFAULT_IR_INVALID_FRJ_SECTOR_SIZE           (0x861A) /* The number of sectors in the fast resynchronization journal entry is 0xFFFFFFFF */
#define IFAULT_IR_FRJ_INVALID_SECTOR_SIZE_0         (0x861B) /* The number of sectors in the fast resynchronization journal entry is 0 */
#define IFAULT_IR_CC_MAX_SECTORS_0                  (0x861C) /* The number of sectors in the IO is 0 */
#define IFAULT_IR_CC_INVALID_SECONDARY_DISK         (0x861D) /* Invalid secondary disk pointer */
#define IFAULT_IR_FRJ_EXCEEDED_FRJ_COUNT            (0x861E) /* Exceeded the number of entries allowed in the fast resynchronization journal */

//Event
#define IFAULT_IR_DEVH_NOT_FOUND_IN_CONFIG          (0x8701) /* Device handle not found in configuration */
#define IFAULT_IR_VDEVH_NOT_FOUND_IN_CONFIG         (0x8702) /* Virtual device handle not found in configuration */
#define IFAULT_IR_EVENT_REPLY_FRAME_ALLOC_FAILURE   (0x8703) /* Failed to allocate reply frame for change event */
#define IFAULT_IR_ALLOC_CONFIG_MSG_FAILED           (0x8704) /* Creation of configuration change event failed */
#define IFAULT_IR_ALLOC_CONFIG_EVENT_FAILED         (0x8705) /* Failed to allocate configuration change event */
#define IFAULT_IR_CRIT_NEW_1                        (0x8706) /* Failed to acquire critical frame */
#define IFAULT_IR_CRIT_NEW_2                        (0x8707) /* Failed to acquire critical frame */
#define IFAULT_IR_CRIT_NEW_3                        (0x8708) /* Failed to acquire critical frame */
#define IFAULT_IR_CRIT_NEW_5                        (0x8709) /* Failed to acquire critical frame */

//Memory Management
#define IFAULT_IR_ACQUIRE_MEMORY_ERROR              (0x8801) /* Tried to acquire a buffer that already exists */
#define IFAULT_IR_CRIT_NEW_4                        (0x8802) /* Failed to acquire critical frame */
#define IFAULT_IR_DELETE_MEMORY_IN_USE              (0x8803) /* Tried to delete memory that is in use */
#define IFAULT_IR_INVALID_MEMORY_MOVE               (0x8804) /* Memory move failed */
#define IFAULT_IR_HEAP_CORRUPTION_SIZE_0            (0x8805) /* IR heap pointers corrupted */
#define IFAULT_IR_IO_BUFFER_NEW_FAILURE_1           (0x8806) /* Null buffer pointer  */
#define IFAULT_IR_IO_BUFFER_NEW_FAILURE_2           (0x8807) /* Null buffer pointer */
#define IFAULT_IR_DMA_HEAP_NEW                      (0x8808) /* Failed to acquire DMA-able critical memory */

//Config Page
#define IFAULT_IR_SHARED_CONFIG_PAGE                (0x8901) /* Shared page buffer already in use */
#define IFAULT_IR_NO_FREE_PAGE_BUFFER               (0x8902) /* No free page buffer is available */
#define IFAULT_IR_REMOVE_CONFIG_PAGE_ERROR          (0x8903) /* Tried to remove a configuration page when there are none */
#define IFAULT_IR_CONST_VIOLATION                   (0x8904) /* Null page pointer */


//#define IFAULT_IR_LDF_DELETE_PHYS_DISK_ERROR        (0x8026)
//#define IFAULT_IR_START_SCSI_IO                     (0x8006)
//#define IFAULT_IR_NO_CRIT_FRAMES_AVAILABLE          (0x800B)
//#define IFAULT_IR_INVALID_DEVICE_HANDLE             (0x8015)
//#define IFAULT_IR_RELOCATE_FAILURE                  (0x8016)
//#define IFAULT_IR_INVALID_PTR_PHYS_DISK             (0x8018)
//#define IFAULT_IR_801E                              (0x801E)
//#define IFAULT_IR_ERROR_RETRIEVE_EVENT_DATA         (0x801F)
//#define IFAULT_IR_DEVICE_CONSTRUCT_FAILURE          (0x8020)
//#define IFAULT_IR_8023                              (0x8023)
//#define IFAULT_IR_LDF_DELETE_VIRT_DISK_ERROR        (0x8027)
//#define IFAULT_IR_SEND_EVENT_ERROR                  (0x802B)
//#define IFAULT_IR_INIT_SENSE_BUFFER_ERROR           (0x802C)
//#define IFAULT_IR_GET_LBA_ERROR                     (0x8032)
//#define IFAULT_IR_START_IO_ERROR                    (0x8033)
//#define IFAULT_IR_NEW_DELETE_ERROR                  (0x8035)
//#define IFAULT_IR_DEVICE_EVENT_DISCOVERY_ERROR      (0x8036)
//#define IFAULT_IR_MAX_INTERNAL_DEVICES_ERROR        (0x8037)
//#define IFAULT_IR_8038                              (0x8038)
//#define IFAULT_IR_803A                              (0x803A)
//#define IFAULT_IR_803B                              (0x803B)
//#define IFAULT_IR_LDF_VD_DEV_NUM_INVALID            (0x803E)
//#define IFAULT_IR_CONFIG_PAGE_ERROR                 (0x8043)
//#define IFAULT_IR_LDF_NO_FREE_VD_ELEMENT            (0x8044)
//#define IFAULT_IR_LDF_VD_ELEMENT_BAD_VD_NUM         (0x8045)
//#define IFAULT_IR_LDF_NO_SECTION_BUFFER             (0x8050)
//#define IFAULT_IR_LDF_PDE_LOOKUP                    (0x8051)
//#define IFAULT_IR_LDF_VDE_LOOKUP                    (0x8052)
//#define IFAULT_IR_SET_INVALID_DEVICE_HANDLE         (0x805C)
//#define IFAULT_IR_INVALID_SCSI_MAP                  (0x805D)
//#define IFAULT_IR_LDF_NO_UPDATES_COMPLETED          (0x8062)
//#define IFAULT_IR_INVALID_METADATA_SIZE             (0x8063)
//#define IFAULT_IR_NON_IR_DISK_AT_VOL_ID             (0x8064)
//#define IFAULT_IR_NO_VALID_PATHS_IN_PD_LDF          (0x8065)
//#define IFAULT_IR_SEND_EVENT_ERROR_1                (0x8067)
//#define IFAULT_IR_SEND_EVENT_ERROR_2                (0x8068)
//#define IFAULT_IR_SEND_EVENT_ERROR_3                (0x8069)
//#define IFAULT_IR_INACTIVE_PATH_INDEX_INVALID       (0x806B)
//#define IFAULT_IR_INVALID_MID_OPERATOR              (0x806C)
//#define IFAULT_IR_MSG_FRAME_CONTEXT_NON_ZERO        (0x806D)


/******************************************************************************
* UFF IFAULT code definitions 0xC000 - 0xFFFF
*******************************************************************************/

/* See pliFault.h */

#endif /* end of file */

