/***************************************************************************
 *                                                                         *
 *  Copyright 2003-2012 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pliIocLogInfo.h
 *          Title:  SAS Firmware IOP Interface IOC Log Info Definitions
 *     Programmer:
 *  Creation Date:  November 30, 2007
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface IOC Log Info codes
 *
 *-------------------------------------------------------------------------
 */

#ifndef PLI_IOCLOGINFO_H_INCLUDED
#define PLI_IOCLOGINFO_H_INCLUDED


/****************************************************************************/
/*  IOC LOGINFO defines, 0x00000000 - 0x0FFFFFFF                            */
/*  Format:                                                                 */
/*      Bits 31-28: MPI2_IOCLOGINFO_TYPE_SAS (3)                            */
/*      Bits 27-24: IOC_LOGINFO_ORIGINATOR: 0=IOP, 1=PL, 2=IR               */
/*      Bits 23-16: LOGINFO_CODE                                            */
/*      Bits 15-0:  LOGINFO_CODE Specific                                   */
/****************************************************************************/

/****************************************************************************/
/* IOC_LOGINFO_ORIGINATOR defines                                           */
/****************************************************************************/
#define IOC_LOGINFO_ORIGINATOR_IOP                      (0x00000000)
#define IOC_LOGINFO_ORIGINATOR_PL                       (0x01000000)
#define IOC_LOGINFO_ORIGINATOR_IR                       (0x02000000)

#define IOC_LOGINFO_ORIGINATOR_MASK                     (0x0F000000)

/****************************************************************************/
/* LOGINFO_CODE defines                                                     */
/****************************************************************************/
#define IOC_LOGINFO_CODE_MASK                           (0x00FF0000)
#define IOC_LOGINFO_CODE_SHIFT                          (16)

/****************************************************************************/
/* IOP LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IOP          */
/****************************************************************************/

/* See iopiIopLogInfo.h */

/****************************************************************************/
/* PL LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = PL            */
/****************************************************************************/
#define PL_LOGINFO_CODE_OPEN_FAILURE                                (0x00010000)    /* Open failure: See sub-codes PL_LOGINFO_SUB_CODE_OPEN_FAILURE. */

#define PL_LOGINFO_CODE_SATA_NON_NCQ_RW_ERR_BIT_SET                 (0x00060000)    /* Error bit is set in SATA D->H FIS for non-NCQ, automated read/write. */
#define PL_LOGINFO_CODE_SATA_NCQ_FAIL_ALL_CMDS_AFTR_ERR             (0x00080000)    /* All NCQ I/Os fail when one NCQ I/O encounters an error. */

#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE                         (0x000F0000)    /* Invalid PL configuration page.*/
#define PL_LOGINFO_CODE_CONFIG_PL_NOT_INITIALIZED                   (0x000F0001)    /* PL is not initialized and cannot perform a configuration page request. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_PT                      (0x000F0100)    /* Invalid page type. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NUM_PHYS                (0x000F0200)    /* Invalid number of phys. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NOT_IMP                 (0x000F0300)    /* The page was not handled. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NO_DEV                  (0x000F0400)    /* No device found. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_FORM                    (0x000F0500)    /* Invalid form. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_PHY                     (0x000F0600)    /* Invalid phy. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NO_OWNER                (0x000F0700)    /* No owner found. */

#define PL_LOGINFO_CODE_RESET                                       (0x00110000)    /* Target reset. The firmware ORs this code with a subcode (lower 16 bits) for explanation. For more information, see Table 8.  */
#define PL_LOGINFO_CODE_ABORT                                       (0x00120000)    /* Task abort. The firmware ORs this code with a subcode (lower 16 bits) for explanation. For more information, see Table 8.*/
#define PL_LOGINFO_CODE_IO_NOT_YET_EXECUTED                         (0x00130000)    /* Aborted I/O from a pending queue and is not yet submitted to the hardware. */
#define PL_LOGINFO_CODE_IO_EXECUTED                                 (0x00140000)    /* Aborted I/O is submitted to the hardware. */
#define PL_LOGINFO_CODE_PERS_RESV_OUT_NOT_AFFIL_OWNER               (0x00150000)    /* Affiliation conflict on Persistent Reserve Out command. */
#define PL_LOGINFO_CODE_NOT_AFFIL_OWNER                             (0x00160000)    /* Affiliation conflict. */

#define PL_LOGINFO_CODE_IO_DEVICE_MISSING_DELAY_RETRY               (0x00170000)    /* Device is missing (device missing delay timer option is enabled). When the timer starts, an internal TM cleans up pending I/Os. The I/Os returned to the host should be retried. This log information is also sent when the I/O reply timer expires. */

#define PL_LOGINFO_CODE_TARGET_INVALID_MID                          (0x00180001)    /* An invalid MID was requested in a Target mode abort. */
#define PL_LOGINFO_CODE_TARGET_INVALID_TYPE                         (0x00180002)    /* An invalid abort type during a Target mode abort. */
#define PL_LOGINFO_CODE_TARGET_MODE_ABORT_ALL_IO                    (0x00180003)    /* Target mode abort request with an ABORT_ALL_IO abort type.*/
#define PL_LOGINFO_CODE_TARGET_MODE_ABORT_EXACT_IO                  (0x00180004)    /* Target mode abort request with an ABORT_EXACT_IO abort type. */
#define PL_LOGINFO_CODE_TARGET_MODE_ABORT_EXACT_IO_REQ              (0x00180005)    /* Target mode abort request with an ABORT_EXACT_IO_REQUEST abort type. */
#define PL_LOGINFO_CODE_TARGET_ASSIST_TERMINATED                    (0x00180007)    /* Target assist terminated. */
#define PL_LOGINFO_CODE_TARGET_STATUS_SEND_TERMINATED               (0x00180008)    /* Target status send terminated. */

#define PL_LOGINFO_CODE_SENSE_DATA_LENGTH_ZERO                      (0x00190000)    /* SSP sense data length is 0. */

/* Codes 0x001A0000 to 0x001A0014 are reserved for BT1680 in Gen 2 */
//#define PL_LOGINFO_CODE_BT1680                                      (0x001A0000)    /* GEN2 ONLY! BT1680 condition detected. */
//#define PL_LOGINFO_CODE_BT1680_BREAK_RECEIVED                       (0x001A0001)    /* GEN2 ONLY! BT1680 condition detected. */
//#define PL_LOGINFO_CODE_BT1680_CRC_MISMATCH                         (0x001A0002)    /* GEN2 ONLY! BT1680 reread of the sector resulted in a CRC mismatch. */
//#define PL_LOGINFO_CODE_BT1680_NO_RECON_OPEN                        (0x001A0003)    /* GEN2 ONLY! BT1680 target did not reconnect during BT1680 recovery. */
//#define PL_LOGINFO_CODE_BT1680_NO_SUSPEND                           (0x001A0004)    /* GEN2 ONLY! BT1680 RxEEDP failed to suspend. */
//#define PL_LOGINFO_CODE_BT1680_RESEND_IO_FAILED                     (0x001A0005)    /* GEN2 ONLY! BT1680 failed to reissue the original I/O. */
//#define PL_LOGINFO_CODE_BT1680_RESEND_IO_CONN_STUCK                 (0x001A0006)    /* GEN2 ONLY! BT1680 connection did not close after the I/O reissue. */
//#define PL_LOGINFO_CODE_BT1680_INQ_FAILED                           (0x001A0007)    /* GEN2 ONLY! BT1680 failed to detect the device type. */
//#define PL_LOGINFO_CODE_BT1680_NO_BREAKUP_FRAME                     (0x001A0008)    /* GEN2 ONLY! BT1680 failed to acquire frame for SATA breakup. */
//#define PL_LOGINFO_CODE_BT1680_SATA_SHORT_FRAME                     (0x001A0009)    /* GEN2 ONLY! BT1680 failed because of a short Data frame. */
//#define PL_LOGINFO_CODE_BT1680_SATA_UNDERRUN                        (0x001A000A)    /* GEN2 ONLY! BT1680 failed because of an underrun. */
//#define PL_LOGINFO_CODE_BT1680_SATA_OVERRUN                         (0x001A000B)    /* GEN2 ONLY! BT1680 failed because of an overrun. */
//#define PL_LOGINFO_CODE_BT1680_WRONG_MID_OR_DEV                     (0x001A000C)    /* GEN2 ONLY! BT1680 failed because of a frame from the incorrect device or for the incorrect MID. */
//#define PL_LOGINFO_CODE_BT1680_RETRIES_EXHAUSTED                    (0x001A000D)    /* GEN2 ONLY! BT1680 failed because of excessive retries on a SCSI error. */
//#define PL_LOGINFO_CODE_BT1680_DEVICE_MISSING                       (0x001A000E)    /* GEN2 ONLY! BT1680 failed because of a missing device. */
//#define PL_LOGINFO_CODE_BT1680_CANT_STOP_RXDMA                      (0x001A000F)    /* GEN2 ONLY! BT1680 failed because the RxDMA did not stop quickly enough. */
//#define PL_LOGINFO_CODE_BT1680_RESET_ON_FAILURE                     (0x001A0010)    /* GEN2 ONLY! BT1680 failed, reset on exit. */
//#define PL_LOGINFO_CODE_BT1680_SATA_INIT_FAILURE                    (0x001A0011)    /* GEN2 ONLY! BT1680 failed to run SATA initialization. */
//#define PL_LOGINFO_CODE_BT1680_QUIESCE_FAILED                       (0x001A0012)    /* GEN2 ONLY! BT1680 failed because it could not quiesce in time. */
//#define PL_LOGINFO_CODE_BT1680_SAS_UNDERRUN                         (0x001A0013)    /* GEN2 ONLY! BT1680 failed because of an underrun. */
//#define PL_LOGINFO_CODE_BT1680_SAS_OVERRUN                          (0x001A0014)    /* GEN2 ONLY! BT1680 failed because of an overrun. */
//#define PL_LOGINFO_CODE_BT1680_NO_RECON_DATA                        (0x001A0015)    /* GEN2 ONLY! BT1680 target did not reconnect with data during the BT1680 recovery. */

#define PL_LOGINFO_CODE_TM_INVALID_REQUEST                          (0x001B0000)    /* Invalid TM request; see PL_LOGINFO_SUBCODE_INVALID_TM_NOT_END_DEVICE in Table 8 for more information. */

/* SMP failure codes */
#define PL_LOGINFO_CODE_SMP_FAILED                                  (0x001C0000)    /* The SMP failed. Lower 2 bytes are a PLSTATUS subcode. */
#define PL_LOGINFO_CODE_SMP_FAILED_ABORT                            (0x001C000A)    /* The SMP failed. Abort. */
#define PL_LOGINFO_CODE_SMP_FAILED_XFER_ERROR                       (0x001C000D)    /* The SMP failed. The Tx context frame manager detected a transfer error. */
#define PL_LOGINFO_CODE_SMP_FAILED_TIMEOUT                          (0x001C000E)    /* The SMP failed. A time-out occurred. */
#define PL_LOGINFO_CODE_SMP_FAILED_DMA_ERROR                        (0x001C0014)    /* The SMP failed. The Tx DMA engine detected an error. */
#define PL_LOGINFO_CODE_SMP_FAILED_HW_TIMEOUT                       (0x001C0020)    /* The SMP failed. The link timed out. */
#define PL_LOGINFO_CODE_SMP_FAILED_LINK_ERROR                       (0x001C0021)    /* The SMP failed. An error occurred on the link. */
#define PL_LOGINFO_CODE_SMP_FAILED_BACK_OFF                         (0x001C0030)    /* The SMP failed. An interrupt was raised to back off the transceiver.	NOTE: Description changed. */
#define PL_LOGINFO_CODE_SMP_FAILED_NO_CONNECTION                    (0x001C0031)    /* The SMP failed. No connection available for transceiver. */
#define PL_LOGINFO_CODE_SMP_FAILED_OPEN_FAIL                        (0x001C0100)    /* The SMP failed. Failed to open a connection. Lowest Byte is the specific failure code. */

#define PL_LOGINFO_CODE_POWER_MANAGEMENT                            (0x00340000)    /* Indicates power management error. See Table 9 for PL_LOGINFO_SUB_CODE_PWRMGMT subcodes. */
#define PL_LOGINFO_CODE_POWER_MANAGEMENT_UNKNOWN_STATUS             (0x00348000)    /* Indicates an unknown power management status code in the lower four bits. */

#define PL_LOGINFO_CODE_SMP_REQUEST_TOO_LONG_4_IMMEDIATE            (0x00400000)    /* Immediate mode cannot be used on an SMP request because its length is larger than a message frame. */

/* Note: the intent here for the OPEN_FAILURES is to use PL_LOGINFO_SUB_CODE_OPEN_FAILURE ORed with the OpenFailCode from the port layer */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE                            (0x00000100)    /* Open failure. Lower two nibbles provide explanation. For more information, see code 0x00000101 to code 0x0000011C that follow. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_NO_DEST_TIMEOUT            (0x00000101)    /* Open failure. OPEN_REJECT (no destination) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_PATHWAY_BLOCKED            (0x00000102)    /* Open failure. OPEN REJECT (pathway blocked) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_CONT0                 (0x00000103)    /* Open failure. OPEN REJECT (reserved continue 0) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_CONT1                 (0x00000104)    /* Open failure. OPEN REJECT (reserved continue 1) received.*/
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_INIT0                 (0x00000105)    /* Open failure. OPEN REJECT (reserved initialize 0) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_INIT1                 (0x00000106)    /* Open failure. OPEN REJECT (reserved initialize 1) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_STOP0                 (0x00000107)    /* Open failure. OPEN REJECT (reserved stop 0) received.  */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_STOP1                 (0x00000108)    /* Open failure. OPEN REJECT (reserved stop 1) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ORR                        (0x00000109)    /* Open failure. OPEN REJECT (retry) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_BREAK                      (0x0000010A)    /* Open failure. BREAK received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_TIMEOUT                    (0x0000010C)    /* Open failure. Open time-out. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_CANCELCON                  (0x0000010D)    /* Open failure. PortCancelCon received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_DEV_DOWN                   (0x0000010E)    /* Open failure. Device table down.  */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_BAD_DEST                   (0x00000111)    /* Open failure. OPEN REJECT (bad destination) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RATE_NOT_SUPPORTED         (0x00000112)    /* Open failure. OPEN REJECT (connection rate not supported) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_PROTOCOL_NOT_SUPPORTED     (0x00000113)    /* Open failure. OPEN REJECT (protocol not supported) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ZONE_VIOLATION             (0x00000114)    /* Open failure. OPEN REJECT (zone violation) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ABANDON1                   (0x00000115)    /* Open failure. OPEN REJECT (reserved abandon 1) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ABANDON2                   (0x00000116)    /* Open failure. OPEN REJECT (reserved abandon 2) received.*/
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ABANDON3                   (0x00000117)    /* Open failure. OPEN REJECT (reserved abandon 3) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_STP_RESOURCES_BSY          (0x00000118)    /* Open failure. OPEN REJECT (STP resources busy) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_WRONG_DEST                 (0x00000119)    /* Open failure. OPEN REJECT (wrong destination) received.*/
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ORR_TIMEOUT                (0x0000011A)    /* Open failure. OPEN REJECT (retry) time-out.  */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_AWT_MAXED                  (0x0000011C)    /* Open failure. Arbitration wait timer expired. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_REQ_PORT_DOWN              (0x000001FF)    /* Open failure. Request sent to a port where all the phys are down. */

#define PL_LOGINFO_SUB_CODE_TARGET_BUS_RESET                        (0x00000120)    /* Received HARD_RESET primitive in Target mode. */

#define PL_LOGINFO_SUB_CODE_UART_TEST                               (0x00000140)    /* I/O Aborted by UART command. */

#define PL_LOGINFO_SUB_CODE_TX_TRANSPORT_LAYER                      (0x00000300)  /* The lower nibble (1-f) is reserved for specific errors. For more information, see code 0x00000301 to code 0x00000322 that follow. */
#define PL_LOGINFO_SUB_CODE_TX_CONNECTED                            (0x00000301)  /* Subcode for TxTransportLayer. TxCtx connected. */
#define PL_LOGINFO_SUB_CODE_TX_CONNECTED_ACK                        (0x00000302)  /* Subcode for TxTransportLayer. TxCtx connected ACK. */
#define PL_LOGINFO_SUB_CODE_TX_NAK                                  (0x00000303)  /* Subcode for TxTransportLayer. TxCtx NAK.   */
#define PL_LOGINFO_SUB_CODE_TX_SMP_BACKOFF                          (0x00000304)  /* Subcode for TxTransportLayer. TxCtx SMP back off. */
#define PL_LOGINFO_SUB_CODE_TX_WRG_REL_OFF                          (0x00000305)  /* Subcode for TxTransportLayer. TxCtx WrgRelOff. */
#define PL_LOGINFO_SUB_CODE_TX_BACK_OFF                             (0x00000306)  /* Subcode for TxTransportLayer. TxCtx back off.  */
#define PL_LOGINFO_SUB_CODE_TX_ABORT_FM                             (0x00000307)  /* Subcode for TxTransportLayer. TxCtx abort FM. */
#define PL_LOGINFO_SUB_CODE_TX_T_LEN_GT_DL                          (0x00000308)  /* Subcode for TxTransportLayer. TxCtx T LEN is greater than the data length. */
#define PL_LOGINFO_SUB_CODE_TX_WDL_GT_DL                            (0x00000309)  /* Subcode for TxTransportLayer. TxCtx WDL is greater than the data length. */
#define PL_LOGINFO_SUB_CODE_TX_EEDP                                 (0x0000030A)  /* Subcode for TxTransportLayer. TxCtx EEDP error, the IOC status indicates which type. */
#define PL_LOGINFO_SUB_CODE_TX_DISCONNECTION                        (0x00000310)  /* Subcode for TxTransportLayer. TxFM disconnected. */
#define PL_LOGINFO_SUB_CODE_TX_NAK_RCVD                             (0x00000311)  /* Subcode for TxTransportLayer. TxFM NAK received. */
#define PL_LOGINFO_SUB_CODE_TX_FM_ABORT_FM                          (0x00000312)  /* Subcode for TxTransportLayer. TxFM abort FM. */
#define PL_LOGINFO_SUB_CODE_TX_BAD_SGE                              (0x00000320)  /* Subcode for TxTransportLayer. TxDMA bad SGE fetch. */
#define PL_LOGINFO_SUB_CODE_TX_FETCH_ERROR                          (0x00000321)  /* Subcode for TxTransportLayer. TxDMA fetch error. */
#define PL_LOGINFO_SUB_CODE_TX_SYS_DMA                              (0x00000322)  /* Subcode for TxTransportLayer. TxDMA SYS DMA error. */
#define PL_LOGINFO_SUB_CODE_TX_SNC_COLLATERAL_IO                    (0x00000323)  /* Subcode for TxTransportLayer. SnC Method collateral failed I/O. */

#define PL_LOGINFO_SUB_CODE_RX_TRANSPORT_LAYER                      (0x00000430)  /* The lower nibble (1-f) is reserved for specific errors. For more information, see code 0x00000431 to code 0x0000043F that follow. */
#define PL_LOGINFO_SUB_CODE_RX_WRONG_REL_OFFSET                     (0x00000431)  /* Subcode for RxTransportLayer. Data offset error. */
#define PL_LOGINFO_SUB_CODE_RX_FRAME_LENGTH                         (0x00000432)  /* Subcode for RxTransportLayer. Frame length mismatch. */
#define PL_LOGINFO_SUB_CODE_RX_ZERO_WDL                             (0x00000433)  /* Subcode for RxTransportLayer. Zero WDL in Xfer Rdy. */
#define PL_LOGINFO_SUB_CODE_RX_OVERRUN                              (0x00000434)  /* Subcode for RxTransportLayer. Overrun. */
#define PL_LOGINFO_SUB_CODE_RX_CDP_FRAME                            (0x00000435)  /* Subcode for RxTransportLayer. CDP frame received. */
#define PL_LOGINFO_SUB_CODE_RX_FRAME_SCHED_ERR                      (0x00000436)  /* Subcode for RxTransportLayer. Frame schedule error. */
#define PL_LOGINFO_SUB_CODE_RX_CUR_FRAME_DENY                       (0x00000437)  /* Subcode for RxTransportLayer. Current frame deny. */
#define PL_LOGINFO_SUB_CODE_RX_CTX_WRONG_FRAME_TYPE                 (0x00000438)  /* Subcode for RxTransportLayer. Wrong frame type. */
#define PL_LOGINFO_SUB_CODE_RETRANSMIT_XFER_RDY                     (0x00000439)  /* Subcode for RxTransportLayer. Retransmit bit set in XFER_RDY. */
#define PL_LOGINFO_SUB_CODE_RX_DATA_FRM_OVERRUN                     (0x0000043A)  /* Subcode for RxTransportLayer. FPDMA overrun (also occurs for SAS). */
#define PL_LOGINFO_SUB_CODE_RX_EEDP                                 (0x0000043B)  /* Subcode for RxTransportLayer. EEDP error, the IOC status indicates which type. */
#define PL_LOGINFO_SUB_CODE_RX_EEDP_RESET                           (0x0000043C)  /* Subcode for RxTransportLayer. EEDP error, the IOC status indicates which type. */
#define PL_LOGINFO_SUB_CODE_RX_BAD_SGE                              (0x0000043D)  /* Subcode for RxTransportLayer. Bad SGE fetch. */
#define PL_LOGINFO_SUB_CODE_RX_FETCH_ERROR                          (0x0000043E)  /* Subcode for RxTransportLayer. Fetch error. */
#define PL_LOGINFO_SUB_CODE_RX_SYS_DMA                              (0x0000043F)  /* Subcode for RxTransportLayer. SYS DMA error. */
#define PL_LOGINFO_SUB_CODE_RX_WRONG_DATA_XFER                      (0x00000440)  /* Subcode for RxTransportLayer. Wrong Data Xfer. */
#define PL_LOGINFO_SUB_CODE_RX_FINAL_EEDP_FRAME                     (0x00000441)  /* Subcode for RxTransportLayer. Final Eedp Frame. */
#define PL_LOGINFO_SUB_CODE_RX_DMA_OVERFLOW                         (0x00000442)  /* Subcode for RxTransportLayer. RxDma Overflow. */

#define PL_LOGINFO_SUB_CODE_INTERNAL_IO_TIMED_OUT                   (0x00000500)  /* Internal SCSI I/O timed out. */

#define PL_LOGINFO_SUB_CODE_SATA_READ_LOG_RECEIVE_DATA_ERR          (0x00000610)  /* Error receiving data on the ReadLogExt SATA command. */
#define PL_LOGINFO_SUB_CODE_SATA_ERR_IN_RCV_SET_DEV_BIT_FIS         (0x00000630)  /* Error in the Set Device Bits FIS for the ReadLogExt SATA command. */
#define PL_LOGINFO_SUB_CODE_RX_FM_CURRENT_FRAME_ERROR               (0x00000B10)  /* RxFM current frame error. */
#define PL_LOGINFO_SUB_CODE_RX_FM_INVALID_TAG_DMA_SETUP_FIS         (0x00000B41)  /* RxFM invalid tag in the FPDMA FIS error. */
#define PL_LOGINFO_SUB_CODE_RX_FM_NCQ_DISABLED                      (0x00000B50)  /* Received FPDMA or Set Device Bits FIS, but NCQ is disabled. */
#define PL_LOGINFO_SUB_CODE_RX_FM_INVALID_TAG_SET_DEVICE_BITS       (0x00000B42)  /* RxFM Invalid tag in the SDB FIS error. */
#define PL_LOGINFO_SUB_CODE_RX_FM_WRONG_DATA_DIRECTION              (0x00000B60)  /* RxFM Wrong Data Direction. */
#define PL_LOGINFO_SUB_CODE_SATA_LINK_DOWN                          (0x00000D00)  /* SATA direct-attached link went down. */
#define PL_LOGINFO_SUB_CODE_ABORT_SATA_RX_STAT                      (0x00000D01)  /* SATA receive transfer aborted. */
#define PL_LOGINFO_SUB_CODE_SATA_UNEXPECTED_FRAME                   (0x00000D02)  /* Received an unexpected SATA FIS. */
#define PL_LOGINFO_SUB_CODE_DISCOVERY_SATA_INIT_W_IOS               (0x00000E00)  /* SATA initialization started with outstanding I/Os. */
#define PL_LOGINFO_SUB_CODE_DISCOVERY_REMOTE_SEP_RESET              (0x00000E01)  /* Remote SEP requires reset. */
#define PL_LOGINFO_SUB_CODE_RCVD_UNCONFIRMED_RESP_FRAME             (0x00000E02)  /* Received unconfirmed Response frame. */
#define PL_LOGINFO_SUB_CODE_DISCOVERY_SATA_INIT_NEEDED_W_IOS        (0x00000E03)  /* SATA initialization started with outstanding I/Os. */
#define PL_LOGINFO_SUB_CODE_INVALID_TM_NOT_END_DEVICE               (0x00000E80)  /* TM sent to a devhandle that is not an end device. */

/* SUB Codes for stuck link conditions (0xF00-0xF10). */
#define PL_LOGINFO_SUB_CODE_LINK_HUNG                               (0x00000F00)  /* TM Sent to a devhandle connected to a stuck link. */

#define PL_LOGINFO_SUB_CODE_PWRMGMT_DEVICE_NOT_CONNECTED            (0x00000F22)  /* A current phy power condition change request was denied because no device is attached to the specified phy. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_IOC_PARTIAL_SLUMBER_DISABLED    (0x00000F24)  /* A current phy power condition change request was denied because power management is disabled for the specified phy. Check that the appropriate bits are asserted in the SAS I/O Unit Page 5 ControlFlags field that corresponds to the specified phy. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_DRIVE_PARTIAL_SLUMBER_DISABLED  (0x00000F26)  /* A current phy power condition change request was denied because the end device does not support power management. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_STATE_PROHIBITS_PHY_POWER_MODE  (0x00000F2A)  /* A current phy power condition change request was denied because the request would transition the phy directly from Partial mode to Slumber mode, or vice-versa. To enter the desired state, first wake up the phy by changing to ACTIVE phy power condition. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_HW_BUSY_RETRY_REQ_LATER         (0x00000F2C)  /* A current phy power condition change request failed because the IOC or the drive transmitted a frame before the transition could complete. Retry the request at a later time. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_QUEUE_FULL_RETRY_REQ_LATER      (0x00000F2E)  /* A current phy power condition change request failed because the firmware's policy change request queue is full. Retry the request at a later time. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_UNDEFINED_STATUS_CODE           (0x00000F30)  /* A power management policy change request failed because of an unknown error for which no suitable translation exists. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_SLUMBER_WAKEUP_FAILED           (0x00000F40)  /* A wakeup from Slumber mode failed due to a hardware configuration issue with the IOC's GigaBlaze(R) transceiver. An internal task management task aborts any outstanding I/Os, resets the phy and sets a flag to prevent future transitions into Slumber mode. */
//#define PL_LOGINFO_SUB_CODE_PWRMGMT_PARTIAL_TRANSITION_FAILED       (0x00000F50)  /* A wakeup from Partial mode failed due to a hardware issue (BT1656). The firmware initiates an internal management task that aborts any outstanding I/Os, resets the phy, and sets a flag to prevent future transitions into Partial power management mode. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_PORT_WIDTH_MOD_DISALLOWED       (0x00000F60)  /* A request to control the port width modulation of a group failed because the group cannot be controlled at this time. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_HOST_IS_NOT_CONTROLLING_GROUP   (0x00000F62)  /* A host request to relinquish control of a port width modulation group failed because the host was not previously in control. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_NEW_AND_OLD_REGIONS_THE_SAME    (0x00000F63)  /* A host request to alter a port's modulation failed because the requested modulation level is the same as the current level (refer to SAS I/O Unit Page 6). */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_GROW_OPERATION_FAILED           (0x00000F64)  /* A host-initiated grow request failed because the phys did not properly link up. */

#define PL_LOGINFO_SUB_CODE_DSCVRY_SATA_INIT_TIMEOUT                (0x00001000)  /* SATA initialization time-out. */

#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_FRAME_FAILURE                 (0x00200000)  /* Cannot acquire an SMP frame. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_READ_ERROR                    (0x00200010)  /* Error occurred on an SMP read. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_WRITE_ERROR                   (0x00200020)  /* Error occurred on an SMP write. */
#define PL_LOGINFO_CODE_ENCL_MGMT_NOT_SUPPORTED_ON_ENCL             (0x00200040)  /* Enclosure management services are not available for this WWID. */
#define PL_LOGINFO_CODE_ENCL_MGMT_ADDR_MODE_NOT_SUPPORTED           (0x00200050)  /* Address mode is not supported. */
#define PL_LOGINFO_CODE_ENCL_MGMT_BAD_SLOT_NUM                      (0x00200060)  /* SEP message contains an invalid slot number. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SGPIO_NOT_PRESENT                 (0x00200070)  /* SGPIO not present or enabled. */
#define PL_LOGINFO_CODE_ENCL_MGMT_GPIO_NOT_CONFIGURED               (0x00200080)  /* Unconfigured GPIO. */
#define PL_LOGINFO_CODE_ENCL_MGMT_GPIO_FRAME_ERROR                  (0x00200090)  /* GPIO cannot allocate a frame. */
#define PL_LOGINFO_CODE_ENCL_MGMT_GPIO_CONFIG_PAGE_ERROR            (0x002000A0)  /* GPIO failed configuration page request. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SES_FRAME_ALLOC_ERROR             (0x002000B0)  /* Cannot get a frame for the SES command. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SES_IO_ERROR                      (0x002000C0)  /* I/O execution error. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SES_RETRIES_EXHAUSTED             (0x002000D0)  /* SEP I/O retries exhausted. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_FRAME_ALLOC_ERROR             (0x002000E0)  /* Cannot get a frame for the SMP command. */
#define PL_LOGINFO_CODE_ENCL_MGMT_UNAUTHORIZED_DEVICE               (0x002000F0)  /* Slot has an unauthorized device. */
                                                                                  
#define PL_LOGINFO_DA_SEP_NOT_PRESENT                               (0x00200100)  /* SEP not present when the message was received. */
#define PL_LOGINFO_DA_SEP_SINGLE_THREAD_ERROR                       (0x00200101)  /* Can only accept one message at a time. */
#define PL_LOGINFO_DA_SEP_ISTWI_INTR_IN_IDLE_STATE                  (0x00200102)  /* ISTWI interrupt received when idle. */
//#define PL_LOGINFO_DA_SEP_RECEIVED_NACK_FROM_SLAVE                (0x00200103)  /* FIRMWARE received NAK from SEP.  SEP is busy */
#define PL_LOGINFO_DA_SEP_DID_NOT_RECEIVE_ACK                       (0x00200104)  /* SEP did not receive ACK (last received bit is 1). */
#define PL_LOGINFO_DA_SEP_BAD_STATUS_HDR_CHKSUM                     (0x00200105)  /* SEP stopped or sent a bad checksum in the header. */
#define PL_LOGINFO_DA_SEP_STOP_ON_DATA                              (0x00200106)  /* SEP stopped when transferring data. */
#define PL_LOGINFO_DA_SEP_STOP_ON_SENSE_DATA                        (0x00200107)  /* SEP stopped when transferring sense data. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_SCSI_STATUS_1                 (0x00200108)  /* SEP returned an unknown SCSI status. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_SCSI_STATUS_2                 (0x00200109)  /* SEP returned an unknown SCSI status. */
#define PL_LOGINFO_DA_SEP_CHKSUM_ERROR_AFTER_STOP                   (0x0020010A)  /* SEP returned a bad checksum after a stop. */
#define PL_LOGINFO_DA_SEP_CHKSUM_ERROR_AFTER_STOP_GETDATA           (0x0020010B)  /* SEP returned a bad checksum after a stop while getting data. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_COMMAND                       (0x0020010C)  /* SEP does not support CDB opcode firmware location 1. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_COMMAND_2                     (0x0020010D)  /* SEP does not support CDB opcode firmware location 2. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_COMMAND_3                     (0x0020010E)  /* SEP does not support CDB opcode firmware location 3. */
                                                                                  
#if COMPILE_OPTION_SAS_PORT_VIRT                                                  
#define PL_LOGINFO_CODE_VEM_VIRT_PHYS_NOT_AVAILABLE                 (0x00210001)  /* A virtual port create request failed because free virtual phys were not available for assignment to the virtual port. */
#define PL_LOGINFO_CODE_VEM_VP_RESOURCE_NOT_AVAILABLE               (0x00210002)  /* A virtual port create request failed because a free virtual port resource was not available. */
#define PL_LOGINFO_CODE_VEM_HW_RESOURCE_NOT_AVAILABLE               (0x00210003)  /* A virtual port create request failed because an entry for the virtual port allocated could not be added to the Hardware Initiator or Virtual ID Response table. */
#define PL_LOGINFO_CODE_VEM_PHYSICAL_PORT_NOT_FUNCTIONAL            (0x00210004)  /* A virtual port create request failed because the physical port specified in the request is not functional. */
#define PL_LOGINFO_CODE_VEM_VP_DELETE_FAILED_VP_NOT_VALID           (0x00210005)  /* A virtual port delete request failed because the virtual port to be deleted is invalid. */
#define PL_LOGINFO_CODE_VEM_VP_DELETE_FAILED_UNALLOCATED_VP         (0x00210006)  /* A virtual port delete request failed because the virtual port to be deleted has not been allocated yet. */
#define PL_LOGINFO_CODE_VEM_VP_DELETE_FAILED_OUTSTAND_IOS           (0x00210007)  /* A virtual port delete request failed because the virtual port to be deleted has outstanding I/Os. */
#define PL_LOGINFO_CODE_VEM_VP_DELETE_FAILED_OUTSTAND_TMS           (0x00210008)  /* A virtual port delete request failed because the virtual port to be deleted has outstanding task managements. */
#define PL_LOGINFO_CODE_VEM_VP_CREATE_INVALID_PPORT                 (0x00210009)  /* A virtual port create request failed because the physical port specified in the request is invalid. */
#define PL_LOGINFO_CODE_VEM_VP_CREATE_INVALID_VE                    (0x0021000A)  /* A virtual port create request failed because the physical port specified in the request does not have a virtual expander associated. */
#endif

#define PL_LOGINFO_CODE_IO_REDUCED_FUNCTIONALITY_RETRY              (0x00220000)   /* I/O aborted for the device behind an expander in the reduced functionality state. */
#define PL_LOGINFO_CODE_IO_QUIESCED_BT1680_RETRY                    (0x00220001)   /* I/O aborted for the device in the BT1680 quiesce I/O state. */

#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY                        (0x00230000)    /* Base host-based discovery error. See the following table for specific failures. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_OPERATION_INVALID      (0x00230001)    /* An MPI HBD action message failed because the operation field contained an invalid value. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEVH_OUT_OF_RANGE      (0x00230002)    /* An MPI HBD action message failed because the device handle specified was not within the allowable range and is, therefore, invalid. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEVH_IN_USE            (0x00230003)    /* An MPI HBD action message failed because of an attempt to allocate an already established device handle. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEVH_NOT_ALLOC         (0x00230004)    /* An MPI HBD action message failed because of an attempt to update or remove a nonexistent device handle. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_PARENT_INVALID         (0x00230005)    /* An MPI HBD action message failed because of an attempt to add or update a device handle whose parent does not exist. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEV_INFO_INVALID       (0x00230006)    /* An MPI HBD action message failed because of an attempt to add a device with invalid HBDDeviceInfo parameters. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_PORT_INVALID           (0x00230007)    /* An MPI HBD action message failed because of an attempt to add a device to an nonexistent port. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_MAX_RATE_INVALID       (0x00230008)    /* An MPI HBD action message failed because of an attempt to add or update a device with a MaxRate value not equal to 0x8, 0x9, or 0xA. */

#if INVADER
#define PL_LOGINFO_FAST_PATH_ENGINE                                 (0x00240000)    /* FPE - Generic Fast Path Engine LogInfo. */
#define PL_LOGINFO_FAST_PATH_ENGINE_IOEXCP_INVALID_DESCR_TYPE       (0x00240001)    /* FPE - Received I/O Exception from Cuda for Mid with Invalid Descriptor Type. */
#define PL_LOGINFO_FAST_PATH_ENGINE_IOEXCP_INVALID_DEVICE_HANDLE    (0x00240002)    /* FPE - Received I/O Exception from Cuda for Mid with Invalid Device Handle. */
#define PL_LOGINFO_FAST_PATH_ENGINE_IOEXCP_VF_MISMATCH              (0x00240003)    /* FPE - Received I/O Exception from Cuda for Mid with VF Mismatch. */
#define PL_LOGINFO_FAST_PATH_ENGINE_REQUESTOR_TIMER_OVERFLOW_ERROR  (0x00240004)    /* FPE - The FPE Requestor has specifed an Add Timer operation with an Add Timeout Value that corresponds to a Timer bucket which is equal to 0xFFFF.*/
#define PL_LOGINFO_FAST_PATH_ENGINE_COMPLETOR_TIMER_OVERFLOW_ERROR  (0x00240005)    /* FPE - The FPE Completor has specifed a Remove Timer operation with a Remove Timer that corresponds to a Timer bucket which is equal to 0x0000.*/
#endif

/****************************************************************************/
/* IR LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IR            */
/****************************************************************************/

/* See iopiIocLogInfo.h */

/****************************************************************************/
/* Defines for convenience                                                  */
/****************************************************************************/
#define IOC_LOGINFO_PREFIX_IOP      ((MPI2_IOCLOGINFO_TYPE_SAS << MPI2_IOCLOGINFO_TYPE_SHIFT) | IOC_LOGINFO_ORIGINATOR_IOP)
#define IOC_LOGINFO_PREFIX_PL       ((MPI2_IOCLOGINFO_TYPE_SAS << MPI2_IOCLOGINFO_TYPE_SHIFT) | IOC_LOGINFO_ORIGINATOR_PL)
#define IOC_LOGINFO_PREFIX_IR       ((MPI2_IOCLOGINFO_TYPE_SAS << MPI2_IOCLOGINFO_TYPE_SHIFT) | IOC_LOGINFO_ORIGINATOR_IR)

#endif /* end of file */

