/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopEngineDrv.h
 *          Title:  IOP Engine Driver Header File
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  August 16, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  08/16/11  NB    Initial version.
 *
 *
 * Description
 * ------------
 * This file contains the definitions used for IOP Engine driver as
 * an interface.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOP_ENGINE_DRV_H
#define IOP_ENGINE_DRV_H

#include <linux/module.h>

#include "basetype.h" /* Base Types */

/* IOP Engine Driver - Status */
typedef enum _IOP_STATUS
{
    IOP_SUCCESS = 0,
    IOP_FAILURE

} IOP_STATUS;

/* Common Interface - IOP */
typedef struct _IOP_ENGINE_DRIVER
{
    ISTATUS (*iopiConfig)              (U32, U32, U32, U32, U32, PU32,
                                        PTR_PAGE_IOC *);
    ISTATUS (*iopiConfigComplete)      (ISTATUS, U32, U32, void *, U32);
    ISTATUS (*iopiDiscoveryComplete)   (IOPI_SUBSYSTEM);
    ISTATUS (*iopiEvent)               (PTR_MPI2_EVENT_NOTIFICATION_REPLY, U32,
                                        IOPI_SUBSYSTEM, U32);
    ISTATUS (*iopiFrameOwned)          (IOPI_SUBSYSTEM, U32);
    ISTATUS (*iopiFreeFrame)           (PTR_MSG_IOC, U32);
    ISTATUS (*iopiGetFrame)            (IOPI_SUBSYSTEM, PTR_MSG_IOC *, PU32);
    ISTATUS (*iopiGetFrameCritical)    (IOPI_SUBSYSTEM,PTR_MSG_IOC *, U32 *,
                                        U32, U32);
    U32     (*iopiGetMsgPoolBaseAddress) (void);
    ISTATUS (*iopiGetRtConfig)         (PTR_RUNTIME_CONFIG RtConfig);
    ISTATUS (*iopiGPIORead)            (U8, PU8);
    ISTATUS (*iopiGPIOWrite)           (U8, U8);
    ISTATUS (*iopiIstwiIORequest)      (PTR_MSG_IOC, U32, IOPI_SUBSYSTEM);
    ISTATUS (*iopiIstwiReleaseBus)     (IOPI_SUBSYSTEM, U8);
    ISTATUS (*iopiIstwiReserveBus)     (IOPI_SUBSYSTEM, void (void), U8, U32);
    ISTATUS (*iopiIstwiReset)          (IOPI_SUBSYSTEM, U32);
    ISTATUS (*iopiMemCmp)              (const void *, const void *, U32);
    ISTATUS (*iopiMemoryMove)          (U32, PTR_MPI2_SGE_IO_UNION, void *, U32,
                                        U32, PU32, U32);
    ISTATUS (*iopiMsgComplete)         (PTR_MSG_IOC, U32, PLI_DEV_HANDLE, U32,
                                        IOPI_SUBSYSTEM);
    ISTATUS (*iopiOutOfResource)       (IOPI_SUBSYSTEM, IOPI_RESOURCE_TYPE);
    ISTATUS (*iopiPortEnable)          (PTR_MSG_IOC PtrMsg);
    ISTATUS (*iopiSataPassthrough)     (U32 Mid, PTR_MSG_IOC PtrMsg);
    ISTATUS (*iopiSMPPassthrough)      (U32 Mid, PTR_MSG_IOC PtrMsg);
    ISTATUS (*iopiSCSIIOComplete)      (PTR_MSG_IOC, U32, PLI_DEV_HANDLE, U32);
    ISTATUS (*iopiSCSIIOCompleteError) (PTR_MSG_IOC, U32, PLI_DEV_HANDLE, U8,
                                        U8, U8, U16, U32, U32, U32, U32, U32, U32);
    ISTATUS (*iopiSetParams)           (U8 IocNum, PTR_PLI_SET_PARAMS PlSetParams);
    ISTATUS (*iopiStartScsiIO)         (PTR_MSG_IOC, U32, PLI_DEV_HANDLE,
                                        MSG_IOC const *);
    ISTATUS (*iopiTargetIoComplete)    (PTR_MSG_IOC, U32);
    ISTATUS (*iopiTaskMgmt)            (PTR_MSG_IOC, PLI_DEV_HANDLE, IOPI_SUBSYSTEM);
    ISTATUS (*iopiTimerCfgFnct)        (IOP_TIMER_HANDLE, void (void*), void*);
    ISTATUS (*iopiTimerChk)            (IOP_TIMER_HANDLE, U32 *);
    ISTATUS (*iopiTimerCreate)         (IOPI_SUBSYSTEM, IOP_TIMER_HANDLE*,
                                        void (void*), void*, U32);
    ISTATUS (*iopiTimerFree)           (IOP_TIMER_HANDLE);
    ISTATUS (*iopiTimerSet)            (IOP_TIMER_HANDLE, U32);
    ISTATUS (*iopiTimerStop)           (IOP_TIMER_HANDLE);
    U16     (*iopiGetMaxDevHandle)     (void);
    void    (*iopiFault)               (U32);
    void    *(*iopiMemCpy)             (const void *, const void *, U32);
    void    *(*iopiMemSet)             (const void *, U32, U32);
    void    *(*iopiIORemap)            (U32 High, U32 Low, U32 Size);
    void    (*iopiIOUnmap)             (void * Addr);
    void    (*iopiTaskMgmtComplete)    (PTR_MSG_IOC,  U32, U16, U32, U32, U32);
    void    (*iopiDaSepReset)          (U32);
    void    (*iopiDelayUSec)           (U32);
    void    (*iopiDiagCmdReadMem)      (U8 *, U32);
    void    (*iopiDiagConsoleRegister) (char *, PTR_DIAG_COMMAND_TABLE, char *);
    void    (*iopiDiagConsolePrint0)   (char *);
    void    (*iopiDiagConsolePrint1)   (char *, U32);
    void    (*iopiDiagConsolePrint2)   (char *, U32, U32);
    void    (*iopiDiagConsolePrint3)   (char *, U32, U32, U32);
    void    (*iopiDiagConsolePrint4)   (char *, U32, U32, U32, U32);
    void    (*iopiTracePrint0)         (long long, U32);
    void    (*iopiTracePrint1)         (long long, U32, U32);
    void    (*iopiTracePrint2)         (long long, U32, U32, U32);
    void    (*iopiTracePrint3)         (long long, U32, U32, U32, U32);
    void    (*iopiTracePrint4)         (long long, U32, U32, U32, U32, U32);
    void    (*iopiTargetProcessCompletions)     (void);
    ISTATUS (*iopiTargetSendCommandBufferReply) (MPI2_TARGET_COMMAND_BUFFER_REPLY_DESCRIPTOR);
    ISTATUS (*iopiGetTimeSinceStartOfDayUsec)   (PU64);
    ISTATUS (*iopiTargetIoCompleteError)        (PTR_MSG_IOC, U32, U16, U32, U16,
                                                 U32, U32);
    ISTATUS (*iopiTimedOutIOFound)              (U32, PLI_DEV_HANDLE);
    ISTATUS (*iopiTimeoutNotification)          (U32, U32);
    ISTATUS (*iopiFPEHandleException)           (PTR_MSG_IOC, U32, U32);
    ISTATUS (*iopiRequestChangeFrameOwner)      (IOPI_SUBSYSTEM, U32);
    ISTATUS (*iopiExtendedBufferDataDump)       (U8, PU8, U32);
    ISTATUS (*iopiExtendedBufferRelease)        (U8);
    void    (*iopMsgHandlerCmdBuffBase)         (U32, U32, PTR_MSG_IOC);
    void    (*iopMsgHandlerCmdBuffList)         (U32, U32, PTR_MSG_IOC);
    void    (*iopMsgHandlerTargetAssist)        (U32, U32, PTR_MSG_IOC);
    void    (*iopMsgHandlerTargetStatusSend)    (U32, U32, PTR_MSG_IOC);
    void    (*iopMsgHandlerTargetModeAbort)     (U32, U32, PTR_MSG_IOC);
} IOP_ENGINE_DRIVER, *PTR_IOP_ENGINE_DRIVER;
extern IOP_ENGINE_DRIVER gIopEngineDriver;

/* Exported Functions - IOP Engine Driver Functions */
extern IOP_STATUS iopEngineRegisterDriver(ENGINE_DRIVER EngineDriver);
extern IOP_STATUS iopEngineUnregisterDriver(U32 Major);

#endif /* IOP_ENGINE_DRV_H */
