/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopPrint.h
 *          Title:  Print Module Header
 *     Programmer:  Thibash Rajamani
 *    Modified by:  Naresh Bhalala
 *  Creation Date:  04 Apr 2012
 *
 *  Version History
 *  ---------------
 *  Date       Who  Description
 *  ---------  ---  -----------------------------------------------------------
 *  04/04/12   TR   Initial Version
 *  05/25/12   NB   Changes for UART module
 *  06/01/12   NB   Changes for Flash module
 *  06/04/12   NB   Changes for I2C module
 *  06/07/12   NB   Changes for RAID module
 *  06/09/12   NB   Changes for ETHERNET module
 *  06/09/12   NB   Changes for GPIO module
 *
 * Description:
 * ------------
 * This include file contains the defines and macros for the print module.
 *
 * ----------------------------------------------------------------------------
 */

#ifndef IOP_MODULE_PRINT_HEADER
#define IOP_MODULE_PRINT_HEADER

#include "iopDebug.h"   /* IOP  String Index defines      */
#include "onfiDebug.h"  /* ONFI String Index defines      */
#include "plDebug.h"    /* PL   String Index defines      */
#include "uartDebug.h"  /* UART String Index defines      */
#include "flashDebug.h" /* FLASH String Index defines     */
#include "i2cDebug.h"   /* I2C String Index defines       */
#include "raidDebug.h"  /* RAID String Index defines      */
#include "ethDebug.h"   /* ETHERNET String Index defines  */
#include "gpioDebug.h"  /* GPIO String Index defines      */
#include "usbDebug.h"   /* USB String Index defines       */
#include "iocDebug.h"   /* IOC String Index defines       */

#define PRINT_TO_UART                       (0)   /* Print the debug info to console */

#if PRINT_TO_UART
#define PRINTK(format, args...)  printk(KERN_EMERG "SDK %s: "format"\n", __func__, ## args)
#else
#define PRINTK(format, args...)
#endif

#define PRINT_SUBSYS_SHIFT                  (12)
#define PRINT_SUBSYS_IOP                    (0 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_ONFI                   (1 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_UART                   (2 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_FLASH                  (3 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_PL                     (4 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_I2C                    (5 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_RAID                   (6 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_ETH                    (7 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_GPIO                   (8 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_USB                    (9 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_IOC                    (10 << PRINT_SUBSYS_SHIFT)
#define PRINT_SUBSYS_MASK                   (0xF000)
#define PRINT_STRING_INDEX_MASK             (0x0FFF)

/* IOP Pattern */
#define PRINT_IOP_DBG_ALL                   (0xFFFFFFFF)
#define PRINT_IOP_DBG_HAL                   (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_IOP_DBG_INIT                  (0x00000002)        /* Initialization prints */
#define PRINT_IOP_DBG_MEM                   (0x00000004)        /* Memory Management prints */
#define PRINT_IOP_DBG_ERROR                 (0x00000008)        /* Error prints */
#define PRINT_IOP_DBG_CONFIG                (0x00000010)        /* Config Module prints */
#define PRINT_IOP_DBG_INT_DRV               (0x00000020)        /* Interface Driver prints */
#define PRINT_IOP_DBG_MASK                  (PRINT_IOP_DBG_HAL        | \
                                             PRINT_IOP_DBG_INIT       | \
                                             PRINT_IOP_DBG_ERROR      | \
                                             PRINT_IOP_DBG_INT_DRV)
                                             /* PRINT_IOP_DBG_MEM    */
                                             /* PRINT_IOP_DBG_CONFIG */

/* ONFI Pattern */
#define PRINT_ONFI_DBG_ALL                  (0xFFFFFFFF)
#define PRINT_ONFI_DBG_HAL                  (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_ONFI_DBG_INIT                 (0x00000002)        /* Initialization prints */
#define PRINT_ONFI_DBG_MAP_CMD              (0x00000004)        /* ONFI Map Command prints */
#define PRINT_ONFI_DBG_OFFLOAD              (0x00000008)        /* ONFI Memory Offload prints */
#define PRINT_ONFI_DBG_RESTORE              (0x00000010)        /* ONFI Memory Restore prints */
#define PRINT_ONFI_DBG_EXTERNAL             (0x00000020)        /* External APIs prints */
#define PRINT_ONFI_DBG_MASK                 (PRINT_ONFI_DBG_HAL       | \
                                             PRINT_ONFI_DBG_INIT      | \
                                             PRINT_ONFI_DBG_MAP_CMD   | \
                                             PRINT_ONFI_DBG_OFFLOAD   | \
                                             PRINT_ONFI_DBG_RESTORE   | \
                                             PRINT_ONFI_DBG_EXTERNAL)

/* UART Pattern */
#define PRINT_UART_DBG_ALL                  (0xFFFFFFFF)
#define PRINT_UART_DBG_HAL                  (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_UART_DBG_INIT                 (0x00000002)        /* Initialization prints */
#define PRINT_UART_DBG_INTERRUPT            (0x00000004)        /* UART Interrupts prints */
#define PRINT_UART_DBG_MASK                 (PRINT_UART_DBG_HAL       | \
                                             PRINT_UART_DBG_INIT      | \
                                             PRINT_UART_DBG_INTERRUPT)

/* FLASH Pattern */
#define PRINT_FLASH_DBG_ALL                  (0xFFFFFFFF)
#define PRINT_FLASH_DBG_HAL                  (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_FLASH_DBG_INIT                 (0x00000002)        /* Initialization prints */
#define PRINT_FLASH_DBG_CMD                  (0x00000004)        /* FLASH Command prints */
#define PRINT_FLASH_DBG_MASK                 (PRINT_FLASH_DBG_HAL       | \
                                              PRINT_FLASH_DBG_INIT      | \
                                              PRINT_FLASH_DBG_CMD)

/* I2C Pattern */
#define PRINT_I2C_DBG_ALL                  (0xFFFFFFFF)
#define PRINT_I2C_DBG_HAL                  (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_I2C_DBG_INIT                 (0x00000002)        /* Initialization prints */
#define PRINT_I2C_DBG_CMD                  (0x00000004)        /* I2C Command prints */
#define PRINT_I2C_DBG_MASK                 (PRINT_I2C_DBG_HAL       | \
                                            PRINT_I2C_DBG_INIT      | \
                                            PRINT_I2C_DBG_CMD)

/* RAID Pattern */
#define PRINT_RAID_DBG_ALL                  (0xFFFFFFFF)
#define PRINT_RAID_DBG_HAL                  (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_RAID_DBG_INIT                 (0x00000002)        /* Initialization prints */
#define PRINT_RAID_DBG_CMD                  (0x00000004)        /* RAID Command prints */
#define PRINT_RAID_DBG_CDB                  (0x00000008)        /* RAID CDB Process prints */
#define PRINT_RAID_DBG_MASK                 (PRINT_RAID_DBG_HAL       | \
                                             PRINT_RAID_DBG_INIT      | \
                                             PRINT_RAID_DBG_CMD       | \
                                             PRINT_RAID_DBG_CDB)

/* IOC Pattern */
#define PRINT_IOC_DBG_ALL                   (0xFFFFFFFF)
#define PRINT_IOC_DBG_HAL                   (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_IOC_DBG_INIT                  (0x00000002)        /* Initialization prints */
#define PRINT_IOC_DBG_DOORBELL              (0x00000004)        /* IOC DOORBELL prints */
#define PRINT_IOC_DBG_EVENTS                (0x00000008)        /* IOC Events prints */
#define PRINT_IOC_DBG_INTERRUPT             (0x00000010)        /* IOC Interrupt prints */ 
#define PRINT_IOC_DBG_SCHEDULER             (0x00000020)        /* IOC Scheduler prints */ 
#define PRINT_IOC_DBG_TRANSPORT             (0x00000030)        /* IOC Transport prints */ 
#define PRINT_IOC_DBG_MASK                  (PRINT_IOC_DBG_HAL       | \
                                             PRINT_IOC_DBG_INIT      | \
                                             PRINT_IOC_DBG_DOORBELL  | \
                                             PRINT_IOC_DBG_EVENTS    | \
                                             PRINT_IOC_DBG_INTERRUPT | \
                                             PRINT_IOC_DBG_SCHEDULER | \
                                             PRINT_IOC_DBG_TRANSPORT)

/* USB Pattern */
#define PRINT_USB_DBG_ALL                  (0xFFFFFFFF)
#define PRINT_USB_DBG_HAL                   (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_USB_DBG_INIT                  (0x00000002)        /* Initialization prints */
#define PRINT_USB_DBG_TIMER                 (0x00000004)        /* USB Timer Prints */
#define PRINT_USB_DBG_UTIL                  (0x00000008)        /* USB UTIL Prints */
#define PRINT_USB_DBG_CORE                  (0x000000010)       /* USB Core Prints */
#define PRINT_USB_DBG_IOCTL                 (0x000000020)       /* USB IOCTL prints */
#define PRINT_USB_DBG_MASK                  (PRINT_USB_DBG_HAL       | \
                                             PRINT_USB_DBG_INIT      | \
                                             PRINT_USB_DBG_TIMER     | \
                                             PRINT_USB_DBG_UTIL      | \
                                             PRINT_USB_DBG_CORE      | \
                                             PRINT_USB_DBG_IOCTL)

/* GPIO Pattern */
#define PRINT_GPIO_DBG_ALL                  (0xFFFFFFFF)
#define PRINT_GPIO_DBG_HAL                  (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_GPIO_DBG_INIT                 (0x00000002)        /* Initialization prints */
#define PRINT_GPIO_DBG_INTERRUPT            (0x00000004)        /* GPIO Interrup Prints */
#define PRINT_GPIO_DBG_MASK                 (PRINT_GPIO_DBG_HAL     | \
                                             PRINT_GPIO_DBG_INIT    | \
                                             PRINT_GPIO_DBG_INTERRUPT)  

/* SGPIO Pattern */

/* Ethernet Pattern */
#define PRINT_ETH_DBG_ALL                   (0xFFFFFFFF)
#define PRINT_ETH_DBG_HAL                   (0x00000001)        /* Hardware Abstraction Layer prints */
#define PRINT_ETH_DBG_INIT                  (0x00000002)        /* Initialization prints */
#define PRINT_ETH_DBG_SIM                   (0x00000004)        /* Ethernet SIM driver prints */
#define PRINT_ETH_DBG_CRAM                  (0x00000008)        /* Ethernet CRAM driver prints */
#define PRINT_ETH_DBG_FRAME                 (0x00000010)        /* Ethernet Frame management prints */
#define PRINT_ETH_DBG_INTERRUPT             (0x00000020)        /* Ethernet Interrupt prints */
#define PRINT_ETH_DBG_MASK                  (PRINT_ETH_DBG_HAL       | \
                                             PRINT_ETH_DBG_INIT      | \
                                             PRINT_ETH_DBG_SIM       | \
                                             PRINT_ETH_DBG_CRAM      | \
                                             PRINT_ETH_DBG_FRAME     | \
                                             PRINT_ETH_DBG_INTERRUPT)

/* Format of Trace Buffer Entry */
typedef union _TRACE_BUFFER_ENTRY
{
    U32 DWord32;
    struct _Word16
    {
        U16 PrintIndex;
        U16 NumberOfArgs;
    } Word16;

} TRACE_BUFFER_ENTRY, *PTR_TRACE_BUFFER_ENTRY;

/* Debug strings constant array */
extern const char * const gIopDbgStrings[];   /* IOP       */
extern const char * const gOnfiDbgStrings[];  /* ONFI      */
extern const char * const gUartDbgStrings[];  /* UART      */
extern const char * const gFlashDbgStrings[]; /* FLASH     */
extern const char * const gplDbgStrings[];    /* PL        */
extern const char * const gI2cDbgStrings[];   /* I2C       */
extern const char * const gRaidDbgStrings[];  /* RAID      */
extern const char * const gEthDbgStrings[];   /* ETHERNET  */
extern const char * const gGpioDbgStrings[];  /* GPIO      */
extern const char * const gUsbDbgStrings[];   /* USB       */
extern const char * const gIocDbgStrings[];   /* IOC       */

/* Pointers to arrays of strings used for ring buffer output */
static char const * const * const gDebugStrings[] =
{
    gIopDbgStrings,    /* 0  - IOP               */
    gOnfiDbgStrings,   /* 1  - ONFI              */
    gUartDbgStrings,   /* 2  - UART              */
    gFlashDbgStrings,  /* 3  - FLASH             */
    gplDbgStrings,     /* 4  - PL                */
    gI2cDbgStrings,    /* 5  - I2C               */
    gRaidDbgStrings,   /* 6  - RAID              */
    gEthDbgStrings,    /* 7  - ETHERNET          */
    gGpioDbgStrings,   /* 8  - GPIO              */
    gUsbDbgStrings,    /* 9  - USB               */
    gIocDbgStrings,    /* 10 - IOC               */
    NULL               /* End of list indicator  */
};

static const U32 subSysMaxIndex[12] = {IOP_DBG_STR_MAX_INDEX,  /* IOP      */
                                      ONFI_DBG_STR_MAX_INDEX,  /* ONFI     */
                                      UART_DBG_STR_MAX_INDEX,  /* UART     */
                                      FLASH_DBG_STR_MAX_INDEX, /* FLASH    */
                                      PL_DBG_STR_MAX_INDEX,    /* PL       */
                                      I2C_DBG_STR_MAX_INDEX,   /* I2C      */
                                      RAID_DBG_STR_MAX_INDEX,  /* RAID     */
                                      ETH_DBG_STR_MAX_INDEX,   /* ETHERNET */
                                      GPIO_DBG_STR_MAX_INDEX,  /* GPIO     */
                                      USB_DBG_STR_MAX_INDEX,   /* USB      */
                                      IOC_DBG_STR_MAX_INDEX,   /* IOC      */
                                      0};

/* Print Function Prototypes */
#if ENABLE_TRACE_BUFFER_MANAGER
void iopPrintTraceBuffer(void);
#endif

extern void iopiTracePrint0(long long Pattern, U32 Index);
extern void iopiTracePrint1(long long Pattern, U32 Index, U32 Arg1);
extern void iopiTracePrint2(long long Pattern, U32 Index, U32 Arg1, U32 Arg2);
extern void iopiTracePrint3(long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3);
extern void iopiTracePrint4(long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);

extern void iopiConsolePrint0(char * FmtStr);
extern void iopiConsolePrint1(char * FmtStr, U32 Arg1);
extern void iopiConsolePrint2(char * FmtStr, U32 Arg1, U32 Arg2);
extern void iopiConsolePrint3(char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3);
extern void iopiConsolePrint4(char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);

#define _PRINT0(Pattern, Index)                         iopiTracePrint0(Pattern, Index);
#define _PRINT1(Pattern, Index, Arg1)                   iopiTracePrint1(Pattern, Index, (U32)Arg1);
#define _PRINT2(Pattern, Index, Arg1, Arg2)             iopiTracePrint2(Pattern, Index, (U32)Arg1, (U32)Arg2);
#define _PRINT3(Pattern, Index, Arg1, Arg2, Arg3)       iopiTracePrint3(Pattern, Index, (U32)Arg1, (U32)Arg2, (U32)Arg3);
#define _PRINT4(Pattern, Index, Arg1, Arg2, Arg3, Arg4) iopiTracePrint4(Pattern, Index, (U32)Arg1, (U32)Arg2, (U32)Arg3, (U32)Arg4);

#define PRINT0(Pattern, Index)                          if ((((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOP) && (Pattern & PRINT_IOP_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ONFI) && (Pattern & PRINT_ONFI_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_UART) && (Pattern & PRINT_UART_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_FLASH) && (Pattern & PRINT_FLASH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_I2C) && (Pattern & PRINT_I2C_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_RAID) && (Pattern & PRINT_RAID_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ETH) && (Pattern & PRINT_ETH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_GPIO) && (Pattern & PRINT_GPIO_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_USB) && (Pattern & PRINT_USB_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOC) && (Pattern & PRINT_IOC_DBG_MASK)) \
                                                           ) \
                                                        { \
                                                            _PRINT0(Pattern, Index); \
                                                        }

#define PRINT1(Pattern, Index, Arg1)                    if ((((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOP) && (Pattern & PRINT_IOP_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ONFI) && (Pattern & PRINT_ONFI_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_UART) && (Pattern & PRINT_UART_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_FLASH) && (Pattern & PRINT_FLASH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_I2C) && (Pattern & PRINT_I2C_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_RAID) && (Pattern & PRINT_RAID_DBG_MASK))|| \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ETH) && (Pattern & PRINT_ETH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_GPIO) && (Pattern & PRINT_GPIO_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_USB) && (Pattern & PRINT_USB_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOC) && (Pattern & PRINT_IOC_DBG_MASK)) \
                                                           ) \
                                                        { \
                                                            _PRINT1(Pattern, Index, Arg1); \
                                                        }

#define PRINT2(Pattern, Index, Arg1, Arg2)              if ((((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOP) && (Pattern & PRINT_IOP_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ONFI) && (Pattern & PRINT_ONFI_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_UART) && (Pattern & PRINT_UART_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_FLASH) && (Pattern & PRINT_FLASH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_I2C) && (Pattern & PRINT_I2C_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_RAID) &&  (Pattern & PRINT_RAID_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ETH) &&  (Pattern & PRINT_ETH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_GPIO) &&  (Pattern & PRINT_GPIO_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_USB) &&  (Pattern & PRINT_USB_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOC) &&  (Pattern & PRINT_IOC_DBG_MASK)) \
                                                           ) \
                                                        { \
                                                            _PRINT2(Pattern, Index, Arg1, Arg2); \
                                                        }

#define PRINT3(Pattern, Index, Arg1, Arg2, Arg3)        if ((((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOP) && (Pattern & PRINT_IOP_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ONFI) && (Pattern & PRINT_ONFI_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_UART) && (Pattern & PRINT_UART_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_FLASH) && (Pattern & PRINT_FLASH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_I2C) && (Pattern & PRINT_I2C_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_RAID) && (Pattern & PRINT_RAID_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ETH) && (Pattern & PRINT_ETH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_GPIO) && (Pattern & PRINT_GPIO_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_USB) && (Pattern & PRINT_USB_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOC) && (Pattern & PRINT_IOC_DBG_MASK)) \
                                                           ) \
                                                        { \
                                                            _PRINT3(Pattern, Index, Arg1, Arg2, Arg3); \
                                                        }

#define PRINT4(Pattern, Index, Arg1, Arg2, Arg3, Arg4)  if ((((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOP) && (Pattern & PRINT_IOP_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ONFI) && (Pattern & PRINT_ONFI_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_UART) && (Pattern & PRINT_UART_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_FLASH) && (Pattern & PRINT_FLASH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_I2C) && (Pattern & PRINT_I2C_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_RAID) && (Pattern & PRINT_RAID_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_ETH) && (Pattern & PRINT_ETH_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_GPIO) && (Pattern & PRINT_GPIO_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_USB) && (Pattern & PRINT_USB_DBG_MASK)) || \
                                                            (((Index & PRINT_SUBSYS_MASK) == PRINT_SUBSYS_IOC) && (Pattern & PRINT_IOC_DBG_MASK)) \
                                                           ) \
                                                        { \
                                                            _PRINT4(Pattern, Index, Arg1, Arg2, Arg3, Arg4); \
                                                        }

#endif /* IOP_MODULE_PRINT_HEADER */
