/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ethEngineDrv.h
 *          Title:  Ethernet Engine Driver Header File
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  April 27, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  04/27/12  NB    Initial version.
 *
 *
 *  Description
 *  ------------
 *  This file contains the definitions used for Ethernet Engine driver as
 *  an interface.
 *
 *-------------------------------------------------------------------------
 */

#ifndef ETH_ENGINE_DRV_H
#define ETH_ENGINE_DRV_H

#include <linux/module.h>

#include "basetype.h"

#define ENGINE_PRINT_ERR(format, args...) \
    if (gEthEngineDriver.osalPrintk != NULL) \
    { \
        gEthEngineDriver.osalPrintk(format, ##args); \
    }
    
typedef enum _ETH_STATUS
{
    ETH_SUCCESS = 0,
    ETH_FAILURE

} ETH_STATUS;

typedef struct _ETH_ENGINE_DRIVER
{
    void *NetDev;
    void (*osalPrintk)                        ( const char*, ... );
    void *(*osalIoremap)                      ( unsigned long long, U32 );
    void (*osalIounmap)                       ( void * );
    void (*osalMemset)                        ( void *, U8, U32 );
    void (*osalUDelay)                        ( U32 );
    void (*osalMoveToDcr)                     ( U32, U32 );
    U32  (*osalMoveFromDcr)                   ( S32 );

} ETH_ENGINE_DRIVER, *PTR_ETH_ENGINE_DRIVER;

extern ETH_ENGINE_DRIVER gEthEngineDriver;

/* Exported Functions - Ethernet Engine Driver Functions */
extern ETH_STATUS ethEngineRegisterDriver   ( ENGINE_DRIVER EngineDriver );
extern ETH_STATUS ethEngineUnregisterDriver ( void  *NetDev );

#endif /* ETH_ENGINE_DRV_H */
