/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Logic Corporation.  All rights reserved.        *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbEngineDrv.h
 *          Title:  Header file for sample driver for exercising USB stack
 *                  library APIs.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  May 06, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who     Description
 *  --------  ---     ------------------------------------------------------
 *  05/06/11  SS      Initial version.
 *
 * Description
 * ------------
 * This file contains the defines for the sample USB stack library exerciser
 * driver.
 *
 * -------------------------------------------------------------------------
 */
#ifndef USB_ENGINE_DRIVER_H
#define USB_ENGINE_DRIVER_H

#ifndef __KERNEL__
#define __KERNEL__
#endif

#ifndef MODULE
#define MODULE
#endif

/* ------------------------------------------------------------------------ */
/* Kernel Includes                                                          */
/* ------------------------------------------------------------------------ */

#include <linux/module.h>

#include "basetype.h"
#include "commonInterface.h"

/* ------------------------------------------------------------------------ */
/* Macros                                                                   */
/* ------------------------------------------------------------------------ */

/* Component engine driver interfaces */
typedef struct _COMPONENT_ENGINE_DRIVER
{
    void *(*osalIoremap)                  (unsigned long long, U32);
    void (*osalIounmap)                   (void*);
    void (*osalMoveToDcr)                 (U32, U32);
    U32  (*osalMoveFromDcr)               (S32);
    void *(*osalConvertPhysicalToVirtual) (U32);
    void (*iopiTracePrint0)               (long long Pattern, U32 Index);
    void (*iopiTracePrint1)               (long long Pattern, U32 Index, U32 Arg1);
    void (*iopiTracePrint2)               (long long Pattern, U32 Index, U32 Arg1, U32 Arg2);
    void (*iopiTracePrint3)               (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3);
    void (*iopiTracePrint4)               (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);
    void (*iopiDiagConsolePrint0)         (char * FmtStr);
    void (*iopiDiagConsolePrint1)         (char * FmtStr, U32 Arg1);
    void (*iopiDiagConsolePrint2)         (char * FmtStr, U32 Arg1, U32 Arg2);
    void (*iopiDiagConsolePrint3)         (char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3);
    void (*iopiDiagConsolePrint4)         (char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);

} COMPONENT_ENGINE_DRIVER, *PTR_COMPONENT_ENGINE_DRIVER;
extern COMPONENT_ENGINE_DRIVER gComponentEngineDriver;

/* ------------------------------------------------------------------------ */
/* Externs                                                                  */
/* ------------------------------------------------------------------------ */

extern int usbHalRegisterInterfaceDriver   ( ENGINE_DRIVER IfaceData );
extern int usbHalUnregisterInterfaceDriver ( U32 MajorNumber );

#endif /* #ifndef USB_ENGINE_DRIVER_H */
