/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2010 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pcieiDefs.h
 *          Title:  PCIe Interface (PCIe) Definitions
 *     Programmer:  Patni
 *  Creation Date:  May 25, 2010
 *
 *
 *  Description
 *  ------------
 *  This include file contains definitions for the PCIe interface exports
 *  to be used by other modules to interface with PCIe core.
 *
 *-------------------------------------------------------------------------
 */

#ifndef PCIEI_DEFS_H_INCLUDED
#define PCIEI_DEFS_H_INCLUDED

/*
 * PCIEI API return status
 */
typedef enum _PCIEI_STATUS
{
    PCIEI_SUCCESS = 0,
    PCIEI_FAILURE,
    PCIEI_INVALID_ARGUMENT
} PCIEI_STATUS;

/*
 * PCIEI bit status
 */
typedef enum _PCIEI_BIT_STATUS
{
    PCIEI_BIT_CLEAR = 0,
    PCIEI_BIT_SET
} PCIEI_BIT_STATUS, *PTR_PCIEI_BIT_STATUS;

/*
 * Bit Masks for lower 16 and upper 16 bits for a 32bit register
 */
#define PCIEI_LOWER16_BIT_MASK                   (0x0000FFFF)
#define PCIEI_UPPER16_BIT_MASK                   (0xFFFF0000)

/*
 * Defines for the WriteSequence register
 */
#define PCIEI_WRSEQ_KEY_MASK                     (0x0000000F)
#define PCIEI_WRSEQ_KEY1                         (0xF)
#define PCIEI_WRSEQ_KEY2                         (0x4)
#define PCIEI_WRSEQ_KEY3                         (0xB)
#define PCIEI_WRSEQ_KEY4                         (0x2)
#define PCIEI_WRSEQ_KEY5                         (0x7)
#define PCIEI_WRSEQ_KEY6                         (0xD)

/*
 * Defines for Local Queues
 */
#define PCIEI_REQUESTPOST_QUEUE                   (0x1)
#define PCIEI_REQUESTFREE_QUEUE                   (0x2)
#define PCIEI_HIPRIOREQPOST_QUEUE                 (0x3)
#define PCIEI_RDMAPOST_QUEUE                      (0x4)
#define PCIEI_REPLYMESSAGE_QUEUE                  (0x5)
#define PCIEI_FPE_REQUEST_POST_QUEUE              (0x6)

/*
 * Defines for Descriptor Queues Indexes
 */
#define IOP_REQ_DESCQ_DEFAULT                   (0)
#define IOP_REQ_DESCQ_SCSI_IO                   (IOP_REQ_DESCQ_DEFAULT)
#define IOP_REQ_DESCQ_SCSI_TARG                 (IOP_REQ_DESCQ_DEFAULT)
#define IOP_REQ_DESCQ_HI_PRIO                   (1) //(IOP_REQ_DESCQ_DEFAULT)
#define IOP_REQ_DESCQ_FPE                       (2)
#define IOP_REQ_DESCQ_RAIDA                     (IOP_REQ_DESCQ_DEFAULT) //(1)
#define IOP_REQ_DESCQ_REQ_FREE                  (3)

#define IOP_REPLYQ_FPE                          (2)
/*
 * Defines for the HostInterruptStatus register
 */
#define PCIEI_HIS_SYS2IOP_DB_STATUS               (0x80000000)
#define PCIEI_HIS_RESET_IRQ_STATUS                (0x40000000)
#define PCIEI_HIS_SAS_ALL_INT_STATUS              (0x00000010)
#define PCIEI_HIS_REPLY_INT_STATUS                (0x00000008)
#define PCIEI_HIS_IOP2SYS_DB_STATUS               (0x00000001)

/*
 * Defines for the Queue Status register
 */
#define PCIEI_QS_HPREQ_OVERFLOW                   (0x00000001)
#define PCIEI_QS_REQPOST_OVERFLOW                 (0x00000002)
#define PCIEI_QS_RDMA_OVERFLOW                    (0x00000004)
#define PCIEI_QS_REPLY_OVERFLOW                   (0x00000008)


/*
 * Defines for the DMA Control/Status register
 */
#define PCIEI_SYSDMA_MASK                         (0x0000001F)
#define PCIEI_SYSDMA_ACTIVE                       (0x00000010)
#define PCIEI_SYSDMA_PARITY_ERROR                 (0x00000008)
#define PCIEI_SYSDMA_BUS_FAULT                    (0x00000004)
#define PCIEI_SYSDMA_CLEAR_DMA_ERROR              (0x00000002)
#define PCIEI_SYSDMA_CLEAR_DMA_INT                (0x00000001)

/*
 * Defines for the DMA Command Type
 */
#define PCIEI_SYSDMA_CMD_PLB_TO_PCI  0
#define PCIEI_SYSDMA_CMD_PCI_TO_PLB  1

/*
 * Defines for the IOP interrupt status register
 */
#define PCIEI_IOPIS_SYS_IRQ_INT                  (0x00000001)
#define PCIEI_IOPIS_HIPRIORITY_INT               (0x00000002)
#define PCIEI_IOPIS_RDMA_INT                     (0x00000004)
#define PCIEI_IOPIS_REPLY_INT                    (0x00000008)
#define PCIEI_IOPIS_SYS2IOPDOORBELL              (0x00000010)
#define PCIEI_IOPIS_SYSTEMDMAINT                 (0x00000020)
#define PCIEI_IOPIS_REPLYPOSTFIFOFULL            (0x00000040)
#define PCIEI_IOPIS_IOP2SYSDOORBELLTAKEN         (0x00000080)
#define PCIEI_IOPIS_RESETADAPTER                 (0x00000100)
#define PCIEI_IOPIS_SYSDMAERROR                  (0x00000200)
#define PCIEI_IOPIS_PCIESTATUSERROR              (0x00000400)
#define PCIEI_IOPIS_FMUTRAP                      (0x00000800)
#define PCIEI_IOPIS_MESSAGEUNITERROR             (0x00001000)
#define PCIEI_IOPIS_LMIDEMPTY                    (0x00002000)
#define PCIEI_IOPIS_MESSAGEQOVERFLOWIRQ          (0x00004000)
#define PCIEI_IOPIS_MSGPULLDMAERROR              (0x00008000)
#define PCIEI_IOPIS_RCTXDONEIRQ                  (0x00010000)
#define PCIEI_IOPIS_RCTXERRORIRQ                 (0x00020000)
#define PCIEI_IOPIS_FMUMSGHDRIRQ                 (0x00040000)
#define PCIEI_IOPIS_FMUMSIXMSGIRQFUSION          (0x00080000)
#define PCIEI_IOPIS_POMSILERRORIRQ               (0x00100000)
#define PCIEI_IOPIS_POBCOMPERRORINT              (0x00200000)
#define PCIEI_IOPIS_DPAIRQ                       (0x00400000)
#define PCIEI_IOPIS_MSGPULLCHAINOVERFLOW         (0x00800000)
#define PCIEI_IOPIS_MSIXERROR                    (0x01000000)
#define PCIEI_IOPIS_VPD                          (0x02000000)

/*
 * Defines for the messaging unit error register
 */
#define PCIEI_FMU_ERROR_STATUS_DESC_OVRFLW       (0x00200000)
#define PCIEI_FMU_ERROR_STATUS_LMID_ERROR        (0x00100000)
#define PCIEI_FMU_ERROR_STATUS_REPLY_BF          (0x00080000)
#define PCIEI_FMU_ERROR_STATUS_REPLY_PAR_ERR     (0x00040000)
#define PCIEI_FMU_ERROR_STATUS_MSG_PUL_PAR       (0x00020000)
#define PCIEI_FMU_ERROR_STATUS_PCI_BUS_FAULT     (0x00010000)
#define PCIEI_FMU_ERROR_STATUS_DIAG_WIND_LK      (0x00008000)
#define PCIEI_FMU_ERROR_STATUS_RDMA_FIFO_DIS     (0x00004000)
#define PCIEI_FMU_ERROR_STATUS_HIPRI_FIFO_DIS    (0x00002000)
#define PCIEI_FMU_ERROR_STATUS_REPLY_POST_DIS    (0x00001000)
#define PCIEI_FMU_ERROR_STATUS_REQ_FREE_DIS      (0x00000800)
#define PCIEI_FMU_ERROR_STATUS_REQ_POST_DIS      (0x00000400)
#define PCIEI_FMU_ERROR_STATUS_PCI_BDOR_RD       (0x00000200)
#define PCIEI_FMU_ERROR_STATUS_PCI_BDOR_WR       (0x00000100)
#define PCIEI_FMU_ERROR_STATUS_AHB_READ_ERR      (0x00000080)
#define PCIEI_FMU_ERROR_STATUS_AHB_WR_ERR        (0x00000040)
#define PCIEI_FMU_ERROR_STATUS_DCR_READ_ERR      (0x00000020)
#define PCIEI_FMU_ERROR_STATUS_DCR_WR_ERR        (0x00000010)
#define PCIEI_FMU_ERROR_STATUS_REG_WR_PAR_ERR    (0x00000008)
#define PCIEI_FMU_ERROR_STATUS_RESERVE_WRITE     (0x00000004)
#define PCIEI_FMU_ERROR_STATUS_DIAG_WND_READ     (0x00000002)
#define PCIEI_FMU_ERROR_STATUS_DIAG_WND_WR       (0x00000001)

/*
 * Defines for the PCI related Queue values
 */
#define PCIEI_IF_QUEUE_SIZE_INC      (16)
#define PCIEI_IF_QUEUE_SIZE(Size)    ((Size)/PCIEI_IF_QUEUE_SIZE_INC - 1)
#define PCIEI_IF_QUEUE_SIZE_SH       1
#define PCIEI_IF_QUEUE_SIZE_ENABLE   (0x00000001)
#define PCIEI_IF_QUEUE_SIZE_MAX      (65536)

/*
 * Defines for the PLB Control register
 */
#define PCIEI_PLB_CONTROL_DISABLE_PARITY             (0x80000000)
#define PCIEI_PLB_CONTROL_DISABLE_READ_SIDE_EFFECT   (0x40000000)
#define PCIEI_PLB_CONTROL_DISABLE_AHB_INCREMENT      (0x10000000)
#define PCIEI_PLB_CONTROL_MSI_STALL_EN               (0x08000000)
#define PCIEI_PLB_CONTROL_DISABLE_TAG_ERROR_SHIMC    (0x00040000)
#define PCIEI_PLB_CONTROL_DISABLE_TAG_ERROR_SHIMB    (0x00020000)
#define PCIEI_PLB_CONTROL_DISABLE_TAG_ERROR_SHIMA    (0x00010000)
#define PCIEI_PLB_CONTROL_ENABLE_DESC_FLAGGING       (0x00008000)
#define PCIEI_PLB_CONTROL_MSG_PULL_STALL_ERROR_EN    (0x00002000)
#define PCIEI_PLB_CONTROL_MSG_PULL_STALL_CHAIN_EN    (0x00001000)
#define PCIEI_PLB_CONTROL_PREVENT_FLR_CLEARING       (0x00000800)
#define PCIEI_PLB_CONTROL_DISABLE_RSVD_REG_WR_ERROR  (0x00000400)
#define PCIEI_PLB_CONTROL_ENABLE_FRAME_SIL_INSERT    (0x00000200)
#define PCIEI_PLB_CONTROL_DISABLE_DESC_SIL_INSERT    (0x00000100)
#define PCIEI_PLB_CONTROL_DISABLE_PCI_INCREMENT      (0x00000080)
#define PCIEI_PLB_CONTROL_DISABLE_DCR_INCREMENT      (0x00000040)
#define PCIEI_PLB_MULTIPLE_REPLY_QUEUE_ENABLE_RW     (0x00000010)
#define PCIEI_PLB_CONTROL_BLOCK_PCI_ACCESS           (0x00000008)
#define PCIEI_PLB_CONTROL_SAS_ALL_INTS_MASK          (0x00000004)
#define PCIEI_PLB_CONTROL_DISABLE_PREFETCHING        (0x00000002)
#define PCIEI_PLB_CONTROL_BYPASS_ENABLE              (0x00000001)

#define PMU_DCR_BASE                    (SASHW_DCR_PMU)
#define PMU_DCR_PLB0_BASE_LOW           (PMU_DCR_BASE + 0x00)
#define PMU_DCR_PLB0_BASE_HIGH          (PMU_DCR_BASE + 0x04)
#define PMU_DCR_PLB0_BASE_SIZE          (PMU_DCR_BASE + 0x08)
#define PMU_DCR_PLB0_CONTROL            (PMU_DCR_BASE + 0x0C)
#define PMU_DCR_PLB1_BASE_LOW           (PMU_DCR_BASE + 0x00)
#define PMU_DCR_PLB1_BASE_HIGH          (PMU_DCR_BASE + 0x04)
#define PMU_DCR_PLB1_BASE_SIZE          (PMU_DCR_BASE + 0x08)
#define PMU_DCR_PLB1_CONTROL            (PMU_DCR_BASE + 0x0C)
#define PMU_DCR_PCIE_CORE_STATUS        (PMU_DCR_BASE + 0x28)

#define PMU_PCIE_STATUS_CPU0_NCRITINT   (0x00000100)
/*
 * Parameters to pass to pcieiConfigPIM() to initialize PIM region
 */
typedef struct _PCIEI_CONFIG_PIM_PARAMS
{
    U32   PIMId;                        /* Specifies PIM region i.e 0,1 or 2 */
    U32   PLBAddrLow;                   /* PLB Low Address */
    U32   PLBAddrHigh;                  /* PLB High Address */
    U32   PIMSize;                      /* PIM window size in bytes*/
    U32   MSIX_Disable;                 /* MSIX window disable flag*/
    U32   MU_Enable;                    /* Message Unit enable flag*/
    U32   Prefetch_Enable;              /* Prefetch Enable flag*/
    U32   PIM_Enable;                   /* PIM enable to map PLB space to PCI*/
} PCIEI_CONFIG_PIM_PARAMS, *PTR_PCIEI_CONFIG_PIM_PARAMS;

/*
 * Parameters to pass to pcieiConfigPOM() to initialize POM region
 */
typedef struct _PCIEI_CONFIG_POM_PARAMS
{
    U32   POMId;                        /* Specifies POM region i.e 0 or 1 */
    U32   PLBAddrLow;                   /* PLB low address */
    U32   PLBAddrHigh;                  /* PLB high address */
    U32   PCIAddrLow;                   /* PCI low address */
    U32   PCIAddrHigh;                  /* PCI high address */
    U32   POMSize;                      /* Size of POM window in bytes*/
} PCIEI_CONFIG_POM_PARAMS, *PTR_PCIEI_CONFIG_POM_PARAMS;

/*
 * Parameters for System DMA
 */
typedef struct _PCIEI_SYSDMA_CMD_PARAMS
{
    U32 PciAddressLow;                  /* System Dma PCI address low */
    U32 PciAddressHigh;                 /* System Dma PCI address high */
    U32 LocalAddressLow;                /* System Dma Local address low */
    U32 LocalAddressHigh;               /* System Dma Local address high */
    U32 DmaLength;                      /* System Dma PCI Length */
    U32 Command;                        /* Specifies the DMA command type */
    U32 SIL;                            /* Specifies the SIL value */
} PCIEI_SYSDMA_CMD_PARAMS, *PTR_PCIEI_SYSDMA_CMD_PARAMS;

/*
 * Parameters for Descriptor Soft Fifo
 */
typedef struct _PCIEI_DESC_GENERIC_FIFO_PARAMS
{
    //U32 BaseAddress;                    /* Descriptor soft fifo base address */
    U64 BaseAddress;                    /* Descriptor soft fifo base address */
    U32 FifoSize;                       /* Descriptor soft fifo size */
    U32 Threshold;                      /* Descriptor soft fifo threshold */
} PCIEI_DESC_GENERIC_FIFO_PARAMS, *PTR_PCIEI_DESC_GENERIC_FIFO_PARAMS;
/*
 * Parameters to pass to initialize Fusion message pull control register
 */

typedef struct _PCIEI_CONFIG_MSG_PULL_CONTROL
{
    U32 QueueType;         /* Indicates the type of Queue to be manipulated */
    U32 LocalMessageSize;  /* Max 512 must be a multiple of 16 */
    U32 HostMessageSize;   /* Max 512 must be a multiple of 16 */
    U32 DelayPost;         /* Delays MFA Posting until all chains are pulled */
    U32 EnChainHW;         /* Pull chains if the chain offset is not zero */
    U32 EnPullHW;          /* Pull messages when MID posted */
    U32 EnBID1;            /* BID1 is enabled */
} PCIEI_CONFIG_MSG_PULL_CONTROL, *PTR_PCIEI_CONFIG_MSG_PULL_CONTROL;

#endif /* End PCIEI_H_INCLUDED */

