/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  rootComplex.h
 *          Title:  LSI SAS3108 PCI-E root complex interface definitions
 *     Programmer:  Sandeep Darisala
 *  Creation Date:  March 13, 2012
 *
 * Description
 * ------------
 *
 *    This file contains the LSI SAS3108 PCI-E root complex library's
 *    interface definitions.
 *-------------------------------------------------------------------------
 */

#ifndef _ROOTCOMPLEX_H
#define _ROOTCOMPLEX_H

/* Supported Link Widths */
#define PCIE_LINK_WIDTH_1               (1)
#define PCIE_LINK_WIDTH_2               (2)
#define PCIE_LINK_WIDTH_4               (4)
#define PCIE_LINK_WIDTH_8               (8)
#define PCIE_LINK_WIDTH_12              (12)
#define PCIE_LINK_WIDTH_16              (16)
#define PCIE_LINK_WIDTH_32              (32)

/* Supported Link Speeds */
#define PCIE_LINK_SPEED_2_5             (1)
#define PCIE_LINK_SPEED_5_0             (2)
#define PCIE_LINK_SPEED_8_0             (3)

/* POM0 window (64 MB) PLB address map */
#define POM0_PLB_START_ADDRESS          (0xF0000000)
#define POM0_PLB_END_ADDRESS            (0xF3FFFFFF)
#define POM0_WINDOW_SIZE                (26)    /* Size represented as a power of 2. So, 26 represents 64 MB */

/* POM1 window (128 MB) PLB address map */
#define POM1_PLB_START_ADDRESS          (0xC8000000)
#define POM1_PLB_END_ADDRESS            (0xCFFFFFFF)
#define POM1_WINDOW_SIZE                (27)    /* Size represented as a power of 2. So, 27 represents 128 MB */

/* MSI-X region defines */
#define PCIE_BASE_ADDRESS_FOR_MSIX_REGION       (0x3C00)
#define PCIE_END_ADDRESS_FOR_MSIX_REGION        (0x3DFF)
#define PLB_BASE_ADDRESS_FOR_MSIX_REGION        (0x4C00)
#define PLB_END_ADDRESS_FOR_MSIX_REGION         (0x4DFF)
#define MSIX_REGION_START_FROM_PIM0_TOP         (8 * 1024)
#define MSIX_PBA_REGION_START_FROM_PIM0_TOP     (4 * 1024)

typedef struct _PIM_ATTRIBUTES
{
    U8      PIMEnable       :1;
    U8      PrefetchEnable  :1;
    U8      MUEn            :1;
    U8      MSIXDisable     :1;
    U8      PIMShell        :1;
    U8      Reserved        :3;
} PIM_ATTRIBUTES;

/* Parameters to pass to pcieRcConfigurePIM() to initialize a PIM region */
typedef struct _PCIE_RC_CONFIG_PIM_PARAMS
{
    U32                 PLBAddressLow;
    U32                 PLBAddressHigh;
    U32                 PCIeAddressLow;
    U32                 PCIeAddressHigh;
    U32                 PIMSizeLow;
    U32                 PIMSizeHigh;
    PIM_ATTRIBUTES      PIMAttributes;
    U8                  PIMId;
    U16                 Reserved;
} PCIE_RC_CONFIG_PIM_PARAMS, *PTR_PCIE_RC_CONFIG_PIM_PARAMS;

/* Parameters to pass to pcieRcConfigurePOM() to initialize a POM region */
typedef struct _PCIE_RC_CONFIG_POM_PARAMS
{
    U32     PLBAddressLow;
    U32     PLBAddressHigh;
    U32     PCIeAddressLow;
    U32     PCIeAddressHigh;
    U32     POMSize;
    U8      SIL;
    U8      POMId;
    U16     Reserved1;
} PCIE_RC_CONFIG_POM_PARAMS, *PTR_PCIE_RC_CONFIG_POM_PARAMS;

/* MSI-X message to pass to pcieRcConfigureMSIX() */
typedef struct _PCIE_RC_MSIX_MESSAGE
{
    U32     LowAddress;
    U32     HighAddress;
    U32     Data;
    U32     VectorControl;
} PCIE_RC_MSIX_MESSAGE, *PTR_PCIE_RC_MSIX_MESSAGE;

static inline void MovToDcr(unsigned int DCRAddress, unsigned int Value)
{
         asm volatile ("mtdcrx  %0,%1" : : "r"(DCRAddress), "r"(Value));
}

static inline unsigned int MovFrmDcr(unsigned int DCRAddress)
{
        unsigned int ret;

        asm volatile ("mfdcrx  %0,%1" : "=r"(ret) : "r"(DCRAddress));

        return ret;
}

void        udelay(U32 USec);
int         pcieRcIsRootComplexEnabled(void);
int         pcieRcIsLinkUp(void);
int         pcieRcPerformLinkWidthTraining(U8 Width);
int         pcieRcPerformLinkSpeedTraining(U8 Speed);
int         pcieRcConfigurePOM(PCIE_RC_CONFIG_POM_PARAMS *PtrPOMConfigParams);
int         pcieRcConfigurePIM(PCIE_RC_CONFIG_PIM_PARAMS *PtrPIMConfigParams);
void        pcieRcEnableMsixCapability(void);
int         pcieRcConfigureMSIX(U8 Bus, U8 Device, U8 Function, U8 Vector, PCIE_RC_MSIX_MESSAGE *PtrMsixMessage);
U8          pcieRcFindCapability(U8 Bus, U8 Device, U8 Function, U32 CapabilityId);
int         pcieRcCfgReadDword(U8 Bus, U8 Device, U8 Function, U32 Address, U32 *PtrData);
int         pcieRcCfgReadWord(U8 Bus, U8 Device, U8 Function, U32 Address, U16 *PtrData);
int         pcieRcCfgReadByte(U8 Bus, U8 Device, U8 Function, U32 Address, U8 *PtrData);
int         pcieRcCfgWriteDword(U8 Bus, U8 Device, U8 Function, U32 Address, U32 Data);
int         pcieRcCfgWriteWord(U8 Bus, U8 Device, U8 Function, U32 Address, U16 Data);
int         pcieRcCfgWriteByte(U8 Bus, U8 Device, U8 Function, U32 Address, U8 Data);
int         pcieRcMemReadDword(U32 Address, U32 *Data);
int         pcieRcMemWriteDword(U32 Address, U32 Data);
void        pcieRcReadBackDoor(U32 Address, U32 *Data);
void        pcieRcWriteBackDoor(U32 Address, U32 Data);
void        pcieRcGetPhyAddr(unsigned long long int *PtrPhysAddr);
void        pcieRcGetVirtAddr(unsigned int *PtrVirtAddr);
int         rootComplexWrapper(int argc, char *argv[]);

#endif  /* _ROOTCOMPLEX_H */
