/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  raidLibApi.c
 *          Title:  RAID Library Header file
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  April 17, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  03/17/12  NB    Initial version
 *
 *  Description
 *  -----------
 *  This file contains the definitions, Structure and Macros implemented 
 *  in the raid library.
 *
 * -------------------------------------------------------------------------
 */

#ifndef RAID_LIB_API_H
#define RAID_LIB_API_H

/* Raid Completion Queue Size */
#define RACC_COMPLETION_Q_SIZE_32                          (0)
#define RACC_COMPLETION_Q_SIZE_128                         (1)
#define RACC_COMPLETION_Q_SIZE_1024                        (2)
#define RACC_COMPLETION_Q_SIZE_2048                        (3)

/* CDB bits */
#define RACC_DMA_NEXT_CDB_STRICT_ORDERING                  (0x8000)
#define RACC_DMA_SINGLE_SOURCE_MOVE_MULTIPLY               (0x4000)
#define RACC_DMA_SECRET_ROOT_KEY_ENABLE                    (0x2000)
#define RACC_DMA_SOURCE_DATA_BIT_BUCKET_XOR_ZERO_CHECK     (0x1000)
#define RACC_DMA_LFSR_INVERT_ENABLE                        (0x0800)
#define RACC_DMA_PCI_NO_SNOOP                              (0x0400)
#define RACC_DMA_PCI_RELAXED_ORDERING                      (0x0200)
#define RACC_DMA_USE_ALT_CRC_SEED                          (0x0100)
#define RACC_DMA_PASSTHROUGH_SRC_REF_TAG                   (0x0080)
#define RACC_DMA_DONT_INTERRUPT                            (0x0020)
#define RACC_DMA_WAIT_FOR_ACK                              (0x0010)
#define RACC_DMA_SOURCE_ADDR_IS_SGL                        (0x0008)
#define RACC_DMA_DEST_ADDR_IS_SGL                          (0x0004)
#define RACC_DMA_XOR_ADDR_IS_SGL                           (0x0002)
#define RACC_DMA_INCREMENTAL_XOR                           (0x0001)

/* Hash modes - Modify Options[2:0] */
#define RACC_DMA_HASH_SHA_224                              (0x00)
#define RACC_DMA_HASH_SHA_256                              (0x01)
#define RACC_DMA_HASH_SHA_384                              (0x02)
#define RACC_DMA_HASH_SHA_512                              (0x03)

/* RAID CDB Flags */
#define RACC_NEXTCDBPTR_END_OF_CHAIN ((U32)(-1UL))
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED0             (0x00000001)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_XOR_ADDR_TYPE         (0x00000002)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_DST_ADDR_TYPE         (0x00000004)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_SRC_ADDR_TYPE         (0x00000008)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_DMA_WAIT_ACK          (0x00000010)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_DMA_NO_IRQ            (0x00000020)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED1             (0x00000040)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_SRC_REF_TAG           (0x00000080)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_ALT_CRC               (0x00000100)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_PCI_RELAX             (0x00000200)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_PCI_NOSNOOP           (0x00000400)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_LSFR_EN_INV           (0x00000800)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_XOR_0_CHK             (0x00001000)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED2             (0x00002000)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED3             (0x00004000)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_NXT_CDB_STRICT        (0x00008000)

/* DMA opcodes */
#define RACC_DMA_NOOP                                      (0x00)
#define RACC_DMA_SINGLE_SOURCE_MOVE                        (0x01)
#define RACC_DMA_MULTICAST_MOVE                            (0x02)
#define RACC_DMA_MULTI_SOURCE_MOVE                         (0x03)
#define RACC_DMA_XOR1_MOVE                                 (0x04)
#define RACC_DMA_XOR2_MOVE                                 (0x05)
#define RACC_DMA_GENERATE_XOR1                             (0x06)
#define RACC_DMA_GENERATE_XOR2                             (0x08)
#define RACC_DMA_CHECK_PATTERN                             (0x09)
#define RACC_DMA_FILL_PATTERN                              (0x0A)
#define RACC_DMA_CHECK_LFSR                                (0x0B)
#define RACC_DMA_FILL_LFSR                                 (0x0C)
#define RACC_DMA_GENERATE_HASH                             (0x0D)
#define RACC_DMA_GENERATE_HMAC                             (0x0E)
#define RACC_DMA_MOVE_MULTIPLY                             (0x0F)

/* Address Type */
#define RACC_DMA_ADDRESS_PCI                               (0x00)
#define RACC_DMA_ADDRESS_MEM                               (0x10)
#define RACC_DMA_ADDRESS_PLB                               (0x20)
#define RACC_DMA_ADDRESS_PLB_NTA                           (0x30)

/* Raid DMA Arbitration Mode */
#define RACC_DMA_ARB_MODE_1_TO_1                           (0) /* 1 from A, 1 from B */
/* Keep fetch from A until it runs out, then fetch from B. */
#define RACC_DMA_ARB_MODE_PRIORITY                         (1)     
#define RACC_DMA_ARB_MODE_2_TO_1                           (2) /* 2 from A, 1 from B */

/* Raid CDB Header */
typedef struct _RACC_DMA_CDB_HEADER
{
    U8                          Opcode;                             /* 0x0       */
    U8                          CommandOptions;                     /* 0x1       */
    U16                         CDBLength;                          /* 0x2-0x3   */
    U8                          ModifyOptions;                      /* 0x4       */
    U8                          NumberOfSourceAddresses;            /* 0x5       */
    U16                         CDBFlags;                           /* 0x6-0x7   */
    U32                         DataByteCount;                      /* 0x8-0xb   */
    struct _RACC_DMA_CDB_HEADER *NextCDBPointer;                    /* 0xc-0xf   */
    U16                         BlockSize;                          /* 0x10-0x11 */
    U8                          SIL;                                /* 0x12      */
    U8                          ComparePolicyGuardCtrl;             /* 0x13      */
    U32                         SourceSkipCount;                    /* 0x14      */
    U32                         DestinationSkipCount;               /* 0x18      */
    U32                         Reserved;                           /* 0x1C      */
} RACC_DMA_CDB_HEADER, * PTR_RACC_DMA_CDB_HEADER;

/* Source Field */
typedef struct _RACC_DMA_SOURCE
{
    U64                         SourceAddress;                      /* 0x0-0x7 */
    U8                          SourceAttributes;                   /* 0x8     */
    U8                          XOR1Multiplier;                     /* 0x9     */
    U8                          XOR2Multiplier;                     /* 0xA     */
    U8                          Reserved1;                          /* 0xB     */
    U32                         SourceReferenceTagInitial;          /* 0xC-0xF */
} RACC_DMA_SOURCE, * PTR_RACC_DMA_SOURCE;

/* Source EEDP Filed */
typedef struct _RACC_DMA_SOURCE_EEDP
{
    U16                         SourceApplicationTagInitial;        /* 0x0-0x1 */
    U16                         SourceApplicationTagMask;           /* 0x2-0x3 */
    U16                         ApplicationTagTranslationMask;      /* 0x4-0x5 */
    U16                         Reserved1;                          /* 0x6-0x7 */
    U32                         Reserved2;                          /* 0x8-0xB */
    U32                         Reserved3;                          /* 0xC-0xF */
} RACC_DMA_SOURCE_EEDP, * PTR_RACC_DMA_SOURCE_EEDP;

/* XOR1 DMA */
typedef struct _RACC_DMA_XOR1
{
    U64                         XOR1Address;                        /* 0x0-0x7 */
    U8                          XOR1BufferAttributes;               /* 0x8 */
    U8                          Reserved;                           /* 0x9 */
    U16                         XOR1ApplicationTagInitial;          /* 0xa-0xb */
    U32                         XOR1ReferenceTagInitial;            /* 0xc-0xf */
} RACC_DMA_XOR1, * PTR_RACC_DMA_XOR1;

/* XOR2 DMA */
typedef struct _RACC_DMA_XOR2
{
    U64                         XOR2Address;                        /* 0x0-0x7 */
    U8                          XOR2BufferAttributes;               /* 0x8 */
    U8                          Reserved;                           /* 0x9 */
    U16                         XOR2ApplicationTagInitial;          /* 0xa-0xb */
    U32                         XOR2ReferenceTagInitial;            /* 0xc-0xf */
} RACC_DMA_XOR2, * PTR_RACC_DMA_XOR2;

/* NOOP CBD */
typedef struct _NOOP_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F */
} NOOP_CDB, * PTR_NOOP_CDB;

/* DMA Destination */
typedef struct _RACC_DMA_DEST
{
    U64                         DestinationAddress;                 /* 0x0-0x7 */
    U8                          DestinationAttributes;              /* 0x8 */
    U8                          Reserved;                           /* 0x9 */
    U16                         DestinationApplicationTagInitial;   /* 0xa-0xb */
    U32                         DestinationReferenceTagInitial;     /* 0xc-0xf */
} RACC_DMA_DEST, * PTR_RACC_DMA_DEST;

/* CBD Aligned */
typedef struct _SSM_CDB_ALIGNED
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src;                                /* 0x20-0x2F */
    RACC_DMA_DEST               Dst;                                /* 0x30-0x3F */
} SSM_CDB_ALIGNED, * PTR_SSM_CDB_ALIGNED;

/* EECP CDB */
typedef struct _SSM_EEDP_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src;                                /* 0x20-0x2F */
    RACC_DMA_SOURCE_EEDP        Eedp;                               /* 0x30-0x3F */
    RACC_DMA_DEST               Dst;                                /* 0x40-0x4F */
} SSM_EEDP_CDB, * PTR_SSM_EEDP_CDB;

/* RACC_DMA_MULTICAST_MOVE (opcode 0x2) */
typedef struct _MCM_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src;                                /* 0x20-0x2F */
    RACC_DMA_DEST               Dst[2];                             /* 0x30-0x4F */
} MCM_CDB, * PTR_MCM_CDB;

/* RACC_DMA_GENERATE_XOR1 (opcode 0x6) */
typedef struct _XOR1_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src[2];                             /* 0x20-0x3F */
    RACC_DMA_XOR1               Xor1;                               /* 0x40-0x4F */
} XOR1_CDB, * PTR_XOR1_CDB;

/* RACC_DMA_GENERATE_XOR2 (opcode 0x8) */
typedef struct _XOR2_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src[2];                             /* 0x20-0x3F */
    RACC_DMA_XOR1               Xor1;                               /* 0x40-0x4F */
    RACC_DMA_XOR2               Xor2;                               /* 0x50-0x5F */
} XOR2_CDB, * PTR_XOR2_CDB;

/* RACC_DMA_CHECK_PATTERN (opcode 0x9) */
typedef struct _CHK_PAT_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    U32                         CheckPattern[4];                    /* 0x20-0x2F */
    U64                         SourceAddress;                      /* 0x30-0x37 */
    U8                          SourceAttributes;                   /* 0x38      */
    U8                          Reserved[7];                        /* 0x39-0x3F */
} CHK_PAT_CDB, * PTR_CHK_PAT_CDB;

/* RACC_DMA_FILL_PATTERN (opcode 0xA) */
typedef struct _FILL_PAT_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F */
    U32                         FillPattern[4];                     /* 0x20-0x2F */
    RACC_DMA_DEST               Dst;                                /* 0x30-0x3F */
} FILL_PAT_CDB, * PTR_FILL_PAT_CDB;

/* RACC_DMA_CHECK_LFSR (opcode 0xB) */
typedef struct _CHK_LFSR_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    U32                         LfsrSeed[4];                        /* 0x20-0x2F */
    U64                         SourceAddress;                      /* 0x30-0x37 */
    U8                          SourceAttributes;                   /* 0x38      */
    U8                          Reserved[7];                        /* 0x39-0x3F */
} CHK_LFSR_CDB, * PTR_CHK_LFSR_CDB;

/* RACC_DMA_FILL_LFSR (opcode 0xC) */
typedef struct _FILL_LFSR_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    U32                         LfsrSeed[4];                        /* 0x20-0x2F */
    U64                         DestinationAddress;                 /* 0x30-0x37 */
    U8                          DestinationAttributes;              /* 0x38      */
    U8                          Reserved[7];                        /* 0x39-0x3F */
} FILL_LFSR_CDB, * PTR_FILL_LFSR_CDB;

/* RACC_DMA_GENERATE_HASH (opcode 0xD) */
typedef struct _GEN_HASH_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src;                                /* 0x20-0x2F */
    U64                         DestinationAddress;                 /* 0x30-0x37 */
    U8                          DestinationAttributes;              /* 0x38      */
    U8                          DestinationReserved[7];             /* 0x39-0x3F */
} GEN_HASH_CDB, * PTR_GEN_HASH_CDB;

/* RACC_DMA_GENERATE_HMAC (opcode 0xE) */
typedef struct _GEN_HMAC_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src;                                /* 0x20-0x2F */
    U64                         HashKeyPointer;                     /* 0x30-0x37 */
    U64                         Reserved1;                          /* 0x38-0x3F */
    U64                         DestinationAddress;                 /* 0x40-0x47 */
    U8                          DestinationAttributes;              /* 0x48      */
    U8                          DestinationReserved[7];             /* 0x49-0x4f */
} GEN_HMAC_CDB, * PTR_GEN_HMAC_CDB;

/* RACC_DMA_MOVE_MULTIPLY (opcode 0xF) */
typedef struct _MM_CDB
{
    RACC_DMA_CDB_HEADER         Header;                             /* 0-0x1F    */
    RACC_DMA_SOURCE             Src;                                /* 0x20-0x2F */
    RACC_DMA_DEST               Dst;                                /* 0x30-0x3F */
} MM_CDB, * PTR_MM_CDB;

/* DMAn Halt Error Attributes */
typedef struct _RACC_DMA_HALT_ERR_ATTR_REG
{
    union
    {
        struct
        {
            U32    ErrorType                 :1;                   /* 0     */
            U32    IllegalCDBFailType        :7;                   /* 1:7   */
            U32    EEDPGuardMiscomp          :1;                   /* 8     */
            U32    EEDPAppTagMiscomp         :1;                   /* 9     */
            U32    EEDPRefTagMiscomp         :1;                   /* 10    */
            U32    DPFAccessErrAgent         :4;                   /* 11:14 */
            U32    Reserved_15               :1;                   /* 15    */
            U32    ErrStatBitLog             :5;                   /* 16:20 */
            U32    Reserved_21_23            :3;                   /* 21:23 */
            U32    EEDPErrSourceNum          :8;                   /* 24:31 */
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_HALT_ERR_ATTR_REG, * PTR_RACC_DMA_HALT_ERR_ATTR_REG;

/* DMAn Halt Error Status */
typedef struct _RACC_DMA_HALT_ERR_STAT_REG
{
    union
    {
        struct
        {
            U32    IllegalCDB                :1;                   /* 0    */
            U32    CDBFetchErr               :1;                   /* 1     */
            U32    NextCDBPntrFetchErr       :1;                   /* 2     */
            U32    SourceSGFetchErr          :1;                   /* 3     */
            U32    DestSGFetchErr            :1;                   /* 4     */
            U32    CDBEncryKeyFetchCompErr   :1;                   /* 5     */
            U32    FillCompErr               :1;                   /* 6     */
            U32    DrainCompErr              :1;                   /* 7     */
            U32    CompQCompErr              :1;                   /* 8     */
            U32    ContentRAMParErr          :1;                   /* 9     */
            U32    KeyRAMParErr              :1;                   /* 10    */
            U32    DatRAMParErr              :1;                   /* 11    */
            U32    Modify1DatRAMParErr       :1;                   /* 12    */
            U32    Modify2DatRAMParErr       :1;                   /* 13    */
            U32    Modify3DatRAMParErr       :1;                   /* 14    */
            U32    Reserved_15               :1;                   /* 15    */
            U32    SGDrainCtxRAMParErr       :1;                   /* 16    */
            U32    SCFillCtxRAMParErr        :1;                   /* 17    */
            U32    EEDPMiscompHalt           :1;                   /* 18    */
            U32    DPFAccessErr              :1;                   /* 19    */
            U32    DIFFillCompErr            :1;                   /* 20    */
            U32    DIFDrainCompErr           :1;                   /* 21    */
            U32    DIFSourceSGFetchErr       :1;                   /* 22    */
            U32    DIFDestSGFetchErr         :1;                   /* 23    */
            U32    DIFFillRAMParErr          :1;                   /* 24    */
            U32    DIFDrainRAMParErr         :1;                   /* 25    */
            U32    XORZeroCheckFail          :1;                   /* 26    */
            U32    CheckLFSRFail             :1;                   /* 27    */
            U32    CheckPatternFail          :1;                   /* 28    */
            U32    Reserved_29_31            :3;                   /* 29:31 */
        } Bits;         U32 Word;
    } u;
} RACC_DMA_HALT_ERR_STAT_REG, * PTR_RACC_DMA_HALT_ERR_STAT_REG;

/* Completion Queue Entry */
typedef struct _COMPLETION_QUEUE_ENTRY
{
          U32      Status                    : 2;                  /* bits  0:1  */
          U32      LinkedList                : 1;                  /* bit   2    */
          U32      NoInterrupt               : 1;                  /* bit   3    */
          U32      ValidEntry                : 1;                  /* bit   4    */
          U32      LowerCP                   :27;                  /* bits  5:31 */
          U32      UpperCP;                                        /* bits 32:63 */
} COMPLETION_QUEUE_ENTRY, *PTR_COMPLETION_QUEUE_ENTRY;

#endif /* RAID_LIB_API_H */
