/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.       *
 *  The receipt of or possession of this file does not convey any rights   *
 *  to reproduce or disclose its contents or to manufacture, use, or sell  *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ubootLib.h
 *          Title:  Common header file for ubootLib standalone module
 *     Programmer:  Nilesh Wilanker
 *     Modified  :  Naresh Madhusudana
 *  Creation Date:  Aug 9, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  08/09/11  NW    Initial version.
 *  10/06/11  NM    Porting this file for Invader
 *  03/27/12  JM    Porting changes for Dual core library.
 *  07/12/12  AM    Root Complex OSS Changes.
 *
 * Description
 * ------------
 * File containing the common definitions for ubootLib standalone module
 *
 *--------------------------------------------------------------------------
 */

#ifndef _UBOOT_LIB_H
#define _UBOOT_LIB_H

#define STANDALONE_ENTRY              (0xFFD00000)

/* Standalone library entry points for Secondary U-boot. */
#define SEC_STANDALONE_ENTRY          (0xFFD00004)

/* Macro definition for different libraries*/
#define DDR3_LIB                      (1)
#define MINI_DMA_LIB                  (2)

/* DDR3 functions */
#define DDR3_INIT                     (1)
#define DDR3_SIZE                     (2)

/* MINI DMA functions */
#define MINI_DMA_RESET_DDR            (1)

/* Macro definition for Root Complex library */
#define RC_LIB                        (5)

/* Root Complex functions */
#define RC_IS_ENABLED                 (1)
#define RC_IS_LINK_UP                 (2)
#define RC_PERFORM_LINK_WIDTH_TRNG    (3)
#define RC_PERFORM_LINK_SPEED_TRNG    (4)
#define RC_CONFIGURE_POM              (5)
#define RC_CONFIGURE_PIM              (6)
#define RC_ENABLE_MSIX                (7)
#define RC_CONFIGURE_MSIX             (8)
#define RC_FIND_CAPABILITY            (9)
#define RC_CFG_READ_DWORD             (10)
#define RC_CFG_READ_WORD              (11)
#define RC_CFG_READ_BYTE              (12)
#define RC_CFG_WRITE_DWORD            (13)
#define RC_CFG_WRITE_WORD             (14)
#define RC_CFG_WRITE_BYTE             (15)
#define RC_MEM_READ_DWORD             (16)
#define RC_MEM_WRITE_DWORD            (17)
#define RC_READ_BACKDOOR              (18)
#define RC_WRITE_BACKDOOR             (19)
#define RC_GET_LINK_SPEED             (20)
#define RC_GET_LINK_WIDTH             (21)
#define RC_GET_PHY_ADDR               (22)
#define RC_GET_VIRT_ADDR              (23)
#define RC_PCI_BOARD_INIT             (24)

#define RC_NUM_FUNCTIONS              (24)


/*********************** For Dual Core feature in U-boot ***********************/
/* Macro definition for Dual core libraries */
#define DC_PRIMARY_UBOOT_LIB          (3)
#define DC_SECONDARY_UBOOT_LIB        (4)

#define DC_SUCCESS                    (0)
#define DC_FAIL                       (1)
#define DC_CORE_CHECK_FAIL            (2)  /* Core checking failure         */
#define DC_PPC0_PMU_FAIL              (3)  /* PMU0 initialize error         */
#define DC_PPC1_PMU_FAIL              (4)  /* PMU0 initialize error         */
#define DC_QNO_INVALID                (5)  /* Invalid QNO                   */
#define DC_Q_NOT_INIT                 (6)  /* Queue not initialized         */
#define DC_Q_FULL                     (7)  /* Queue is full                 */
#define DC_Q_RESET_REQUESTED          (8)  /* Reset on R/W Ptr is requested */
#define DC_INVALID_CORE               (9)  /* Invalid core                  */
#define DC_SEC_CORE_ENABLED           (10) /* Secondary Core is Enabled     */
#define DC_SEC_CORE_DISABLED          (11) /* Secondary Core is Disabled    */

/* Invalid U-Boot library check  */
#define INVALID_LIB                   (0xFFFFFFFF)

#define RD_BUF_MAX_SIZE               (50)

#define GET_DC_DB_STATUS              (1)
#define CLEAR_DC_DB_INT               (2)

/* Dual core library functions */
typedef enum _DC_FUNCTION_NAME
{
    DC_SEC_CORE_ENABLE = 1,
    DC_SEC_CORE_DISABLE,
    DC_SEC_CORE_STATUS,
    DC_SET_ENABLE_DB_INT,
    DC_GET_DB_STATUS,
    DC_SET_DB_INT,
    DC_GET_ERROR_STATUS,
    DC_WRITE_QP,
    DC_RESET_TXQ,
    DC_RESET_RXQ,
    DC_SET_COAL_INT_CNT,
    DC_SET_COAL_INT_TIMER,
    DC_INIT_PMU,
    PRIM_PMU_INT_HANDLER,
    SEC_PMU_INT_HANDLER
} DC_FUNCTION_NAME;

/* Dual core common structure for function and its arguments */
typedef struct _DC_FUNCTION
{
    DC_FUNCTION_NAME FunctionName;
    int              Argc;
    char             *PtrArgv[3];
} DC_FUNCTION, *PTR_DC_FUNCTION;

typedef volatile struct _DC_PMU_DB_REG
{
    unsigned int dbCntReg;
    unsigned int dbEnbReg;
    unsigned int dbAssReg;
    unsigned int dbEnbOthReg;
} DC_PMU_DB_REG, *PTR_DC_PMU_DB_REG;

#endif  /* _UBOOT_LIB_H */
