/***************************************************************************
 *                                                                         *
 *  Copyright 2010-2011 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ddr3.h
 *          Title:  DDR3 Training Code header file
 *     Programmer:  craigc
 *  Creation Date:  4/22/09
 *
 *
 * Description
 * ------------
 */



#ifndef DDR3_H_INCLUDED
#define DDR3_H_INCLUDED

/*
 **********************
 * Defines
 **********************
 */


#if (COBRA_SUPPORT == FALSE)
    #ifndef SASHW_DCR_MEM_CONTROLLER
    #define MSS_REG_ADDR 0x20000
    #else
    #define MSS_REG_ADDR (SASHW_DCR_MEM_CONTROLLER)
    #endif
#endif

#ifndef _DCR_OCM_ERR_CONTROL
#define _DCR_OCM_ERR_CONTROL 0xC019
#endif

#ifndef _DCR_OCM1_ERR_CONTROL
#define _DCR_OCM1_ERR_CONTROL 0xC059
#endif

#if(COBRA_SUPPORT == FALSE)

    /* ECC Scrub status */
    #define ECC_SCRUB_STATUS_FAILURE 1
    #define ECC_SCRUB_STATUS_SUCCESS 0

    #define DDR3_MAX_DP 9    /* range 0..8 - a DP is the physical hardmac - 8 data lanes + 1 ecc lane = 9 DPs */
    #define DDR3_ECC_DP 4

    #define DP_OFFSET   0x200   /* offset to each hardmacro (slice) in the phy */

#else /* Cobra support is TRUE -  */

    /*COBRA supports only 3 DPs */
    #define DDR3_MAX_DP 3
    #define DDR3_ECC_DP  2

    #define DP_OFFSET   0x800   /* offset to each hardmacro (slice) in the phy */

#endif /* Cobra Support*/

#define DP_SLICE(qqv1)  (qqv1 * DP_OFFSET)      /* macro to calculate the offset to the next slice */

#define DDR3_MAX_SECTION (DDR3_MAX_DP)  /* same range 0..8 - a section is a software byte-lane, or also known as a PHY DP - 8 data lanes + 1 ecc lane = 9 DPs */
#define DDR3_MAX_RANK 2     /* only support rank[0] and rank[1] */

#define DDR3_FAILING_SECTION_NOT_FOUND      0xFF  /*Initialize value, until a failing section is identified */

/*******************  DEBUG PRINT LEVELS  **************************************/
#ifndef ENABLE_DDR_PRINTS
#define ENABLE_DDR_PRINTS                   1  /* Generic prints, first level of debug.  Turn on before anything else */
#endif

#ifndef ENABLE_DDR_DEBUG_PRINTS
#define ENABLE_DDR_DEBUG_PRINTS             0  /* additional general debug prints, state-machine steps, etc., 2nd level of debug */
#endif

#ifndef ENABLE_DDR_REGISTER_DEBUG_PRINTS
#define ENABLE_DDR_REGISTER_DEBUG_PRINTS    0  /* every register write / read documented  (currently only used in ddriInitPalladiumFromRtlSim) */
#endif

#ifndef ENABLE_DDR_DUMP_SPD_DATA
#define ENABLE_DDR_DUMP_SPD_DATA            0  /* When SPD data is read from I2C, prints out every byte (IOP specific!) */
#endif

#ifndef ENABLE_DDR_QTRCYC_DEBUG_PRINTS
#define ENABLE_DDR_QTRCYC_DEBUG_PRINTS      0  /* Hardware Quarter Cycle training */
#endif

#ifndef ENABLE_DDR_LOAD_CHECK_DELAY_PRINTS     /* Load&Check delays, used in rl/wl/gt flow printing (i.e.  ENABLE_DDR_WLFLOW_DEBUG_PRINTS) */
#define ENABLE_DDR_LOAD_CHECK_DELAY_PRINTS  0  /* does nothing, if FLOW_DEBUG_PRINTS are off. If enabled, results in additional flow prints */
#endif                                         /* Added independently to reduce the number of prints seen by default when enabling flow prints */

#ifndef ENABLE_DDR_WLFLOW_DEBUG_PRINTS
#define ENABLE_DDR_WLFLOW_DEBUG_PRINTS      0  /* Write Leveling flow chart    */
#endif

#ifndef ENABLE_DDR_WLFINAL_DEBUG_PRINTS
#define ENABLE_DDR_WLFINAL_DEBUG_PRINTS     0  /* Write Leveling final results */
#endif

#ifndef ENABLE_DDR_PFFLOW_DEBUG_PRINTS
#define ENABLE_DDR_PFFLOW_DEBUG_PRINTS      0  /* Pre-amble finder flow chart    */
#endif

#ifndef ENABLE_DDR_PFBUFF_DEBUG_PRINTS
#define ENABLE_DDR_PFBUFF_DEBUG_PRINTS      0  /* Pre-amble finder buffer dumps  */
#endif

#ifndef ENABLE_DDR_PFFINAL_DEBUG_PRINTS
#define ENABLE_DDR_PFFINAL_DEBUG_PRINTS     0  /* Pre-amble finder final results */
#endif

#ifndef ENABLE_DDR_WLAFLOW_DEBUG_PRINTS
#define ENABLE_DDR_WLAFLOW_DEBUG_PRINTS     0  /* Write Leveling Adjustment flow chart */
#endif

#ifndef ENABLE_DDR_WLABUFFER_DEBUG_PRINTS
#define ENABLE_DDR_WLABUFFER_DEBUG_PRINTS   0  /* Write Leveling Adjustment buffer dumps */
#endif

#ifndef ENABLE_DDR_WLAFINAL_DEBUG_PRINTS
#define ENABLE_DDR_WLAFINAL_DEBUG_PRINTS    0  /* Write Leveling Adjustment flow chart   */
#endif

#ifndef ENABLE_DDR_GTFLOW_DEBUG_PRINTS
#define ENABLE_DDR_GTFLOW_DEBUG_PRINTS      0  /* Gate Training flow chart */
#endif

#ifndef ENABLE_DDR_GTFINAL_DEBUG_PRINTS
#define ENABLE_DDR_GTFINAL_DEBUG_PRINTS     0  /* Gate Training final results */
#endif

#ifndef ENABLE_DDR_RLFLOW_DEBUG_PRINTS
#define ENABLE_DDR_RLFLOW_DEBUG_PRINTS      0  /* Read Leveling flow */
#endif

#ifndef ENABLE_DDR_RLEDGE_DEBUG_PRINTS
#define ENABLE_DDR_RLEDGE_DEBUG_PRINTS      0  /* Read Leveling Data Eye */
#endif

#ifndef ENABLE_DDR_RLFINAL_DEBUG_PRINTS
#define ENABLE_DDR_RLFINAL_DEBUG_PRINTS     0  /* Read Leveling Final results*/
#endif

#ifndef ENABLE_DDR_STE882_DEBUG_PRINTS
#define ENABLE_DDR_STE882_DEBUG_PRINTS      0  /* RDIMM STE882 register buffer commands */
#endif

#ifndef ENABLE_DDR_RLRESP_DEBUG_PRINTS
#define ENABLE_DDR_RLRESP_DEBUG_PRINTS      0  /* Read leveling sweep prints*/
#endif

#ifndef ENABLE_DDR_TIMESTAMP_PRINTS
#define ENABLE_DDR_TIMESTAMP_PRINTS         0  /* Print timestamp at major points */
#endif

#ifndef ENABLE_DDR_DFIRLFLOW_DEBUG_PRINTS
#define ENABLE_DDR_DFIRLFLOW_DEBUG_PRINTS   0  /* DFI_RL FIFO Training flow */
#endif

#ifndef ENABLE_DDR_DFIRL_ITERATION_DEBUG_PRINTS
#define ENABLE_DDR_DFIRL_ITERATION_DEBUG_PRINTS     0  /* DFI_RL FIFO Training print each trial iteration */
#endif

#ifndef ENABLE_DDR_DFIRL_ECC_ERROR_DEBUG_PRINTS
#define ENABLE_DDR_DFIRL_ECC_ERROR_DEBUG_PRINTS     0  /* DFI_RL FIFO Training ECC errors */
#endif

#ifndef ENABLE_DDR_DFIRL_DMA_READ_TIME_DEBUG_PRINTS
#define ENABLE_DDR_DFIRL_DMA_READ_TIME_DEBUG_PRINTS 0  /* DFI_RL FIFO Training DMA engine READ execution time prints */
#endif

#ifndef ENABLE_DDR_DFIRLFINAL_DEBUG_PRINTS
#define ENABLE_DDR_DFIRLFINAL_DEBUG_PRINTS          0  /* DFI_RL FIFO Training final results */
#endif

/*#defines for READ FIFO (DFI_RL) training put here due to _DDR3_GLOBAL_VARS requiring them*/
#define DDR3_DFIRL_PHY_MAX_DELAY (MSS_DP_TOP_PHY_CFG_0_FIXED_RL_MASK >> MSS_DP_TOP_PHY_CFG_0_FIXED_RL_SHIFT)
#define DDR3_DFIRL_MC_MAX_DELAY (MSS_MC_DEN_CTL_113_TDFI_PHY_RDLAT_MASK >> MSS_MC_DEN_CTL_113_TDFI_PHY_RDLAT_SHIFT)

/* MC dfi-rl reg has fewer bits than the PHY dfi-rl reg - so make it the absolute maximum - can't test PHY values the MC doesn't support */
#define DDR3_DFIRL_MAX_DELAY (DDR3_DFIRL_MC_MAX_DELAY)
/* the array needs to hold all possible RL delays from zero to max */
#define DDR3_DFIRL_DELAY_ARRAY_SIZE (DDR3_DFIRL_MAX_DELAY + 1)
/* validation only option - enable this to compare on a per-dq-bit basis */
#define DDR3_DFIRL_ENABLE_COMPARE_PER_DQ_BIT_BASIS 0  /* !!! not optimized for speed !!! */
/* DQ range from 0..63 */
#define DDR3_DFIRL_MAX_DQ_BITS 64

/* ECC error types */
#define DDR_SINGLEBIT_ECC_ERROR 0x0
#define DDR_MULTIBIT_ECC_ERROR  0x1
/* ECC forcing Status */
#define DDR_FORCE_ECC_SUCCESS 0
#define DDR_FORCE_ECC_FAILURE 1

/* defines for functions */
#define DDR3LIB_INW(addr)       MoveFromDCR(addr)
#define DDR3LIB_OUTW(addr,data) MoveToDCR(data,addr)

#if ENABLE_DDR_PRINTS
  #define DDRINIT_PRINTF0(format)              iopinitDiagConsolePrint0((char*)format)
  #define DDRINIT_PRINTF1(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define DDRINIT_PRINTF2(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define DDRINIT_PRINTF3(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define DDRINIT_PRINTF4(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define DDRINIT_PRINTF0(format)
  #define DDRINIT_PRINTF1(format,a)
  #define DDRINIT_PRINTF2(format,a,b)
  #define DDRINIT_PRINTF3(format,a,b,c)
  #define DDRINIT_PRINTF4(format,a,b,c,d)
#endif

#if ENABLE_DDR_TIMESTAMP_PRINTS
  #define DDR_TIMESTAMP_PRINTF1(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
#else
  #define DDR_TIMESTAMP_PRINTF1(format,a)
#endif


#if ENABLE_DDR_DEBUG_PRINTS
  /* debug prints are done this way in case the iop uses a ringbuffer - params have to be spelled out explicitly in that case */
  #define DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define DEBUG_PRINTF0_DDR(format)
  #define DEBUG_PRINTF1_DDR(format,a)
  #define DEBUG_PRINTF2_DDR(format,a,b)
  #define DEBUG_PRINTF3_DDR(format,a,b,c)
  #define DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_REGISTER_DEBUG_PRINTS
  #define REG_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define REG_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define REG_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define REG_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define REG_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define REG_DEBUG_PRINTF0_DDR(format)
  #define REG_DEBUG_PRINTF1_DDR(format,a)
  #define REG_DEBUG_PRINTF2_DDR(format,a,b)
  #define REG_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define REG_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_QTRCYC_DEBUG_PRINTS
  #define QC_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define QC_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define QC_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define QC_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define QC_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define QC_DEBUG_PRINTF0_DDR(format)
  #define QC_DEBUG_PRINTF1_DDR(format,a)
  #define QC_DEBUG_PRINTF2_DDR(format,a,b)
  #define QC_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define QC_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_MC_DEBUG_PRINTS
  #define MC_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define MC_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define MC_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define MC_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define MC_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define MC_DEBUG_PRINTF0_DDR(format)
  #define MC_DEBUG_PRINTF1_DDR(format,a)
  #define MC_DEBUG_PRINTF2_DDR(format,a,b)
  #define MC_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define MC_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_PFFLOW_DEBUG_PRINTS
  #define PF_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define PF_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define PF_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define PF_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define PF_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define PF_DEBUG_PRINTF0_DDR(format)
  #define PF_DEBUG_PRINTF1_DDR(format,a)
  #define PF_DEBUG_PRINTF2_DDR(format,a,b)
  #define PF_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define PF_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_WLFLOW_DEBUG_PRINTS
  #define WL_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define WL_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define WL_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define WL_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define WL_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define WL_DEBUG_PRINTF0_DDR(format)
  #define WL_DEBUG_PRINTF1_DDR(format,a)
  #define WL_DEBUG_PRINTF2_DDR(format,a,b)
  #define WL_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define WL_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_WLAFLOW_DEBUG_PRINTS
  #define WLA_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define WLA_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define WLA_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define WLA_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define WLA_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define WLA_DEBUG_PRINTF0_DDR(format)
  #define WLA_DEBUG_PRINTF1_DDR(format,a)
  #define WLA_DEBUG_PRINTF2_DDR(format,a,b)
  #define WLA_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define WLA_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_GTFLOW_DEBUG_PRINTS
  #define GT_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define GT_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define GT_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define GT_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define GT_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define GT_DEBUG_PRINTF0_DDR(format)
  #define GT_DEBUG_PRINTF1_DDR(format,a)
  #define GT_DEBUG_PRINTF2_DDR(format,a,b)
  #define GT_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define GT_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_RLFLOW_DEBUG_PRINTS
  #define RL_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define RL_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define RL_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define RL_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define RL_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define RL_DEBUG_PRINTF0_DDR(format)
  #define RL_DEBUG_PRINTF1_DDR(format,a)
  #define RL_DEBUG_PRINTF2_DDR(format,a,b)
  #define RL_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define RL_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_STE882_DEBUG_PRINTS
  #define STE882_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
  #define STE882_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
  #define STE882_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
  #define STE882_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
  #define STE882_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define STE882_DEBUG_PRINTF0_DDR(format)
  #define STE882_DEBUG_PRINTF1_DDR(format,a)
  #define STE882_DEBUG_PRINTF2_DDR(format,a,b)
  #define STE882_DEBUG_PRINTF3_DDR(format,a,b,c)
  #define STE882_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

#if ENABLE_DDR_DFIRLFLOW_DEBUG_PRINTS
    #define DFIRL_DEBUG_PRINTF0_DDR(format)              iopinitDiagConsolePrint0((char*)format)
    #define DFIRL_DEBUG_PRINTF1_DDR(format,a)            iopinitDiagConsolePrint1((char*)format,(U32)a)
    #define DFIRL_DEBUG_PRINTF2_DDR(format,a,b)          iopinitDiagConsolePrint2((char*)format,(U32)a,(U32)b)
    #define DFIRL_DEBUG_PRINTF3_DDR(format,a,b,c)        iopinitDiagConsolePrint3((char*)format,(U32)a,(U32)b,(U32)c)
    #define DFIRL_DEBUG_PRINTF4_DDR(format,a,b,c,d)      iopinitDiagConsolePrint4((char*)format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
    #define DFIRL_DEBUG_PRINTF0_DDR(format)
    #define DFIRL_DEBUG_PRINTF1_DDR(format,a)
    #define DFIRL_DEBUG_PRINTF2_DDR(format,a,b)
    #define DFIRL_DEBUG_PRINTF3_DDR(format,a,b,c)
    #define DFIRL_DEBUG_PRINTF4_DDR(format,a,b,c,d)
#endif

/*
 **********************
 * Typedefs
 **********************
 */
typedef enum _DDR_DIMM_WIDTH
{
    FULL_64_OR_72_BIT_WIDTH,
    HALF_32_OR_40_BIT_WIDTH,
    QUARTER_16_OR_20_BIT_WIDTH
} DDR_DIMM_WIDTH;

typedef enum _DDR_ECC_MODE
{
    ECC_DISABLED,
    ECC_REPORT_NO_CORRECT,
    ECC_DISABLED_HARDWIRE,
    ECC_ENABLED
} DDR_ECC_MODE;

typedef enum _DDR3_MR1_RTT_NOM_TYPE
{                           /* M9 M6 M2 */
    MR1_RTT_NOM_DISABLED,   /*  0  0  0 */
    MR1_RTT_NOM_60_OHM,     /*  0  0  1 */
    MR1_RTT_NOM_120_OHM,    /*  0  1  0 */
    MR1_RTT_NOM_40_OHM,     /*  0  1  1 */
    MR1_RTT_NOM_20_OHM,     /*  1  0  0 */
    MR1_RTT_NOM_30_OHM,     /*  1  0  1 */
    MR1_RTT_NOM_RESERVED1,  /*  1  1  0 */
    MR1_RTT_NOM_RESERVED2   /*  1  1  1 */
} DDR3_MR1_RTT_NOM_TYPE;

typedef enum _DDR3_MR1_ODS_TYPE
{                       /* M5 M1 */
    MR1_ODS_40_OHM,     /*  0  0 */
    MR1_ODS_34_OHM,     /*  0  1 */
    MR1_ODS_RESERVED1,  /*  1  0 */
    MR1_ODS_RESERVED2   /*  1  1 */
} DDR3_MR1_ODS_TYPE;

typedef enum _DDR3_MR2_RTT_WR_TYPE
{                          /* M10 M9 */
    MR2_RTT_WR_DISABLED,   /*   0  0 */
    MR2_RTT_WR_RQZ_DIV_4,  /*   0  1 */
    MR2_RTT_WR_RQZ_DIV_2,  /*   1  0 */
    MR2_RTT_WR_RESERVED    /*   1  1 */
} DDR3_MR2_RTT_WR_TYPE;



typedef struct _DDR3_GLOBAL_VARS
{
    U32 HwDcrBaseAddress;
    U8 *ptrDdrMemRank0;
    U8 *ptrDdrMemRank1;
    U8 *ptrDdrMemRank0NoCache;
    U8 *ptrDdrMemRank1NoCache;
    U32 MemFreq;
    U32 QtrCycleStepsAveraged;
    U32 QtrCycleInPs;
    U32 AveTapDelayInFs;
    U32 tCL;
    U32 wrlat;
    U32 subtractRl;
    U32 failingSection;
    U16 refreshPerZq;
#if (COBRA_SUPPORT == FALSE)
    U16 ZqcsRotate;
#endif
    U8 *ptrSrcAddr;
    U8 *ptrDstAddr;
    U32 InitFlags;
#if (COBRA_SUPPORT == FALSE)
    BOOL HalfMode;
#endif
    U32 NumberOfRanks;
    BOOL IsEccEnabled;
    BOOL IsDimmRegistered;
    U32 PreambleDelay[DDR3_MAX_RANK][DDR3_MAX_SECTION];
    U32 ReadLevelingPosDqsDelays[DDR3_MAX_RANK][DDR3_MAX_SECTION];
    U32 ReadLevelingNegDqsDelays[DDR3_MAX_RANK][DDR3_MAX_SECTION];
    U32 DfiRlPassCount[DDR3_MAX_RANK][DDR3_DFIRL_DELAY_ARRAY_SIZE];
    U32 DfiRlFailCount[DDR3_MAX_RANK][DDR3_DFIRL_DELAY_ARRAY_SIZE];
    U32 DfiRlPassCountEcc[DDR3_MAX_RANK][DDR3_DFIRL_DELAY_ARRAY_SIZE];
    U32 DfiRlFailCountEcc[DDR3_MAX_RANK][DDR3_DFIRL_DELAY_ARRAY_SIZE];

    /* Note: This will use up a lot of memory, not recommended for production code */
    #if (DDR3_DFIRL_ENABLE_COMPARE_PER_DQ_BIT_BASIS==1)
    U32 DfiRlPassCountRank0[DDR3_DFIRL_DELAY_ARRAY_SIZE][DDR3_DFIRL_MAX_DQ_BITS]; /* (0x20 * 0x40 == 0x1 0000) * sizeof U32 == 0x4 0000 */
    U32 DfiRlPassCountRank1[DDR3_DFIRL_DELAY_ARRAY_SIZE][DDR3_DFIRL_MAX_DQ_BITS];
    U32 DfiRlFailCountRank0[DDR3_DFIRL_DELAY_ARRAY_SIZE][DDR3_DFIRL_MAX_DQ_BITS];
    U32 DfiRlFailCountRank1[DDR3_DFIRL_DELAY_ARRAY_SIZE][DDR3_DFIRL_MAX_DQ_BITS];
    #endif

    U8  SiliconRev;
    U32 LastCorrectableEccError_DataWordHigh;
    U32 LastCorrectableEccError_DataWordLow;
    U32 LastCorrectableEccError_AddressHigh;
    U32 LastCorrectableEccError_AddressLow;
    U32 LastCorrectableEccError_SrcId;
    U32 LastCorrectableEccError_Syndrome;
    U32 LastUncorrectableEccError_DataWordHigh;
    U32 LastUncorrectableEccError_DataWordLow;
    U32 LastUncorrectableEccError_AddressHigh;
    U32 LastUncorrectableEccError_AddressLow;
    U32 LastUncorrectableEccError_SrcId;
    U32 LastUncorrectableEccError_Syndrome;
    U32 CoreInterruptAlwaysMask;
    BOOL IsSwLevelingModeEnabled;
    BOOL UseHighTempRefSettings;
    BOOL IsDimmAddressMirrored;
    BOOL IsModuleThermalSensorPresent;
    DDR3_MR1_RTT_NOM_TYPE RttNom;
    DDR3_MR1_ODS_TYPE Ods;
    DDR3_MR2_RTT_WR_TYPE RttWr;
} DDR3_GLOBAL_VARS;




/*
 **********************
 * Inlines
 **********************
 */

/*  getRegVal -  Extract a value from a register using a bitmask.  Creates the shift on its own.
 *
 *  operation - (mask & -mask) gets the least signifcant bit from the mask. When compiled with a
 *              constant the compiler breaks it down to a shift operation(no divide or multiply)
 *              5-6 instructions for a get/set...same as conventional (inw(addr) & mask)>>shift
 *
 *  WARNING   -  mask must have *only* consecutive 1's
 */
INLINE U32 getRegVal(U32 addr, U32 mask)
{
    U32 value = ((DDR3LIB_INW(addr) & mask) / (mask & -mask));
    REG_DEBUG_PRINTF3_DDR("READ (getRegVal) addr=0x%x mask=0x%x value=0x%x\n", addr, mask, (value*(mask & -mask)));
    return value;
}

/*  setRegVal -   Perform a read modify write of a register using a bitmask and creating the shift
 *                operation (mask & -mask) -- See above.  Same as conventional outw(addr, inw(addr) & ~x | y<<shift)
 *
 *  WARNING   -  mask must have *only* consecutive 1's
 */
INLINE void setRegVal(U32 addr, U32 mask, U32 value)
{
    REG_DEBUG_PRINTF3_DDR("WRITE (setRegVal) addr=0x%x mask=0x%x value=0x%x\n", addr, mask, (value*(mask & -mask)));
    DDR3LIB_OUTW(addr, ((DDR3LIB_INW(addr) & ~mask) | (value * (mask & -mask))));
}

/*  SetRegBit -   Perform a read modify write of a register and set/clear bits
 */
INLINE void setRegBits(U32 addr, U32 bitmask)
{
    REG_DEBUG_PRINTF3_DDR("WRITE (setRegBits) addr=0x%x mask=0x%x value=0x%x\n", addr, bitmask, bitmask);
    DDR3LIB_OUTW(addr, DDR3LIB_INW(addr) | bitmask);
}

/*  ClrRegBit -  (mask & -mask) -- See above.  non-consecutvie 1's allowed(unlike above two)
 */
INLINE void clrRegBits(U32 addr, U32 bitmask)
{
    REG_DEBUG_PRINTF3_DDR("WRITE (clrRegBits) addr=0x%x mask=0x%x value=0x%x\n", addr, bitmask, 0);
    DDR3LIB_OUTW(addr, DDR3LIB_INW(addr) & ~bitmask);
}



/*
* Byte-swap 4-byte quantity (U32)
*
* Therefore, using the following statement:
*    x = SWAB4(0x12345678);
* x would then countain 0x78563412
*
*/
INLINE U32 DDR3LIB_SWAB4(U32 value)
{
    U32 t;

    t = value ^ ((value <<16) | (value >> 16));
    t &= ~0x00ff0000;
    value = (value << 24) | (value >> 8);
    return (value ^ (t >> 8));
}




/*
 **********************
 * Prototypes
 **********************
 */
void ddr3_printFailingLane(void);
#if (COBRA_SUPPORT == FALSE)
U32 ddr3_ConvDpToSection(U32 dp);
U32 ddr3_ConvSectionToDp(U32 section);
#endif

/*
 **********************
 * Globals
 **********************
 */
extern DDR3_GLOBAL_VARS gDdr3;





#endif  /* if not defined DDR3_H_INCLUDED */
