/***************************************************************************
 *                                                                         *
 *                                                                         *
 *  Copyright 2008-2012 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  minidma.h
 *          Title:  Definitions for DDR training DMA engine routines
 *  Creation Date:  April 21, 2009
 *
 *
 * Description
 * ------------
 *
 *    This software module contains the functionality needed to perform
 *    DMA operations utilizing the DMA Engine (Katana).
 *
 *-------------------------------------------------------------------------
 */

#ifndef __MINIDMA_H
#define __MINIDMA_H

#ifndef MINIDMA_LIBRARY_VERSION
#define MINIDMA_LIBRARY_VERSION   (0x00FA0200) /* This is the value to update */
#endif

#define CHIP_VERSION 0x3108A0
/*
 * MINIDMA compile options
 */
#define MINIDMA_USES_WRITE_XOR          (0)
#define DDRINIT_TEST_MINI_DMA           (0)
#if  DDRINIT_TEST_MINI_DMA
U32 testMiniDma(U32 keepGoing, U32 stopOnError,DDRINIT_MEM_INFO  *PtrMem);
#endif

/*
 * MINIDMA #defs
 */
#define MINI_DMA_FAULT_NO_CDBS          0x9100 //needs to be moved, verify value
#define MINI_DMA_MAX_CDBS               32
#define MINI_DMA_COMP_Q_SIZE            32
#define MINI_DMA_STATUS_NORMAL          RACC_CQ_STATUS_NORMAL
#define MINI_DMA_STATUS_SOURCE_ZERO     RACC_CQ_STATUS_SOURCE_ZERO
#define MINI_DMA_STATUS_MISCOMPARE      RACC_CQ_STATUS_MISCOMPARE
#define MINI_DMA_STATUS_FAILURE         (MINI_DMA_STATUS_MISCOMPARE + 1)

#ifdef MINI_DMA_DEBUG
    #define MINI_DMA_TIMEOUT_US             1000000         //more time when we're printing debug info inline
#else
    #define MINI_DMA_TIMEOUT_US             10000
#endif

#define DDRINIT_SRAM_ADDR_TO_PLB_ADDR_LOW(x)            ((U32)(x))
#define DDRINIT_SRAM_ADDR_TO_PLB_ADDR_HIGH(addr)        (0)
#define DDRINIT_MAX_DRAM_SIZE_BYTES                     (0x80000000) /* 2 GB */
#define DDRINIT_MAX_DRAM_SIZE_BYTES_MASK                (DDRINIT_MAX_DRAM_SIZE_BYTES - 1)

#ifndef MAKE64
#define  MAKE64(HIGH,LOW) make64((U32)(LOW),(U32)(HIGH))
#define GET64_LOW(x)                            (x.Low)
#define GET64_HIGH(x)                           (x.High)

INLINE U64 make64(U32 Low, U32 High) {
    U64 Scratch;
    Scratch.Low = Low;
    Scratch.High = High;
    return Scratch;
}
#endif

typedef struct _DDR3_MINI_DMA_INFO
{
    /*
     * These define DMA-able regions (EA 1:1) needed for
     * CDB's, completion queues et al
     */
    RACC_CDB_UNION         *ptrCDBs;        /* Pointer to array of CDB's used for DMA */
    COMPLETION_QUEUE_ENTRY *ptrCompQ;       /* Poitner to array of completion queue entries */
    PTR_RACC_DMA_REGS      ptrRaidAccRegs;   /* base of RaidAcc dma register */

} DDR3_MINI_DMA_INFO;

/*External functions provided by user of library. These need to be moved to common interface header file*/
extern void iopinitDiagConsolePrint0(char * FmtStr);
extern void iopinitDiagConsolePrint1(char * FmtStr, U32 Arg1);
extern void iopinitDiagConsolePrint2(char * FmtStr, U32 Arg1, U32 Arg2 );
extern void iopinitDiagConsolePrint3(char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3 );
extern void iopinitDiagConsolePrint4(char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4 );
extern U32  iopinitGetUSec(void);
extern void * iopinitMemSet(void *  Ptr1, U32 Var, U32 Len);
extern void iopinitFault(U32 faultCode);


/* TO DO: RC find another place for these.  Is 0x4 the best page for everybody? */
#define PLB_ADDR_XOR_HIGH                       (0x40000000)
#define PLB_ADDR_XOR_LOW                        (0x00000000)
#define WRITE_XOR_ENABLE                        (0x00000100)
#define DDR_ADDR_TO_HB_PLB_ADDR_LOW(addr)       ((U32)(addr) & DDRINIT_MAX_DRAM_SIZE_BYTES_MASK)

#define MINIDMA_SET_PLB_SLAVE_1_ERROR_LOG(x)    MoveToDCR((x),RACC_DCR_SLAVE_ERROR_LOG1)
#define MINIDMA_SET_PLB_SLAVE_0_ERROR_LOG(x)    MoveToDCR((x),RACC_DCR_SLAVE_ERROR_LOG0)

typedef struct _MINI_DMA_XFER
{
    RACC_CDB_UNION *pDdbChain;
    U8   status;
    U8   failedOpcode;
} MINI_DMA_XFER;

INLINE U64 miniDmaGenWriteXorPhysAddr(void *buffer, U32 multiplier)
{
    /* multiplier for 1st block is applied to the source data,
     * with the result XOR'd (assumed multiplier of 1) to destination
     */
    return MAKE64(PLB_ADDR_XOR_HIGH | WRITE_XOR_ENABLE | (multiplier<<16),
                  DDR_ADDR_TO_HB_PLB_ADDR_LOW(buffer));
}

INLINE U32 miniDmaGetLocalAddrSpaceFromBuffer(RACC_DMA_SOURCE *pBuf)
{
    /* This assumes DDR is mapped starting at zero, and up to 2GB.
       It also accounts for the DDR alias region (0x4_xxxx_xxxx to 0x7_xxxx_xxxx) */
    if ((pBuf->SourceAddress.Low < DDRINIT_MAX_DRAM_SIZE_BYTES) ||  /* 0 to 2GB */
        ((pBuf->SourceAddress.High & 0xC) == 0x4))  /* alias region (16GB) */
        return RACC_DMA_ADDRESS_MEM;
    else
        return RACC_DMA_ADDRESS_PLB;
}

/*
 * MiniDMA Library API functions
 */
U32     miniDmaLibInfo(U32 *PtrCdbBufferSize, U32 *PtrCompletionQueueSize);
void    miniDmaAddCopy(MINI_DMA_XFER *pDmaXfer, U64 source, U64 dest, U32 count);
void    miniDmaAddCopyWithWxor(MINI_DMA_XFER *pDmaXfer, U64 source, U64 dest, U32 count);
void    miniDmaInit(DDR3_MINI_DMA_INFO  *PtrMem);
void    miniDmaOpenXfer(MINI_DMA_XFER *pDmaXfer);
void    miniDmaAddFill(MINI_DMA_XFER *pDmaXfer, U64 dest, U32 count, U32 pattern);
void    miniDmaAddCheck(MINI_DMA_XFER *pDmaXfer, U64 dest, U32 count, U32 pattern);
U32     miniDmaIssueXferAndWait(MINI_DMA_XFER *pDmaXfer, U32 TimeoutUsec);
void    miniDmaDisableAndCleanUp(void);
void    miniDmaAddFillEx(MINI_DMA_XFER *pDmaXfer, U64 dest, U32 count, U32 patterns[4]);
void    miniDmaAddCheckEx(MINI_DMA_XFER *pDmaXfer, U64 dest, U32 count, U32 patterns[4]);
void    miniDmaAddFillLFSR(MINI_DMA_XFER *pDmaXfer, U64 dest, U32 count, U32 seed[4], U32 invert);
void    miniDmaAddCheckLFSR(MINI_DMA_XFER *pDmaXfer, U64 dest, U32 count, U32 seed[4], U32 invert);
void    miniDmaAddMemRead(MINI_DMA_XFER *pDmaXfer, U64 source, U32 count);
U32     miniDmaTimeoutOccurred(U32 startTime, U32 timeoutValue);

/* If green hills and powerpc */
#if defined __ghs__
#if defined(__ppc) || defined(__PPC__)

/***********************
 * Assembly
 ***********************/
/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     miniDmaDisableIrqs
*
* SUMMARY:  This subroutine disables the
*           PPC Decrmenter, Critical and External interrupts
*
* ARGUMENTS:
*   NONE
*
* RETURNS:
*   -   Irq's that were cleared by this routine
*
* NOTES:
*   R3-R6 are all volatile so it is OK for this routine to destroy them
----------------------------------------------------------------------------*/
asm U32 miniDmaDisableIrqs(void)
{
        /* Disable all PPC regular interrupts */
        mfmsr   r3
        mr      r5,r3
        mr      r6,r3
        lis     r4, %hi(0x00029000)             /* PPC Machine Check, Critical, and External interrupts */
        ori     r4, r4, %lo(0x00029000)
        and     r6,r6,r4                        /* keep current state to return */
        andc    r5, r5, r4
        mtmsr   r5                              /* Turn off 476 irqs */
        /* Also disable PPC decrementer interrupt */
        mftcr   r3
        lis     r4, %hi(0x04000000)             /* Decrementer interrupt*/
        ori     r4, r4, %lo(0x04000000)
        and     r5,r3,r4                        /* Retrieve DIE state */
        or      r5,r6,r5                        /* Or results */
        andc    r3,r3,r4
        mttcr   r3                              /* turn off DIE irq */
        mr      r3,r5                           /* Set up return register r3 */
}
/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     miniDmaDisableIrqs
*
* SUMMARY:  This subroutine re-enables the
*           PPC Decrmenter, Critical and External interrupts
*
* ARGUMENTS:
*   IrqMask (r3) - This set of DIE and 440 IRQ bits to enable
*
* RETURNS:
*   NONE
*
* NOTES:
*   R3-R5 are all volatile so it is OK for this routine to destroy them
----------------------------------------------------------------------------*/
asm void miniDmaEnableIrqs(IrqMask)
{

%reg IrqMask
        mr      r3, IrqMask
        /* Also disable PPC decrementer interrupt */
        lis     r4, %hi(0x04000000)
        ori     r4, r4, %lo(0x04000000)
        and.    r5,r3,r4
        beq     EnableOtherIrqs /* DIE Bit not set, branch */
        mftcr   r5
        or      r5,r5,r4
        mttcr   r5              /* Set DIE bit */
EnableOtherIrqs:
        /* Re-enable all the interrupts indicated by the IrqMask (r3) */
        mfmsr   r5
        lis     r4, %hi(0x00029000)        /* PPC Machine Check, Critical, and External interrupts */
        ori     r4, r4, %lo(0x00029000)
        and.    r3,r3,r4           /* Mask off bits not supported from IrqMask*/
        beq     DoneWithEnableIrqs /* Branch if no irq's to enable */
        or      r5, r5, r3
        mtmsr   r5
DoneWithEnableIrqs:

}
#endif
#endif /*endif if ppc and GHS*/

#endif /*minidma.h */

