/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibDebug.h
 *          Title:  USB library debug definitions common header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains definitions for the USB stack debug support.
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIBDEBUG_H_
#define _USBLIBDEBUG_H_

#if (IOC_USB_SUPPORT )

/* ************************************************************************** */
/* Various enable/disable flags. 1 = Enabled; 0 = Diabled                     */
/* ************************************************************************** */

#ifndef USB_PRINT_ENABLE_GENERIC
#define USB_PRINT_ENABLE_GENERIC                            1
#endif

#ifndef USB_PRINT_ENABLE_HAL
#define USB_PRINT_ENABLE_HAL                                1
#endif

#ifndef USB_PRINT_ENABLE_PROTOCOL
#define USB_PRINT_ENABLE_PROTOCOL                           1
#endif

#ifndef USB_PRINT_ENABLE_DEBUG_LOW
#define USB_PRINT_ENABLE_DEBUG_LOW                          1
#endif

#ifndef USB_PRINT_ENABLE_DEBUG_MEDIUM
#define USB_PRINT_ENABLE_DEBUG_MEDIUM                       1
#endif

#ifndef USB_PRINT_ENABLE_DEBUG_HIGH
#define USB_PRINT_ENABLE_DEBUG_HIGH                         1
#endif

#ifndef USB_PRINT_ENABLE_DEBUG_ERR
#define USB_PRINT_ENABLE_DEBUG_ERR                          1   /* Always keep enabled */
#endif

#ifndef USB_PRINT_ENABLE_DEBUG_WRN
#define USB_PRINT_ENABLE_DEBUG_WRN                          1
#endif

#ifndef USB_PRINT_ENABLE_DEBUG_DATA
#define USB_PRINT_ENABLE_DEBUG_DATA                         1
#endif

#ifndef USB_PRINT_ENABLE_HOST
#define USB_PRINT_ENABLE_HOST                               1
#endif

#ifndef USB_PRINT_ENABLE_OTG
#define USB_PRINT_ENABLE_OTG                                1
#endif

#ifndef USB_PRINT_ENABLE_STACK
#define USB_PRINT_ENABLE_STACK                              1
#endif

#ifndef USB_PRINT_ENABLE_MASS_STOR_CLASS
#define USB_PRINT_ENABLE_MASS_STOR_CLASS                    1
#endif

/* ************************************************************************** */
/* Various print layer prefixes                                               */
/* ************************************************************************** */

#define USB_LAYER_PREFIX_GENERIC                " USB GENERAL: "
#define USB_LAYER_PREFIX_HAL                    "     USB HAL: "
#define USB_LAYER_PREFIX_PROTOCOL               "USB PROTOCOL: "
#define USB_LAYER_PREFIX_PROTOCOL_SCSI          "    USB SCSI: "
#define USB_LAYER_PREFIX_STACK                  "   USB STACK: "
#define USB_LAYER_PREFIX_HOST                   "    USB HOST: "
#define USB_LAYER_PREFIX_OTG                    "     USB OTG: "
#define USB_LAYER_PREFIX_DEV_MSC                " USB DEV MSC: "
#define USB_LAYER_PREFIX_DBG_LOW                " USB DBG LOW: "
#define USB_LAYER_PREFIX_DBG_MED                " USB DBG MED: "
#define USB_LAYER_PREFIX_DBG_HGH                " USB DBG HGH: "
#define USB_LAYER_PREFIX_DATA                   "USB DBG DATA: "
#define USB_LAYER_PREFIX_ERR                    "!!!    ERROR: "
#define USB_LAYER_PREFIX_WRN                    "***  WARNING: "

/* ************************************************************************** */
/* Function mappings                                                          */
/* ************************************************************************** */

void iopiDiagConsolePrint0(char * FmtStr);
void iopiDiagConsolePrint1(char * FmtStr, U32 Arg1);
void iopiDiagConsolePrint2(char * FmtStr, U32 Arg1, U32 Arg2 );
void iopiDiagConsolePrint3(char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3 );
void iopiDiagConsolePrint4(char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4 );

#if USB_PRINT_ENABLE_GENERIC
  #define USB_GENERIC_PRINT0(format)                iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_GENERIC format)
  #define USB_GENERIC_PRINT1(format,a)              iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_GENERIC format,(U32)a)
  #define USB_GENERIC_PRINT2(format,a,b)            iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_GENERIC format,(U32)a,(U32)b)
  #define USB_GENERIC_PRINT3(format,a,b,c)          iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_GENERIC format,(U32)a,(U32)b,(U32)c)
  #define USB_GENERIC_PRINT4(format,a,b,c,d)        iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_GENERIC format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_GENERIC_PRINT0(format)
  #define USB_GENERIC_PRINT1(format,a)
  #define USB_GENERIC_PRINT2(format,a,b)
  #define USB_GENERIC_PRINT3(format,a,b,c)
  #define USB_GENERIC_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_HAL
  #define USB_HAL_PRINT0(format)                    iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_HAL format)
  #define USB_HAL_PRINT1(format,a)                  iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_HAL format,(U32)a)
  #define USB_HAL_PRINT2(format,a,b)                iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_HAL format,(U32)a,(U32)b)
  #define USB_HAL_PRINT3(format,a,b,c)              iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_HAL format,(U32)a,(U32)b,(U32)c)
  #define USB_HAL_PRINT4(format,a,b,c,d)            iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_HAL format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_HAL_PRINT0(format)
  #define USB_HAL_PRINT1(format,a)
  #define USB_HAL_PRINT2(format,a,b)
  #define USB_HAL_PRINT3(format,a,b,c)
  #define USB_HAL_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_PROTOCOL
  #define USB_PROTO_PRINT0(format)                  iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_PROTOCOL format)
  #define USB_PROTO_PRINT1(format,a)                iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_PROTOCOL format,(U32)a)
  #define USB_PROTO_PRINT2(format,a,b)              iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_PROTOCOL format,(U32)a,(U32)b)
  #define USB_PROTO_PRINT3(format,a,b,c)            iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_PROTOCOL format,(U32)a,(U32)b,(U32)c)
  #define USB_PROTO_PRINT4(format,a,b,c,d)          iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_PROTOCOL format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_PROTO_PRINT0(format)
  #define USB_PROTO_PRINT1(format,a)
  #define USB_PROTO_PRINT2(format,a,b)
  #define USB_PROTO_PRINT3(format,a,b,c)
  #define USB_PROTO_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_DEBUG_LOW
  #define USB_DBGLOW_PRINT0(format)                 iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_DBG_LOW format)
  #define USB_DBGLOW_PRINT1(format,a)               iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_DBG_LOW format,(U32)a)
  #define USB_DBGLOW_PRINT2(format,a,b)             iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_DBG_LOW format,(U32)a,(U32)b)
  #define USB_DBGLOW_PRINT3(format,a,b,c)           iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_DBG_LOW format,(U32)a,(U32)b,(U32)c)
  #define USB_DBGLOW_PRINT4(format,a,b,c,d)         iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_DBG_LOW format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_DBGLOW_PRINT0(format)
  #define USB_DBGLOW_PRINT1(format,a)
  #define USB_DBGLOW_PRINT2(format,a,b)
  #define USB_DBGLOW_PRINT3(format,a,b,c)
  #define USB_DBGLOW_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_DEBUG_MEDIUM
  #define USB_DBGMED_PRINT0(format)                 iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_DBG_MED format)
  #define USB_DBGMED_PRINT1(format,a)               iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_DBG_MED format,(U32)a)
  #define USB_DBGMED_PRINT2(format,a,b)             iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_DBG_MED format,(U32)a,(U32)b)
  #define USB_DBGMED_PRINT3(format,a,b,c)           iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_DBG_MED format,(U32)a,(U32)b,(U32)c)
  #define USB_DBGMED_PRINT4(format,a,b,c,d)         iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_DBG_MED format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_DBGMED_PRINT0(format)
  #define USB_DBGMED_PRINT1(format,a)
  #define USB_DBGMED_PRINT2(format,a,b)
  #define USB_DBGMED_PRINT3(format,a,b,c)
  #define USB_DBGMED_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_DEBUG_HIGH
  #define USB_DBGHGH_PRINT0(format)                 iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_DBG_HGH format)
  #define USB_DBGHGH_PRINT1(format,a)               iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_DBG_HGH format,(U32)a)
  #define USB_DBGHGH_PRINT2(format,a,b)             iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_DBG_HGH format,(U32)a,(U32)b)
  #define USB_DBGHGH_PRINT3(format,a,b,c)           iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_DBG_HGH format,(U32)a,(U32)b,(U32)c)
  #define USB_DBGHGH_PRINT4(format,a,b,c,d)         iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_DBG_HGH format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_DBGHGH_PRINT0(format)
  #define USB_DBGHGH_PRINT1(format,a)
  #define USB_DBGHGH_PRINT2(format,a,b)
  #define USB_DBGHGH_PRINT3(format,a,b,c)
  #define USB_DBGHGH_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_DEBUG_ERR
  #define USB_ERR_PRINT0(format)                    iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_ERR format)
  #define USB_ERR_PRINT1(format,a)                  iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_ERR format,(U32)a)
  #define USB_ERR_PRINT2(format,a,b)                iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_ERR format,(U32)a,(U32)b)
  #define USB_ERR_PRINT3(format,a,b,c)              iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_ERR format,(U32)a,(U32)b,(U32)c)
  #define USB_ERR_PRINT4(format,a,b,c,d)            iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_ERR format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_ERR_PRINT0(format)
  #define USB_ERR_PRINT1(format,a)
  #define USB_ERR_PRINT2(format,a,b)
  #define USB_ERR_PRINT3(format,a,b,c)
  #define USB_ERR_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_DEBUG_WRN
  #define USB_WRN_PRINT0(format)                    iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_WRN format)
  #define USB_WRN_PRINT1(format,a)                  iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_WRN format,(U32)a)
  #define USB_WRN_PRINT2(format,a,b)                iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_WRN format,(U32)a,(U32)b)
  #define USB_WRN_PRINT3(format,a,b,c)              iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_WRN format,(U32)a,(U32)b,(U32)c)
  #define USB_WRN_PRINT4(format,a,b,c,d)            iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_WRN format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_WRN_PRINT0(format)
  #define USB_WRN_PRINT1(format,a)
  #define USB_WRN_PRINT2(format,a,b)
  #define USB_WRN_PRINT3(format,a,b,c)
  #define USB_WRN_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_DEBUG_DATA
  #define USB_DATA_PRINT0(format)                   iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_DATA format)
  #define USB_DATA_PRINT1(format,a)                 iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_DATA format,(U32)a)
  #define USB_DATA_PRINT2(format,a,b)               iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_DATA format,(U32)a,(U32)b)
  #define USB_DATA_PRINT3(format,a,b,c)             iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_DATA format,(U32)a,(U32)b,(U32)c)
  #define USB_DATA_PRINT4(format,a,b,c,d)           iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_DATA format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_DATA_PRINT0(format)
  #define USB_DATA_PRINT1(format,a)
  #define USB_DATA_PRINT2(format,a,b)
  #define USB_DATA_PRINT3(format,a,b,c)
  #define USB_DATA_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_HOST
  #define USB_HOST_PRINT0(format)                   iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_HOST format)
  #define USB_HOST_PRINT1(format,a)                 iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_HOST format,(U32)a)
  #define USB_HOST_PRINT2(format,a,b)               iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_HOST format,(U32)a,(U32)b)
  #define USB_HOST_PRINT3(format,a,b,c)             iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_HOST format,(U32)a,(U32)b,(U32)c)
  #define USB_HOST_PRINT4(format,a,b,c,d)           iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_HOST format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_HOST_PRINT0(format)
  #define USB_HOST_PRINT1(format,a)
  #define USB_HOST_PRINT2(format,a,b)
  #define USB_HOST_PRINT3(format,a,b,c)
  #define USB_HOST_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_OTG
  #define USB_OTG_PRINT0(format)                    iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_OTG format)
  #define USB_OTG_PRINT1(format,a)                  iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_OTG format,(U32)a)
  #define USB_OTG_PRINT2(format,a,b)                iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_OTG format,(U32)a,(U32)b)
  #define USB_OTG_PRINT3(format,a,b,c)              iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_OTG format,(U32)a,(U32)b,(U32)c)
  #define USB_OTG_PRINT4(format,a,b,c,d)            iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_OTG format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_OTG_PRINT0(format)
  #define USB_OTG_PRINT1(format,a)
  #define USB_OTG_PRINT2(format,a,b)
  #define USB_OTG_PRINT3(format,a,b,c)
  #define USB_OTG_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_STACK
  #define USB_STACK_PRINT0(format)                  iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_STACK format)
  #define USB_STACK_PRINT1(format,a)                iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_STACK format,(U32)a)
  #define USB_STACK_PRINT2(format,a,b)              iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_STACK format,(U32)a,(U32)b)
  #define USB_STACK_PRINT3(format,a,b,c)            iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_STACK format,(U32)a,(U32)b,(U32)c)
  #define USB_STACK_PRINT4(format,a,b,c,d)          iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_STACK format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_STACK_PRINT0(format)
  #define USB_STACK_PRINT1(format,a)
  #define USB_STACK_PRINT2(format,a,b)
  #define USB_STACK_PRINT3(format,a,b,c)
  #define USB_STACK_PRINT4(format,a,b,c,d)
#endif

#if USB_PRINT_ENABLE_MASS_STOR_CLASS
  #define USB_MSC_PRINT0(format)                    iopiDiagConsolePrint0((char*)USB_LAYER_PREFIX_DEV_MSC format)
  #define USB_MSC_PRINT1(format,a)                  iopiDiagConsolePrint1((char*)USB_LAYER_PREFIX_DEV_MSC format,(U32)a)
  #define USB_MSC_PRINT2(format,a,b)                iopiDiagConsolePrint2((char*)USB_LAYER_PREFIX_DEV_MSC format,(U32)a,(U32)b)
  #define USB_MSC_PRINT3(format,a,b,c)              iopiDiagConsolePrint3((char*)USB_LAYER_PREFIX_DEV_MSC format,(U32)a,(U32)b,(U32)c)
  #define USB_MSC_PRINT4(format,a,b,c,d)            iopiDiagConsolePrint4((char*)USB_LAYER_PREFIX_DEV_MSC format,(U32)a,(U32)b,(U32)c,(U32)d)
#else
  #define USB_MSC_PRINT0(format)
  #define USB_MSC_PRINT1(format,a)
  #define USB_MSC_PRINT2(format,a,b)
  #define USB_MSC_PRINT3(format,a,b,c)
  #define USB_MSC_PRINT4(format,a,b,c,d)
#endif


#endif /* #if (IOC_USB_SUPPORT ) */

#endif /* #ifndef _USBLIBDEBUG_H_ */

