/***************************************************************************
 *                                                                         *
 *  Copyright 2009 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibStatus.h
 *          Title:  USB Library stack / core various status definitions.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains header definitions for the USB stack / core source.
 *
 * Note:
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIBSTATUS_H_
#define _USBLIBSTATUS_H_

#if (IOC_USB_SUPPORT)

/* ************************************************************************** */
/* USB Stack various status / error codes (0x40000 - 0x4FFFF)                 */
/* ************************************************************************** */

typedef U32 USB_STATUS;

/* Good Status Codes */
#define USB_STATUS_GENERIC_BASE                            (0x00040000)
#define USB_STATUS_SUCCESS                                 (USB_STATUS_GENERIC_BASE + 0x01)
#define USB_STATUS_FAILURE                                 (USB_STATUS_GENERIC_BASE + 0x02)
#define USB_STATUS_INVALID_PARAMETER                       (USB_STATUS_GENERIC_BASE + 0x03)
#define USB_STATUS_STACK_NOT_INITIALIZED                   (USB_STATUS_GENERIC_BASE + 0x04)

/* Error Codes - Generic */
#define USB_STATUS_ERRORS_GENERIC_BASE                     (0x00040100)
#define USB_STATUS_ERR_UKNOWN                              (USB_STATUS_ERRORS_GENERIC_BASE + 0x01)
#define USB_STATUS_ERR_FATAL                               (USB_STATUS_ERRORS_GENERIC_BASE + 0x02)
#define USB_STATUS_ERR_ULPI_INTERFACE_NOT_READY            (USB_STATUS_ERRORS_GENERIC_BASE + 0x03)
#define USB_STATUS_ERR_PROTOCOL                            (USB_STATUS_ERRORS_GENERIC_BASE + 0x04)
#define USB_STATUS_ERR_BUFFER                              (USB_STATUS_ERRORS_GENERIC_BASE + 0x05)
#define USB_STATUS_ERR_INVALID_PARAMETERS                  (USB_STATUS_ERRORS_GENERIC_BASE + 0x06)
#define USB_STATUS_ERR_INSUFFICIENT_DATA                   (USB_STATUS_ERRORS_GENERIC_BASE + 0x07)
#define USB_STATUS_ERR_CANNOT_ALLOCATE_MEMORY              (USB_STATUS_ERRORS_GENERIC_BASE + 0x08)
#define USB_STATUS_ERR_INVALID_PORT_NUMBER                 (USB_STATUS_ERRORS_GENERIC_BASE + 0x09)

#define USB_STATUS_ERR_CTRLR_SOFT_RESET_FAILED                      (USB_STATUS_ERRORS_GENERIC_BASE + 0x0A)
#define USB_STATUS_ERR_PORT_SUSPEND_FAILED                          (USB_STATUS_ERRORS_GENERIC_BASE + 0x0B)
#define USB_STATUS_ERR_PORT_RESUME_FAILED                           (USB_STATUS_ERRORS_GENERIC_BASE + 0x0C)
#define USB_STATUS_ERR_PORT_ENABLE_DISABLE_FAILED                   (USB_STATUS_ERRORS_GENERIC_BASE + 0x0D)
#define USB_STATUS_ERR_PORT_POWER_ON_OFF_FAILED                     (USB_STATUS_ERRORS_GENERIC_BASE + 0x0E)
#define USB_STATUS_ERR_PORT_RESET_FAILED                            (USB_STATUS_ERRORS_GENERIC_BASE + 0x0F)
#define USB_STATUS_ERR_ASYNC_SCHED_ENABLE_DISABLE_FAILED            (USB_STATUS_ERRORS_GENERIC_BASE + 0x10)
#define USB_STATUS_ERR_INT_ON_ASYNC_ADVANCE_ENABLE_DISABLE_FAILED   (USB_STATUS_ERRORS_GENERIC_BASE + 0x11)
#define USB_STATUS_ERR_MAX_TRANS_ELEMENT_LIMIT_HIT                  (USB_STATUS_ERRORS_GENERIC_BASE + 0x12)

/* Error Codes - Core */
#define USB_STATUS_ERRORS_BASE_CORE                        (0x00040200)
#define USB_STATUS_ERR_CORE_IDX_INVALID                    (USB_STATUS_ERRORS_BASE_CORE + 0x01)
#define USB_STATUS_ERR_CORE_BUSY                           (USB_STATUS_ERRORS_BASE_CORE + 0x02)

/* Error Codes - Register access */
#define USB_STATUS_ERRORS_BASE_REGS                        (0x00040300)
#define USB_STATUS_ERR_REGISTER_READ_FAILED                (USB_STATUS_ERRORS_BASE_REGS + 0x01)
#define USB_STATUS_ERR_REGISTER_WRITE_FAILED               (USB_STATUS_ERRORS_BASE_REGS + 0x02)
#define USB_STATUS_ERR_REGISTER_ACCESS_VIOLATION_RO        (USB_STATUS_ERRORS_BASE_REGS + 0x03)

/* Error Codes - Port access */
#define USB_STATUS_ERRORS_BASE_PORT_ACCESS                 (0x00040400)
#define USB_STATUS_ERR_PORT_NOT_ENABLED                    (USB_STATUS_ERRORS_BASE_PORT_ACCESS + 0x01)
#define USB_STATUS_ERR_PORT_ALREADY_ENABLED                (USB_STATUS_ERRORS_BASE_PORT_ACCESS + 0x02)
#define USB_STATUS_ERR_PORT_NOT_SUSPENDED                  (USB_STATUS_ERRORS_BASE_PORT_ACCESS + 0x03)
#define USB_STATUS_ERR_PORT_ALREADY_SUSPENDED              (USB_STATUS_ERRORS_BASE_PORT_ACCESS + 0x04)

/* Error Codes - End device (ED) */
#define USB_STATUS_ERRORS_BASE_ED                          (0x00040500)
#define USB_STATUS_ERR_END_DEVICE_NOT_PRESENT              (USB_STATUS_ERRORS_BASE_ED + 0x01)
#define USB_STATUS_ERR_END_DEVICE_CONFIGURED               (USB_STATUS_ERRORS_BASE_ED + 0x02)
#define USB_STATUS_ERR_IO_TIMEOUT                          (USB_STATUS_ERRORS_BASE_ED + 0x03)
#define USB_STATUS_ERR_END_DEVICE_BUSY                     (USB_STATUS_ERRORS_BASE_ED + 0x04)
#define USB_STATUS_ERR_END_DEVICE_UNPLUGGED                (USB_STATUS_ERRORS_BASE_ED + 0x05)
#define USB_STATUS_ERR_END_DEVICE_IN_LOW_POWER             (USB_STATUS_ERRORS_BASE_ED + 0x06)
#define USB_STATUS_ERR_END_DEVICE_IN_STANDBY               (USB_STATUS_ERRORS_BASE_ED + 0x07)
#define USB_STATUS_ERR_END_DEVICE_MEDIA_CHECK_FAILED       (USB_STATUS_ERRORS_BASE_ED + 0x08)
#define USB_STATUS_ERR_END_DEVICE_MEDIA_CORRUPT            (USB_STATUS_ERRORS_BASE_ED + 0x09)

/* Error Codes - End point (EP) */
#define USB_STATUS_ERRORS_BASE_EP                          (0x00040600)
#define USB_STATUS_ERR_END_POINT_CREATION_FAILURE          (USB_STATUS_ERRORS_BASE_EP + 0x01)
#define USB_STATUS_ERR_END_POINT_NOT_PRESENT               (USB_STATUS_ERRORS_BASE_EP + 0x02)
#define USB_STATUS_ERR_END_POINT_BUSY                      (USB_STATUS_ERRORS_BASE_EP + 0x03)
#define USB_STATUS_ERR_END_POINT_INCORRECT_XFR_MODE        (USB_STATUS_ERRORS_BASE_EP + 0x04)
#define USB_STATUS_ERR_END_POINT_DELETED                   (USB_STATUS_ERRORS_BASE_EP + 0x05)

/* Error Codes - Data transfer (DT) */
#define USB_STATUS_ERRORS_BASE_DT                          (0x00040700)
#define USB_STATUS_ERR_DT_INITIATION_FAILED                (USB_STATUS_ERRORS_BASE_DT + 0x01)
#define USB_STATUS_ERR_DT_WRITE_XFR_FAILED                 (USB_STATUS_ERRORS_BASE_DT + 0x02)
#define USB_STATUS_ERR_DT_READ_XFR_FAILED                  (USB_STATUS_ERRORS_BASE_DT + 0x03)
#define USB_STATUS_ERR_DT_CANNOT_READ_SOURCE_MEMORY        (USB_STATUS_ERRORS_BASE_DT + 0x04)
#define USB_STATUS_ERR_DT_CANNOT_WRITE_TARGET_MEMORY       (USB_STATUS_ERRORS_BASE_DT + 0x05)

/* Error Codes - Power management (PM) */
#define USB_STATUS_ERRORS_BASE_PM                          (0x00040800)
#define USB_STATUS_ERR_CORE_POWERED_OFF                    (USB_STATUS_ERRORS_BASE_PM + 0x01)
#define USB_STATUS_ERR_CORE_POWERED_INSUFFICIENT           (USB_STATUS_ERRORS_BASE_PM + 0x02)

/* Error Codes - Timers (Non-EHCI) */
#define USB_STATUS_ERRORS_BASE_NON_EHCI_TIMERS             (0x00040900)
#define USB_STATUS_ERR_TIMER_INVALID_NUMBER                (USB_STATUS_ERRORS_BASE_NON_EHCI_TIMERS + 0x01)
#define USB_STATUS_ERR_TIMER_ALREADY_RUNNING               (USB_STATUS_ERRORS_BASE_NON_EHCI_TIMERS + 0x02)
#define USB_STATUS_ERR_TIMER_ALREADY_STOPPED               (USB_STATUS_ERRORS_BASE_NON_EHCI_TIMERS + 0x03)

/* Error Codes - End Device Interface - Mass Storage Class */
#define USB_STATUS_ERRORS_BASE_MSC                         (0x00040A00)
#define USB_STATUS_ERR_MSC_FATAL_ERROR                     (USB_STATUS_ERRORS_BASE_MSC + 0x01)
#define USB_STATUS_ERR_MSC_CMD_PHASE                       (USB_STATUS_ERRORS_BASE_MSC + 0x02)
#define USB_STATUS_ERR_MSC_DATA_PHASE                      (USB_STATUS_ERRORS_BASE_MSC + 0x03)
#define USB_STATUS_ERR_MSC_STATUS_PHASE                    (USB_STATUS_ERRORS_BASE_MSC + 0x04)
#define USB_STATUS_ERR_MSC_PHASE                           (USB_STATUS_ERRORS_BASE_MSC + 0x05)
#define USB_STATUS_ERR_MSC_RETRY_CMD                       (USB_STATUS_ERRORS_BASE_MSC + 0x06)
#define USB_STATUS_ERR_MSC_CHECK_SUPPORT                   (USB_STATUS_ERRORS_BASE_MSC + 0x07)
#define USB_STATUS_ERR_MSC_CMD_ERROR                       (USB_STATUS_ERRORS_BASE_MSC + 0x08)
#define USB_STATUS_ERR_MSC_INIT_NOT_DONE                   (USB_STATUS_ERRORS_BASE_MSC + 0x09)

#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIBSTATUS_H_ */

