/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usb.h
 *          Title:  USB definitions from the USB 2.0 specification.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains header definitions from the USB 2.0 standards
 * specification.
 *
 * Note:
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USB_H_
#define _USB_H_

/* ************************************************************************** */
/* Various definitions / constant from the USB 2.0 standards specification    */
/* ************************************************************************** */

/* ************************************************************************** */
/* USB standard defined visible device states                                 */
/* ************************************************************************** */
#define USB_DEV_STATE_ATTACHED                          0x00
#define USB_DEV_STATE_POWERED                           0x01
#define USB_DEV_STATE_DEFAULT                           0x02
#define USB_DEV_STATE_ADDRESS                           0x03
#define USB_DEV_STATE_CONFIGURED                        0x04
#define USB_DEV_STATE_SUSPENDED                         0x05

/* ************************************************************************** */
/* USB standard device request codes                                          */
/* ************************************************************************** */
#define USB_DEV_REQ_CLEAR_DEV_FEATURE                   0x00
#define USB_DEV_REQ_CLEAR_INTF_FEATURE                  0x01
#define USB_DEV_REQ_CLEAR_ENDPT_FEATURE                 0x02

#define USB_DEV_REQ_GET_CONFIG                          0x80
#define USB_DEV_REQ_GET_DESC                            0x80
#define USB_DEV_REQ_GET_INTERFACE                       0x81
#define USB_DEV_REQ_GET_DEV_STATUS                      0x80
#define USB_DEV_REQ_GET_INTF_STATUS                     0x81
#define USB_DEV_REQ_GET_ENDPT_STATUS                    0x82

#define USB_DEV_REQ_SET_ADDRESS                         0x00
#define USB_DEV_REQ_SET_CONFIG                          0x00
#define USB_DEV_REQ_SET_DESC                            0x00

#define USB_DEV_REQ_SET_DEV_FEATURE                     0x00
#define USB_DEV_REQ_SET_INTF_FEATURE                    0x01
#define USB_DEV_REQ_SET_ENDPT_FEATURE                   0x02

#define USB_DEV_REQ_SET_INTERFACE                       0x01

#define USB_DEV_REQ_SYNCH_FRAME                         0x82

/* ************************************************************************** */
/* USB standard request codes                                                 */
/* ************************************************************************** */
#define USB_REQ_CODE_GET_STATUS                         0x00
#define USB_REQ_CODE_CLEAR_FEATURE                      0x01
#define USB_REQ_CODE_RESERVED_2                         0x02
#define USB_REQ_CODE_SET_FEATURE                        0x03
#define USB_REQ_CODE_RESERVED_4                         0x02
#define USB_REQ_CODE_SET_ADDRESS                        0x05
#define USB_REQ_CODE_GET_DESCRIPTOR                     0x06
#define USB_REQ_CODE_SET_DESCRIPTOR                     0x07
#define USB_REQ_CODE_GET_CONFIG                         0x08
#define USB_REQ_CODE_SET_CONFIG                         0x09
#define USB_REQ_CODE_GET_INTERFACE                      0x0A
#define USB_REQ_CODE_SET_INTERFACE                      0x0B
#define USB_REQ_CODE_SYNCH_FRAME                        0x0C

/* ************************************************************************** */
/* USB standard descriptor types                                              */
/* ************************************************************************** */
#define USB_DESC_TYPE_DEVICE                            0x01
#define USB_DESC_TYPE_CONFIG                            0x02
#define USB_DESC_TYPE_STRING                            0x03
#define USB_DESC_TYPE_INTF                              0x04
#define USB_DESC_TYPE_ENDPOINT                          0x05
#define USB_DESC_TYPE_DEVICE_QUALIFIER                  0x06
#define USB_DESC_TYPE_OTH_SPEED_CONFIG                  0x07
#define USB_DESC_TYPE_INTF_POWER                        0x08

/* ************************************************************************** */
/* Usb Feature Selectors                                                      */
/* ************************************************************************** */
#define USB_FEATURE_SELECTOR_ENDPOINT_HALT              0x00
#define USB_FEATURE_SELECTOR_DEVICE_REMOTE_WAKEUP       0x01
#define USB_FEATURE_SELECTOR_TEST_MODE                  0x02

/* ************************************************************************** */
/* Usb Descriptor Lengths                                                     */
/* ************************************************************************** */
#define USB_DEVICE_DESCRIPTOR_LENGTH                    18
#define USB_CONFIG_DESCRIPTOR_LENGTH                    9

/* ************************************************************************** */
/* USB defined protocol packets                                               */
/* ************************************************************************** */

/* ************************************************************************** */
/* USB Standard defined - Request packet                                      */
/* ************************************************************************** */
#pragma pack(push)
#pragma pack(1)
typedef struct _USB_PKT_REQUEST
{
    U8          RequestType;
    U8          Request;
    U16         Value;

    U16         Index;
    U16         Length;

} USB_PKT_REQUEST, *PTR_USB_PKT_REQUEST;


/* ************************************************************************** */
/* USB Standard defined - Device descriptor                                   */
/* ************************************************************************** */
typedef struct _USB_DESC_DEVICE
{
    U8          Length;
    U8          DescType;
    U16         BcdUSB;

    U8          DeviceClass;            /* As defined by USB-IF */
    U8          DeviceSubClass;         /* As defined by USB-IF */
    U8          DeviceProtocol;
    U8          MaxPacketSize0;         /* Only - 8, 16, 32 or 64 are valid */

    U16         VendorId;
    U16         ProductId;

    U16         BcdDevice;

    U8          ManufacturerIndex;
    U8          ProductIndex;
    U8          SerialNumberIndex;
    U8          NumConfigs;

} USB_DESC_DEVICE, *PTR_USB_DESC_DEVICE;


/* ************************************************************************** */
/* USB Standard defined - Device qualifier                                    */
/* ************************************************************************** */
typedef struct _USB_DEVICE_QUALIFIER
{
    U8          Length;
    U8          DescType;
    U16         BcdUSB;

    U8          DeviceClass;
    U8          DeviceSubClass;
    U8          DeviceProtocol;
    U8          MaxPacketSize0;

    U8          NumConfigs;
    U8          Reserved;

} USB_DEV_QUALIFIER, *PTR_USB_DEV_QUALIFIER;

/* ************************************************************************** */
/* USB Standard defined - Configuration descriptor                            */
/* ************************************************************************** */
typedef struct _USB_DESC_CONFIG
{
    U8          Length;
    U8          DescType;
    U16         TotalLength;

    U8          NumInterfaces;
    U8          ConfigValue;
    U8          Config;
    U8          Attributes;

    U8          MaxPower;

} USB_DESC_CONFIG, *PTR_USB_DESC_CONFIG;

/* ************************************************************************** */
/* USB Standard defined - Other speed configurator descriptor                 */
/* ************************************************************************** */
typedef struct _USB_DESC_OTH_SPEED_CONFIG
{
    U8          Length;
    U8          DescType;
    U16         TotalLength;

    U8          NumInterfaces;
    U8          ConfigValue;
    U8          ConfigIndex;
    U8          Attributes;

    U8          MaxPower;

} USB_DESC_OTH_SPEED_CONFIG, *PTR_USB_DESC_OTH_SPEED_CONFIG;

/* ************************************************************************** */
/* USB Standard defined - Interface descriptor                                */
/* ************************************************************************** */
typedef struct _USB_DESC_INTERFACE
{
    U8          Length;
    U8          DescType;
    U8          InterfaceNum;
    U8          AltSetting;

    U8          NumEndPoints;
    U8          InterfaceClass;
    U8          InterfaceSubClass;
    U8          InterfaceProtocol;

    U8          InterfaceIndex;

} USB_DESC_INTERFACE, *PTR_USB_DESC_INTERFACE;

/* ************************************************************************** */
/* USB Standard defined - Endpoint descriptor                                 */
/* ************************************************************************** */
typedef struct _USB_DESC_ENDPOINT
{
    U8          Length;
    U8          DescType;
    U8          EndPointAddress;
    U8          Attributes;

    U16         MaxPacketSize;
    U8          Interval;

} USB_DESC_ENDPOINT, *PTR_USB_DESC_ENDPOINT;
//#pragma pack
#pragma pack(pop)

#endif /* #ifndef _USB_H_ */

