/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLib.h
 *          Title:  USB Library stack / core source header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains header definitions for the USB stack / core source.
 *
 * Note:
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIB_H_
#define _USBLIB_H_

#if (IOC_USB_SUPPORT)

/* ************************************************************************** */
/* USB Stack - Definitions                                                    */
/* ************************************************************************** */

/* USB End Device Type */
typedef enum _USB_ED_TYPE
{
    USB_ED_TYPE_INVALID,    /* Invalid device / unknown device / init value */
    USB_ED_TYPE_MSC,        /* Device type - Mass Storage Class */
    USB_ED_TYPE_OTG,        /* Device type - On-the-Go */
    USB_ED_TYPE_HID         /* Device type - Human Interface Device */

    /* Add valid End Device types here */

} USB_ED_TYPE, *PTR_USB_ED_TYPE;


/* USB Connection Speeds */
typedef enum _USB_PORT_SPEED
{
    USB_PORT_SPEED_INVALID,     /* Invalid connection mode / init value */
    USB_PORT_SPEED_LOW,         /* Low speed connection */
    USB_PORT_SPEED_FULL,        /* Full speed connection */
    USB_PORT_SPEED_HIGH         /* High speed connection: 480 MBits / sec */

    /* Add mode connection speed enumerations here */

} USB_PORT_SPEED, *PTR_USB_PORT_SPEED;


/* USB End Device States */
typedef enum _USB_ED_STATE
{
    USB_ED_STATE_UNKNOWN,       /* Unknown end device state */
    USB_ED_STATE_ATTACHED,      /* Device attached to hub, but not powered */
    USB_ED_STATE_POWERED,       /* Attached to bus, but not drawing more than 100ma of power, awaiting host to initialize */
    USB_ED_STATE_DEFAULT,       /* Device in just-after-reset condition */
    USB_ED_STATE_ADDRESSED,     /* Device has received a Set Address notification from host, responds to only NEW address */
    USB_ED_STATE_CONFIGURED,    /* Device has received a Set Configuration request, now in fully functional state */
    USB_ED_STATE_SUSPENDED      /* Device was idle for more than 3ms, now draws 500ua of current */

} USB_ED_STATE, *PTR_USB_ED_STATE;

/* USB Power Modes */
typedef enum _USB_POWER_MODE
{
    USB_POWER_MODE_OFF,
    USB_POWER_MODE_ON

} USB_POWER_MODE, *PTR_USB_POWER_MODE;

/* USB End Device Capabilities */
typedef struct _USB_DEVICE_CAPABILITIES
{
    U8          DeviceClass;            /* As defined by USB-IF */
    U8          DeviceSubClass;         /* As defined by USB-IF */
    U8          DeviceProtocol;
    U8          MaxPacketSize0;         /* Only - 8, 16, 32 or 64 are valid */

    U16         VendorId;
    U16         ProductId;

    U8          ManufacturerIndex;
    U8          ProductIndex;
    U8          SerialNumberIndex;
    U8          NumConfigs;

} USB_DEVICE_CAPABILITIES, *PTR_USB_DEVICE_CAPABILITIES;


typedef struct _USB_SCSI_STATUS
{
    U8 SenseKey;
    U8 ASC;
    U8 ASCQ;
} USB_SCSI_STATUS;
/* ************************************************************************** */
/* USB Stack - API - Callback function signature                              */
/* ************************************************************************** */
typedef void (*CompletionHandler)(U32 UsbPort, USB_STATUS Status, USB_SCSI_STATUS ScsiStatus, U32 CmdHandle);

/* ************************************************************************** */
/* USB Stack - Bus/Port Control                                               */
/* ************************************************************************** */
USB_STATUS usbiLibGetNumPorts( PU32 PtrUsbNumPorts );
USB_STATUS usbiLibPortSoftReset( U32 UsbPort );
USB_STATUS usbiLibPortHardReset( U32 UsbPort );
USB_STATUS usbiLibGetPortSpeed( U32 UsbPort, PU32 PtrConnectionSpeed );

/* ************************************************************************** */
/* USB Stack - API - Device Interfacing (host/device/otg)                     */
/* ************************************************************************** */
USB_STATUS usbiLibCheckEndDevice( U32 UsbPort );
USB_STATUS usbiLibEndDeviceRemove( U32 UsbPort );
USB_STATUS usbiLibEndDeviceForceRemove( U32 UsbPort );
USB_STATUS usbiLibEndDeviceSafeRemove( U32 UsbPort );

/* ************************************************************************** */
/* USB Stack - API - Data endpoint / connection                               */
/* ************************************************************************** */

/* ************************************************************************** */
/* USB Stack - API - IO / Transfers                                           */
/* ************************************************************************** */

/* ************************************************************************** */
/* USB Stack - API - Power management                                         */
/* ************************************************************************** */
USB_STATUS usbiLibSuspendPort( U32 UsbPort );
USB_STATUS usbiLibResumePort( U32 UsbPort );

#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIB_H_ */

