/*******************************************************************************

NAME            ddr3Init.h
VERSION         %version: 3 %
UPDATE DATE     %date_modified: Thu Oct 17 11:27:49 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ddr3Init.h
 *          Title:  Denali Memory Controller Initialization Definitions
 *     Programmer:  Nilesh Wilanker
 *     Modified  :  Jignesh Moradiya
 *  Creation Date:  July 4, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  07/04/11  NW    Initial version.
 *  10/06/11  NM    Porting this file for Invader.
 *  05/20/12  JM    Alignment changes.
 *
 * Description
 * ------------
 * This software module will prepare the Denali memory controller for
 * normal operation.
 *
 *-------------------------------------------------------------------------
 */

#include "basetype.h"

#ifndef __DDR3INIT_H
#define __DDR3INIT_H

#define PPC476_TIMER_TICKS_PER_10USEC   10   /* 1MHZ for real chip */

#define PPC476_ADD_USEC_TO_TICKS(AdditionalUSEC)    \
      (PPC476_GET_TICKS()+ ((AdditionalUSEC)*10))/PPC476_TIMER_TICKS_PER_10USEC

#define L3_MEM0_REGION_REG          0x30
#define L3_MEM1_REGION_REG          0x34
#define L3_MEM2_REGION_REG          0x38
#define L3_MEM3_REGION_REG          0x3c

#define L3_MEM_REGION_INVALID       0x00000000
#define L3_MEM_REGION_SIZE_64MB     0x00000001
#define L3_MEM_REGION_SIZE_128MB    0x00000002
#define L3_MEM_REGION_SIZE_256MB    0x00000003
#define L3_MEM_REGION_SIZE_512MB    0x00000004
#define L3_MEM_REGION_SIZE_1GB      0x00000005
#define L3_MEM_REGION_SIZE_2GB      0x00000006
#define L3_MEM_REGION_SIZE_4GB      0x00000007
#define L3_MEM_REGION_SIZE_8GB      0x00000008
#define L3_MEM_REGION_SIZE_16GB     0x00000009
#define L3_MEM_REGION_SIZE_32GB     0x0000000A
#define L3_MEM_REGION_SIZE_64GB     0x0000000B
#define L3_MEM_REGION_SIZE_128GB    0x0000000C

#define L3_MEM_REGION_STARTADDRESS_DDR3_NORMAL       0x00000000
#define L3_MEM_REGION_STARTADDRESS_DDR3_ALIAS        0x10000000



#define L3_CONFIG_REG               0x14

#define L3_INDEX_REG                0x11000
#define L3_DATA_REG                 0x11004

#define CONFIG_DDR3_PAT_BUF         0xC0200000
#define CONFIG_DDR3_CMP_BUF         0xC0280000
#define CONFIG_DDR3_TRAINING_DATA   0xC0300000
#define CONFIG_DDR3_CALIB_MEM       0xC0320000

#define CONFIG_SYS_ISRAM_BASE3      0xC0300000

#define CONFIG_DDR3_DCR_BASE        0x20000

#define DDR3_CACHE_INHIBITED_MAP    0x04000000

#define DDR3_TRAINING_L3_MEM_REGION_SIZE    (64*1024*1024)
#define DDR3_TRAINING_TLB_MEM_REGION_SIZE   (1*1024*1024)

/*
 * With 64 MB L3 cache region size, we have:
 *     0 MB - 64 MB L3 cached,   bank 1, training area at 0 MB.
 *     64MB - 128MB L3 uncached, bank 1, training area at 64-65 MB.
 *     Second to last 64 MB region is L3 cached, with training area as last 1MB in that region.
 *     Last 64 MB region is L3 uncached, with training area as first 1MB in that region.
 *
 *  Any changes to these constants also need to be propgatated to lsisas3108_common.h
 *  in the uboot compilation module.
 */
#define DDR3_PHY_ADDR_ALIAS_BASE                0x400000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_NON_CACHED     0x404000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_1GB            0x47bfff000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_1GB_NON_CACHED 0x47c000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_2GB            0x4fbfff000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_2GB_NON_CACHED 0x4fc000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_4GB            0x5fbfff000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_4GB_NON_CACHED 0x5fc000000ULL


#define DDR3_VIRT_ADDR_1ST_RANK_CACHED            0x00000000
#define DDR3_VIRT_ADDR_1ST_RANK_NON_CACHED        0x04000000
#define DDR3_VIRT_ADDR_1GB_2ND_RANK_CACHED        0x20000000
#define DDR3_VIRT_ADDR_1GB_2ND_RANK_NON_CACHED    0x30000000
#define DDR3_VIRT_ADDR_2GB_2ND_RANK_CACHED        0x40000000
#define DDR3_VIRT_ADDR_2GB_2ND_RANK_NON_CACHED    0x50000000
#define DDR3_VIRT_ADDR_4GB_2ND_RANK_CACHED        0x60000000
#define DDR3_VIRT_ADDR_4GB_2ND_RANK_NON_CACHED    0x70000000


void iopiDelayUSec (U32 USec);
U32  iopDDRInit    (int argc, char * argv[]);
U32  iopDDRSize    (int argc, char * argv[]);
U32  ddrPostUseDma(int argc, char * argv[]);
#endif /* __DDR3INIT_H */
