/*******************************************************************************

NAME            rootComplexWrapper.c
VERSION         %version: 2 %
UPDATE DATE     %date_modified: Thu Oct 17 11:34:16 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  rootComplexWrapper.c
 *          Title:  Root Comlex Wrapper function file
 *     Programmer:  Ankur Mehrotra
 *  Creation Date:  Sep 6, 2011
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  09/06/11  AM   Initial version.
 *
 * Description
 * ------------
 * This contains wrapper function to call other root complex functions through
 * standalone u-boot library.
 *
 *-------------------------------------------------------------------------
 */
#include "basetype.h"
#include "ubootLib.h"
#include "rootComplex.h"
#include "iopCommon.h"

extern int pci_rc_init_board(void);

/* Warning: need to change this in case of any modification to root complex lib */
/* Number of arguments for each root complex api function */
int gRCFunctionNumArgs[RC_NUM_FUNCTIONS] = 
                {0, 0, 1, 1, 1, 1, 0, 5, 4, 5, 5, 5, 5, 5, 5, 2, 2, 2, 2, 1, 1, 1, 1, 0 };

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: rootComplexWrapper ()
* SUMMARY: Root Complex wrapper function.
* SCOPE: global.
*
* DESCRIPTION:
* This function is the interface fuction for U-Boot Standalone library for calling
* root complex library api functions.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* argc: number of variable arguments
* argv: pointer to variable arguments
*
* RETURNS:
*   -1      : if the api does not exist or wrong number of arguments passed
*   result  : return value matches the return value of the specific library api
*    0      : if api has no return value (void)
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
* argv[0]: Function id
* argv[1]...argv[n-1]: Arguments needed by library functions.
----------------------------------------------------------------------------*/
int rootComplexWrapper(int argc, char * argv[])
{
    int result;
    int funcId = *((int *)argv[0]);

    if( (funcId < 1 || funcId > RC_NUM_FUNCTIONS) ||
        ((argc - 1) < gRCFunctionNumArgs[funcId - 1]) )
    {
        result = -1;
    }
    else
    {
        switch( funcId )
        {
            case RC_IS_ENABLED:
            {
                result = pcieRcIsRootComplexEnabled();
                break;
            }
            case RC_IS_LINK_UP:
            {
                result = pcieRcIsLinkUp();
                break;
            }
            case RC_PERFORM_LINK_WIDTH_TRNG:
            {
                U8 width = *((U8 *)argv[1]);

                result = pcieRcPerformLinkWidthTraining(width);
                break;
            }
            case RC_PERFORM_LINK_SPEED_TRNG:
            {
                U8 speed = *((U8 *)argv[1]);

                result = pcieRcPerformLinkSpeedTraining(speed);
                break;
            }
            case RC_CONFIGURE_POM:
            {
                PCIE_RC_CONFIG_POM_PARAMS *ptrPOMConfigParams;
                ptrPOMConfigParams = (PCIE_RC_CONFIG_POM_PARAMS *)argv[1];

                result = pcieRcConfigurePOM(ptrPOMConfigParams);
                break;
            }
            case RC_CONFIGURE_PIM:
            {
                PCIE_RC_CONFIG_PIM_PARAMS *ptrPIMConfigParams;
                ptrPIMConfigParams = (PCIE_RC_CONFIG_PIM_PARAMS *)argv[1];

                result = pcieRcConfigurePIM(ptrPIMConfigParams);
                break;
            }
            case RC_ENABLE_MSIX:
            {
                pcieRcEnableMsixCapability();
                result = 0;
                break;
            }
            case RC_CONFIGURE_MSIX:
            {
                U8                      bus;
                U8                      device;
                U8                      func;
                U8                      vectorMSIX;
                PCIE_RC_MSIX_MESSAGE    *ptrMsixMessage;

                bus            = *((U8 *)argv[1]);
                device         = *((U8 *)argv[2]);
                func           = *((U8 *)argv[3]);
                vectorMSIX     = *((U8 *)argv[4]);
                ptrMsixMessage = (PCIE_RC_MSIX_MESSAGE *)argv[5];

                result = pcieRcConfigureMSIX(bus, device, func, vectorMSIX, ptrMsixMessage);
            }
            case RC_FIND_CAPABILITY:
            {
                U8  bus;
                U8  device;
                U8  func;
                U32 capabilityId;

                bus          = *((U8 *)argv[1]);
                device       = *((U8 *)argv[2]);
                func         = *((U8 *)argv[3]);
                capabilityId = *((U32 *)argv[4]);

                result = pcieRcFindCapability(bus, device, func, capabilityId);
                break;
            }
            case RC_CFG_READ_DWORD:
            {
                U8  bus;
                U8  device;
                U8  func;
                U32 address;
                U32 *data;

                bus     = *((U8 *)argv[1]);
                device  = *((U8 *)argv[2]);
                func    = *((U8 *)argv[3]);
                address = *((U32 *)argv[4]);
                data    = (U32 *)argv[5];

                result = pcieRcCfgReadDword(bus, device, func, address, data);
                break;
            }
            case RC_CFG_READ_WORD:
            {
                U8  bus;
                U8  device;
                U8  func;
                U32 address;
                U16 *data;

                bus     = *((U8 *)argv[1]);
                device  = *((U8 *)argv[2]);
                func    = *((U8 *)argv[3]);
                address = *((U32 *)argv[4]);
                data    = (U16 *)argv[5];

                result = pcieRcCfgReadWord(bus, device, func, address, data);
                break;
            }
            case RC_CFG_READ_BYTE:
            {
                U8  bus;
                U8  device;
                U8  func;
                U32 address;
                U8  *data;

                bus     = *((U8 *)argv[1]);
                device  = *((U8 *)argv[2]);
                func    = *((U8 *)argv[3]);
                address = *((U32 *)argv[4]);
                data    = (U8 *)argv[5];

                result = pcieRcCfgReadByte(bus, device, func, address, data);
                break;
            }
            case RC_CFG_WRITE_DWORD:
            {
                U8  bus;
                U8  device;
                U8  func;
                U32 address;
                U32 data;

                bus     = *((U8 *)argv[1]);
                device  = *((U8 *)argv[2]);
                func    = *((U8 *)argv[3]);
                address = *((U32 *)argv[4]);
                data    = *((U32 *)argv[5]);

                result = pcieRcCfgWriteDword(bus, device, func, address, data);
                break;
            }
            case RC_CFG_WRITE_WORD:
            {
                U8  bus;
                U8  device;
                U8  func;
                U32 address;
                U16 data;

                bus     = *((U8 *)argv[1]);
                device  = *((U8 *)argv[2]);
                func    = *((U8 *)argv[3]);
                address = *((U32 *)argv[4]);
                data    = *((U16 *)argv[5]);

                result = pcieRcCfgWriteWord(bus, device, func, address, data);
                break;
            }
            case RC_CFG_WRITE_BYTE:
            {
                U8  bus;
                U8  device;
                U8  func;
                U32 address;
                U8  data;

                bus     = *((U8 *)argv[1]);
                device  = *((U8 *)argv[2]);
                func    = *((U8 *)argv[3]);
                address = *((U32 *)argv[4]);
                data    = *((U8 *)argv[5]);

                result = pcieRcCfgWriteByte(bus, device, func, address, data);
                break;
            }
            case RC_MEM_READ_DWORD:
            {
                U32 address;
                U32 *data;

                address = *((U32 *)argv[1]);
                data    = (U32 *)argv[2];

                result = pcieRcMemReadDword(address, data);
                break;
            }
            case RC_MEM_WRITE_DWORD:
            {
                U32 address;
                U32 data;

                address = *((U32 *)argv[1]);
                data    = *((U32 *)argv[2]);

                result = pcieRcMemWriteDword(address, data);
                break;
            }
            case RC_READ_BACKDOOR:
            {
                U32 address;
                U32 *data;

                address = *((U32 *)argv[1]);
                data = (U32 *)argv[2];

                pcieRcReadBackDoor(address,data);
                result = 0;
                break;
            }
            case RC_WRITE_BACKDOOR:
            {
                U32 address;
                U32 data;

                address = *((U32 *)argv[1]);
                data    = *((U32 *)argv[2]);
                
                pcieRcWriteBackDoor(address, data);
                result = 0;
                break;
            }
            case RC_GET_PHY_ADDR:
            {
                unsigned long long int *physAddr;
                physAddr = (unsigned long long int *)argv[1];
                pcieRcGetPhyAddr(physAddr);
                result = 0;
                break;
            }
            case RC_GET_VIRT_ADDR:
            {
                unsigned int *virtAddr;
                virtAddr = (unsigned int *)argv[1];
                pcieRcGetVirtAddr(virtAddr);
                result = 0;
                break;
            }
            case RC_PCI_BOARD_INIT:
            {
                result = pci_rc_init_board();
                break;
            }
            default:
            {
                result = -1;
                break;
            }
        }
    }
    
    return result;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: udelay ()
* SUMMARY: delay function used in root complex library.
* SCOPE: global.
*
* DESCRIPTION:
* This function is used by root complex library api functions to perform a delay
* of given number of micro seconds.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
*   USec: number of micro seconds to delay
*
* RETURNS:
*   void
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
----------------------------------------------------------------------------*/
void udelay(U32 USec)
{
    iopinitDelayUSec(USec);
}
